/*
** This code is under development.  Status is as follows:

Working, subject to more testing: char, line, dot, memory load modes
both ascii and classic.  Connection failure handling.  Print screen.
Save screen.  flood fill (-paint-) command.  -font- command and text
display when in font mode.  Full screen mode.  Print preview.
Copy text.  2x mode, -stretch- mode.  Scrollbars.  Save/restore window.
Scrolling in dumb terminal mode.  GSW mode.  Touch in zoomed and stretch
modes.  Full screen and position save/restore for multi-screen systems.
Paste printout.  Copy and paste of unicode text, including handling of
backspaced accents and backspaced compound characters.

New features: unicode copy/paste.

Not working: <none>

Not tested: <none>

Not coded yet: <none>

*/


////////////////////////////////////////////////////////////////////////////
// Name:        pterm_wx.cpp
// Purpose:     pterm interface to wxWindows 
// Authors:     Paul Koning, Joe Stanton
// Modified by: 
// Created:     03/26/2005
// Copyright:   (c) Paul Koning, Joe Stanton
// Licence:     see pterm-license.txt
/////////////////////////////////////////////////////////////////////////////

/*
The key of the Pterm V5 design is that it just uses a 512x512 bitmap with
raw pixel access to construct most of the screen content, then it displays
that bitmap on the window.  Similarly, it displays that bitmap on the
printout, or processes it for save-screen, etc.  Display transformations
like stretch and 2x scale are done when that bitmap is displayed, not 
by making the bitmap itself different.  
*/

// ============================================================================
// declarations
// ============================================================================

#define _CRT_SECURE_NO_WARNINGS 1  // for MSVC to not be such a pain

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#include <string.h>
#include <wchar.h>

// For compilers that support precompilation, includes <wx/wx.h>.
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifdef _WIN32
#include <wx/setup.h>
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/button.h>
#include <wx/clipbrd.h>
#include <wx/colordlg.h>
#include <wx/config.h>
#include <wx/dir.h>
#include <wx/image.h>
#include <wx/file.h>
#include <wx/filename.h>
#include <wx/filefn.h>
#include <wx/metafile.h>
#include <wx/notebook.h>
#include <wx/print.h>
#include <wx/printdlg.h>
#include <wx/sound.h>
#include <wx/textfile.h>
#include <wx/utils.h>
#include <wx/rawbmp.h>
#include <wx/uri.h>
#include <wx/display.h>
#include <wx/validate.h>
#include <wx/valnum.h>

extern "C"
{
#if defined (_WIN32)
#include <winsock.h>
#include <process.h>
#else
#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#endif
#include <stdlib.h>
#include <time.h>

#include <sndfile.h>

#include "const.h"
#include "types.h"
#include "proto.h"
#include "ptermversion.h"
#include "8080a.h"
#include "8080avar.h"
#include "ppt.h"
#include "touch-beep.h"

#if wxUSE_LIBGNOMEPRINT
#include "wx/html/forcelnk.h"
//FORCE_LINK (gnome_print)
#endif

#include "pterm.h"

#if defined (__WXGTK__)

// Attempting to include the gtk.h file yields infinite compile errors, so
// instead we just declare the two functions we need, leaving any 
// issues of data structures unstated.

struct GtkSettings;
    
extern void gtk_settings_set_string_property (GtkSettings *, const char *,
                                              const char *, const char *);
extern GtkSettings * gtk_settings_get_default (void);
#endif
}
    
// ----------------------------------------------------------------------------
// resources
// ----------------------------------------------------------------------------

// the application icon (under Windows and OS/2 it is in resources)
#if defined (__WXGTK__) || defined (__WXMOTIF__) || defined (__WXMAC__) || defined (__WXMGL__) || defined (__WXX11__)
#include "pterm_32.xpm"
#endif

static const u32 keyboardhelp[] = {
#include "ptermkeys.h"
};

// On Mac, Arial is not a standard font, so you get warnings about
// partial matches.  Helvetica is, though (and besides, that's what
// Arial really is) so use that name on Mac.
#if defined (__WXMAC__)
#define SSFONT "Helvetica"
#else
#define SSFONT "Arial"
#endif

// ----------------------------------------------------------------------------
// global variables
// ----------------------------------------------------------------------------

bool emulationActive = true;

// Global print data, to remember settings during the session
wxPrintData *g_printData;

// Global page setup data
wxPageSetupDialogData* g_pageSetupData;

wxSound *g_beep;

// ----------------------------------------------------------------------------
// local variables
// ----------------------------------------------------------------------------

class PtermApp;
static PtermApp *ptermApp;

class Trace
{
public:
    Trace ();
    void Open (const char *fn = NULL);
    void Close (void);
    // Note that the "format" attribute gives the argument index counting
    // the implied "this" as argument #1.
#ifdef _MSC_VER
    void Log (const char *fmt, ...);
#else
    void Log (const char *fmt, ...) __attribute__ ((format (printf, 2, 3)));
#endif
    void Log (const wxString &s);
    bool Active (void) const
    {
        return (fd != NULL);
    }
    
private:
    wxMessageOutputStderr *tp;
    FILE *fd;
};

Trace::Trace ()
{
    tp = NULL;
    fd = NULL;
}

void Trace::Open (const char *fn)
{
    Close ();
    if (fn == NULL || fn[0] == '\0')
    {
        fd = stdout;
    }
    else
    {
        fd = fopen (fn, "w");
        if (fd == NULL)
        {
            fprintf (stderr, "Failure opening trace file %s\n", fn);
            return;
        }
    }

    tp = new wxMessageOutputStderr (fd);
    if (tp == NULL)
    {
        fprintf (stderr, "Failure opening trace message object\n");
        fclose (fd);
        fd = NULL;
    }
}

void Trace::Close (void)
{
    if (fd != NULL)
    {
        delete tp;
        tp = NULL;
        if (fd != stdout)
        {
            fclose (fd);
        }
        fd = NULL;
    }
}

void Trace::Log (const char *fmt, ...)
{
    va_list v;
    wxString msg;
    
    if (Active ())
    {
        va_start (v, fmt);
        msg.PrintfV (fmt, v);
        va_end (v);
        Log (msg);
    }
}

void Trace::Log (const wxString &s)
{
    char tbuf[10];
    wxString hdr;

    if (Active ())
    {
#ifdef _WIN32
        SYSTEMTIME tv;
	
        GetLocalTime (&tv);
        GetTimeFormatA (LOCALE_SYSTEM_DEFAULT, 0, &tv, 
                        "'HH':'mm':'ss'.", &tbuf[0], 10);
        hdr.Printf ("%s.%03d: ", tbuf, tv.wMilliseconds);
#else
        struct timeval tv;

        gettimeofday (&tv, NULL);
        strftime (tbuf, 10, "%T", localtime (&tv.tv_sec));
        hdr.Printf ("%s.%03ld: ", tbuf, (long) tv.tv_usec / 1000);
#endif
        hdr.Append (s);
        hdr.Append ("\n");
        tp->Output (hdr);
    }
}

Trace traceF;
#define tracex traceF.Log

#ifdef DEBUG
Trace debugF;
#define debug debugF.Log
#else
#define debug(x,...) /* Nothing */
#endif

// Keycode translation tables.

// Several tables are generated by the composed.py script.  They are
// ASCII keycodes to PLATO keycodes, pasted ASCII characters to PLATO
// keycodes, and non-ASCII Unicode values to PLATO keycodes.  The first
// two are 128 entry arrays, the third is a list of Unicode/PLATO code
// pairs.
// In each case, the PLATO keycode is given using the KEY macro, which
// encodes 4 keycode values into a 32 bit integer (simply as those four
// values, in little endian order in each of the 4 bytes).  "None" is
// a macro that translates to all ones, and represents an unused entry.
// In this way, the keycode lookups can produce from 0 to 4 PLATO keycodes
// for each entry.
// The Unicode entries are built by the script to represent either combined
// (precomposed) or separated encodings.  We just keep entries for both,
// rather than bothering with the rather heavy solution of using libunistring
// or equivalent.  There's one special case: A with ring, which PLATO
// handles as a single code, there is no "autobackspacing ring accent".
// By contrast, all the other supported accents are accepted as "combining
// xyz accent" codes, since PLATO basically handles them as detached.  And
// any of the combined codes are stored in the keytable as the unaccented
// letter followed by the PLATO key code for the accent.

#define None 0xffffffff
#define KEY(a, b, c, d) (((a) & 0xff) + (((b) & 0xff) << 8) + \
                         + (((c) & 0xff) << 16) + (((d) & 0xff) << 24))
#define KEY1(a) KEY ((a), None, None, None)
typedef struct 
{
    u32 u;
    u32 p;
} ukey;

typedef wxChar cmentry[4];

typedef struct 
{
    cmentry s;
    wxChar  c;
} autobsentry;

// ptermkeytabs.h is generated by composed.py.  It defines a number of
// character mapping tables:
// asciiToPlato maps ASCII (7 bit) keyboard character values to the
// corresponding PLATO key code(s).  Each entry is either None (ignored)
// or the KEY macro specifying up to four PLATO key codes, with None as 
// filler if less than four are needed.  Note that this table only has 
// entries for keystrokes whose shift handling is the same as in a 
// conventional ASCII keyboard.  For example, SPACE is not listed here
// (it is handled explicitly elsewhere in the code) because shift-SPACE
// is backspace.
// pastedAsciiToPlato -- same as asciiToPlato except that this one is
// used to map ASCII character codes found in paste strings.  Most
// control characters are unused here, and SPACE is a normal entry.
// unicodeToPlato -- a lookup table consisting of pairs; each entry is a 
// Unicode character code and the corresponding PLATO key sequence.
// The entries are in ascending order to allow for binary search, if
// desired.  There are entries here for precomposed characters (like 
// O with umlaut) as well as for the separate accent marks (like
// combining umlaut).
// autobsmap -- a lookup table of autobsentry items.  Each entry maps
// a sequence of up to 3 Unicode characters to a replacement Unicode
// character.  The inputs correspond to entries in the screen text map:
// characters plotted, including any autobackspaced bits like accents
// or parts of composite characters.  The output is the Unicode character
// we want to use for that sequence.

#include "ptermkeytabs.h"

// Keycode translation for ALT-keypress.  -1 means not valid. 
const i8 altKeyToPlato[128] =
{
 /*                                                                         */
 /* 000- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*                                                                         */
 /* 010- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*                                                                         */
 /* 020- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*                                                                         */
 /* 030- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*          space   !       "       #       $       %       &       '      */
 /* 040- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,   
 /*          (       )       *       +       ,       -       .       /      */
 /* 050- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,   
 /*          0       1       2       3       4       5       6       7      */
 /* 060- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,   
 /*          8       9       :       ;       <       =       >       ?      */
 /* 070- */ -1,     -1,     -1,     -1,     -1,      0015,  -1,     -1,   
 /*          @       A       B       C       D       E       F       G      */
 /* 100- */ -1,      0062,   0070,   0073,   0071,   0067,   0064,  -1,   
 /*          H       I       J       K       L       M       N       O      */
 /* 110- */  0065,  -1,     -1,     -1,      0075,   0064,   0066,  -1,   
 /*          P       Q       R       S       T       U       V       W      */
 /* 120- */ -1,      0074,   0063,   0072,   0062,  -1,     -1,     -1,   
 /*          X       Y       Z       [       \       ]       ^       _      */
 /* 130- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,   
 /*          `       a       b       c       d       e       f       g      */
 /* 140- */ -1,      0022,   0030,   0033,   0031,   0027,   0064,  -1,   
 /*          h       i       j       k       l       m       n       o      */
 /* 150- */  0025,  -1,     -1,     -1,      0035,   0024,   0026,  -1,   
 /*          p       q       r       s       t       u       v       w      */
 /* 160- */ -1,      0034,   0023,   0032,   0062,  -1,     -1,     -1,   
 /*          x       y       z       {       |       }       ~              */
 /* 170- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1
};

/*
**  This table is for translating PLATO text as it would appear in a
**  upper case only printout.  Shift shows up as a separate character (')
**  and has to be handled specially, and some other punctuation characters
**  have unexpected meanings:
**      "   multiply
**      #   divide
**      &   super
**      !   sub
**      ?   cr
**      \   font
**      ^   access
**      @   backspace
**      _   assign
**  In this table, the unshifted code values appear.  -1 means no
**  translation, -2 marks Shift.
*/
const int printoutToPlato[128] =
{
 /*                                                                         */
 /* 000- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*                                                                         */
 /* 010- */ -1,    014,    026,     -1,     -1,     -1,     -1,     -1,
 /*                                                                         */
 /* 020- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*                                                                         */
 /* 030- */ -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 /*          space   !       "       #       $       %       &       '      */
 /* 040- */  0100,   0021,   0012,   0013,   0044,   0045,   0020,  -2,
 /*          (       )       *       +       ,       -       .       /      */
 /* 050- */  0051,   0173,   0050,   0016,   0137,   0017,   0136,   0135,
 /*          0       1       2       3       4       5       6       7      */
 /* 060- */  0000,   0001,   0002,   0003,   0004,   0005,   0006,   0007,
 /*          8       9       :       ;       <       =       >       ?      */
 /* 070- */  0010,   0011,   0174,   0134,   0040,   0133,   0041,   0175,
 /*          @       A       B       C       D       E       F       G      */
 /* 100- */  0140,   0101,   0102,   0103,   0104,   0105,   0106,   0107,
 /*          H       I       J       K       L       M       N       O      */
 /* 110- */  0110,   0111,   0112,   0113,   0114,   0115,   0116,   0117,
 /*          P       Q       R       S       T       U       V       W      */
 /* 120- */  0120,   0121,   0122,   0123,   0124,   0125,   0126,   0127,
 /*          X       Y       Z       [       \       ]       ^       _      */
 /* 130- */  0130,   0131,   0132,   0042,   0064,   0043,   0074,   0015,
 /*          `       a       b       c       d       e       f       g      */
 /* 140- */ -1,      0101,   0102,   0103,   0104,   0105,   0106,   0107,
 /*          h       i       j       k       l       m       n       o      */
 /* 150- */  0110,   0111,   0112,   0113,   0114,   0115,   0116,   0117,
 /*          p       q       r       s       t       u       v       w      */
 /* 160- */  0120,   0121,   0122,   0123,   0124,   0125,   0126,   0127,
 /*          x       y       z       {       |       }       ~              */
 /* 170- */  0130,   0131,   0132,  -1,     -1,     -1,     -1,     -1,
};

// A bunch of these characters are outside the ASCII set, so they are
// written as Unicode escapes.  Too bad C doesn't accept plain Unicode
// text inside quoted strings, as Python does.  Comments say what characters
// those escapes reference.
// The first 64 characters are for the set 0 codes; the rest are for
// the set 1 codes.  Codes past offset 64 in set 1 correspond to the
// "special" characters in ASCII mode, which are essentially built-in
// composites that on a Classic terminal would be made by the formatter.
// Here we just treat them as extra characters for simplicity.
static const wxChar rom01char[] =
    L":abcdefg"
    L"hijklmno"
    L"pqrstuvw"
    L"xyz01234"
    L"56789+-*"
    L"/()$= ,."
    L"\u00F7[]%\u00D7\u21E6'\""     // divide, multiply, left arrow (assign)
    L"!;<>_?\u2AA2 "                // double greater than (plato "arrow")
    L"#ABCDEFG"
    L"HIJKLMNO"
    L"PQRSTUVW"
    L"XYZ\u02DC\u00A8^\u00B4`"      // small tilde, dieresis, acute
    L"\u2191\u2192\u2193\u2190"     // up, right, down, left arrow
    L"~\u03A3\u0394\u222A"          // Sigma, Delta, union
    L"\u2229{}&\u2260 |\u00B0"      // intersection, not-equal, degree
    L"\u2263\u03B1\u03B2\u03B4"     // equiv, alpha, beta, delta
    L"\u03BB\u03BC\u03C0\u03C1"     // lambda, mu, pi, rho
    L"\u03C3\u03C9\u2264\u2265"     // sigma, omega, less/equal, grt/equal
    L"\u0398@\\ "                   // Theta
    L"\u2993\u2994\u00A9\u25AB"     // embed left, right, copyright, box
    L"\u25C6\u2715\u02C7\u2195"     // diamond, cross prod, hacek, up/down
    L"\u25CB\u00B8";               // dot product, cedilla

// Tables to map ASCII characters to the correct character image.  There
// are two tables, one for the M0 set, one for the M1 set.  Note that
// these sets do not directly correspond to the classic terminal sets.
// Each table is indexed by the ASCII character code (7 bit value).
//
// The value found in the table is the classic ROM character index for
// this character, in the low 7 bits.  The top bit selects M0 vs. M1
// in the classic ROM.  0xff means unused code.
//
// Codes 0xc0 and up are specials (ASCII only extra character codes); in
// the classic terminal these are instead formatted as sequences or codes
// with displayed Y position, but in ASCII they are directly coded as 
// additional characters.  We simply handle these as additional character
// shapes, stored in the M1 character ROM data beyond the usual 64 entries.
//
// 0xc0: embed left
// 0xc1: embed right
// 0xc2: copyright body
// 0xc3: box
// 0xc4: diamond
// 0xc5: cross product
// 0xc6: hacek
// 0xc7: universal delimiter
// 0xc8: dot product
// 0xc9: cedilla

static const u8 asciiM0[] = 
{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x2d, 0x38, 0x37, 0x80, 0x2b, 0x33, 0xab, 0x36,
  0x29, 0x2a, 0x27, 0x25, 0x2e, 0x26, 0x2f, 0x28,
  0x1b, 0x1c, 0x1d, 0x1e, 0x1f, 0x20, 0x21, 0x22,
  0x23, 0x24, 0x00, 0x39, 0x3a, 0x2c, 0x3b, 0x3d,
  0xbd, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
  0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
  0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97,
  0x98, 0x99, 0x9a, 0x31, 0xbe, 0x32, 0x9d, 0x3c,
  0x9f, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
  0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
  0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
  0x18, 0x19, 0x1a, 0xa9, 0xae, 0xaa, 0xa4, 0xff
};
static const u8 asciiM1[] = 
{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x2d, 0x28, 0xb0, 0x9b, 0x35, 0xac, 0xa0, 0xa1,
  0xa2, 0xa3, 0x34, 0xa5, 0xa6, 0xa7, 0xa8, 0x30,
  0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8,
  0xb9, 0xba, 0xbb, 0xbc, 0xc0, 0xaf, 0xc1, 0x3e,
  0xc2, 0x9c, 0xc3, 0xc8, 0xc4, 0xc5, 0x9e, 0xc9,
  0xc6, 0xc7, 0xae, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};
static const u8 asciiKeycodes[] =
{ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
  0x38, 0x39, 0x26, 0x60, 0x0a, 0x5e, 0x2b, 0x2d,
  0x13, 0x04, 0x07, 0x08, 0x7b, 0x0b, 0x0d, 0x1a,
  0x02, 0x12, 0x01, 0x03, 0x7d, 0x0c, 0xff, 0xff,
  0x3c, 0x3e, 0x5b, 0x5d, 0x24, 0x25, 0x5f, 0x7c,
  0x2a, 0x28, 0x40, 0x27, 0x1c, 0x5c, 0x23, 0x7e,
  0x17, 0x05, 0x14, 0x19, 0x7f, 0x09, 0x1e, 0x18,
  0x0e, 0x1d, 0x11, 0x16, 0x00, 0x0f, 0xff, 0xff,
  0x20, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,
  0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
  0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77,
  0x78, 0x79, 0x7a, 0x3d, 0x3b, 0x2f, 0x2e, 0x2c,
  0x1f, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,
  0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f,
  0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,
  0x58, 0x59, 0x5a, 0x29, 0x3a, 0x3f, 0x21, 0x22
};

// Conversion from ascii mode codes to classic codes
static const u8 ascmode[] = { 0, 3, 2, 1 };

// ----------------------------------------------------------------------------
// private classes
// ----------------------------------------------------------------------------

class PtermFrame;

// Pterm screen printout
class PtermPrintout: public wxPrintout
{
public:
    PtermPrintout (PtermFrame *owner,
                   const wxString &title = _("Pterm printout")) 
        : wxPrintout (title),
          m_owner (owner)
    {}
    bool OnPrintPage (int page);
    bool HasPage (int page);
    void GetPageInfo (int *minPage, int *maxPage, int *selPageFrom,
                      int *selPageTo);
    void DrawPage (wxDC *dc);

private:
    PtermFrame *m_owner;
};


// Pterm connection state
class PtermConnection
{
public:
    // override base class virtuals
    // ----------------------------
    PtermConnection (PtermFrame *owner, wxString &host, int port);
    ~PtermConnection ();
    
    // Callback handlers
    static void s_connCallback (NetFet *np, int portNum, void *arg);
    static void s_dataCallback (NetFet *np, int bytes, void *arg);
    void connCallback (void);
    void dataCallback (void);

    int AssembleNiuWord (void);
    int AssembleAsciiWord (void);
    int AssembleAutoWord (void);
    int NextWord (void);
    int NextGswWord (bool catchup);
    
    void SendData (const void *data, int len);

    bool IsEmpty (void) const
    {
        return (m_displayIn == m_displayOut);
    }
    bool IsFull (void) const
    {
        int next;
    
        next = m_displayIn + 1;
        if (next == RINGSIZE)
        {
            next = 0;
        }
        return (next == m_displayOut);
    }
    int RingCount (void) const
    {
        if (m_displayIn >= m_displayOut)
        {
            return m_displayIn - m_displayOut;
        }
        else
        {
            return RINGSIZE + m_displayIn - m_displayOut;
        }
    }
    bool Ascii (void) const
    {
        return (m_connMode == ascii);
    }
    bool GswActive (void) const
    {
        return m_gswActive;
    }
    void StoreWord (int word);

private:
    NetPortSet  m_portset;
    NetFet      *m_fet;
    u32         m_displayRing[RINGSIZE];
    volatile int m_displayIn, m_displayOut;
    u32         m_gswRing[GSWRINGSIZE];
    volatile int m_gswIn, m_gswOut;
    PtermFrame  *m_owner;
    wxString    m_hostName;
    int         m_port;
    wxCriticalSection m_pointerLock;
    bool        m_gswActive;
    bool        m_gswStarted;
    int         m_savedGswMode;
    int         m_gswWord2;
    enum { both, niu, ascii } m_connMode;
    int         m_pending;
    in_addr_t   m_hostAddr;
    bool        m_connActive;
    
    int NextRingWord (void);
};

extern "C" int ptermNextGswWord (void *connection, int catchup);

// Define a new application type, each program should derive a class from wxApp
class PtermApp : public wxApp
{
public:
    // override base class virtuals
    // ----------------------------

    // this one is called on application startup and is a good place for the app
    // initialization (doing it here and not in the ctor allows to have an error
    // return: if OnInit () returns false, the application terminates)
    virtual bool OnInit (void);
    virtual int OnExit (void);

    // event handlers
    void OnConnect (wxCommandEvent &event);
    void OnQuit (wxCommandEvent& event);
    void OnHelpKeys (wxCommandEvent &event);
    void OnAbout (wxCommandEvent& event);

    bool DoConnect (bool ask);
    int m_connError;
    int m_connAction;
    
    bool LoadProfile (wxString profile, wxString filename);
    wxString ProfileFileName (wxString profile);

    static wxColour SelectColor (wxWindow &parent, const wxChar *title, 
                                 wxColour &initcol);
    
    wxConfig    *m_config;

    //general
    int         m_lastTab;
    //tab0
    wxString    m_curProfile;
    //tab1
    wxString    m_ShellFirst;
    bool        m_connect;
    wxString    m_hostName;
    long        m_port;
    long        m_termType;
    
    //tab2
    bool        m_showSignon;
    bool        m_showSysName;
    bool        m_showHost;
    bool        m_showStation;
    //tab3
    bool        m_classicSpeed;
    bool        m_gswEnable;
    bool        m_numpadArrows;
    bool        m_ignoreCapLock;
    bool        m_platoKb;
    bool        m_useAccel;
    bool        m_beepEnable;
    bool        m_DisableShiftSpace;
    bool        m_DisableMouseDrag;
    //tab4
    bool        m_scale2;
    bool        m_stretch;
    bool        m_showStatusBar;
#if !defined (__WXMAC__)
    bool        m_showMenuBar;
#else
    static const bool m_showMenuBar;
#endif
    bool        m_noColor;
    wxColour    m_fgColor;
    wxColour    m_bgColor;
    
    //tab5
    long        m_charDelay;
    long        m_lineDelay;
    long        m_autoLF;
    bool        m_splitWords;
    bool        m_smartPaste;
    bool        m_convDot7;
    bool        m_conv8Sp;
    bool        m_TutorColor;
    //tab6
    wxString    m_Email;      
    wxString    m_SearchURL;      

    PtermFrame  *m_firstFrame;
    wxString    m_defDir;
    PtermFrame  *m_helpFrame;

    long        prefX;
    long        prefY;

    PtermFrame *m_CurFrame;

    FILE *m_testdata;
    bool m_testascii;
    bool m_testreq;
    char traceFn[20];

private:
    wxLocale    m_locale; // locale we'll be using
    
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE ()
};

#if defined (__WXMAC__)
// On the Mac, menus are always displayed
const bool PtermApp::m_showMenuBar = true;
#endif

// define a scrollable canvas for drawing onto
class PtermCanvas : public wxScrolledCanvas
{
public:
    PtermCanvas (PtermFrame *parent);

    void ptermTouchPanel (bool enable);

    void OnDraw (wxDC &dc);
    void OnCharHook (wxKeyEvent& event);
    void OnChar (wxKeyEvent& event);
    void OnMouseDown (wxMouseEvent &event);
    void OnMouseUp (wxMouseEvent &event);
    void OnMouseContextMenu (wxMouseEvent &event);
    void OnMouseMotion (wxMouseEvent &event);
    void OnMouseWheel (wxMouseEvent &event);

#if defined (__WXMSW__)
    WXLRESULT MSWWindowProc (WXUINT message, WXWPARAM wParam, WXLPARAM lParam);
#endif
    
    void Unadjust (int x, int y, int *xx, int *yy) const;
    
    int m_mouseX;
    int m_mouseY;
    
private:
    PtermFrame *m_owner;
    bool        m_touchEnabled;

    DECLARE_EVENT_TABLE ()
};

#if defined (__WXMAC__)
#define PTERM_MDI 1
#else
#define PTERM_MDI 0
#endif

#if PTERM_MDI
#define PtermFrameBase  wxMDIChildFrame

class PtermMainFrame : public wxMDIParentFrame
{
public:
    PtermMainFrame (void);
    wxMenuItem* AppendWinItem (int itemid,
                               const wxString& text,
                               const wxString& help = wxEmptyString,
                               wxItemKind kind = wxITEM_NORMAL)
    {
        if (m_windowMenu != NULL)
            return m_windowMenu->Append (itemid, text, help, kind);
        else
            return NULL;
    }
    wxMenuBar   *menuBar;
    wxMenu      *menuFile;
    wxMenu      *menuHelp;
};

static PtermMainFrame *PtermFrameParent;
#else
#define PtermFrameBase wxFrame
#define PtermFrameParent NULL
#endif

// Define a new frame type: this is going to be our main frame
class PtermFrame : public PtermFrameBase, public emul8080
{
    friend void PtermCanvas::OnDraw (wxDC &dc);
    friend void PtermApp::OnHelpKeys (wxCommandEvent &event);
    friend bool PtermApp::OnInit (void);
    friend int PtermConnection::NextWord (void);
    friend void PtermPrintout::DrawPage (wxDC *);
    friend void PtermCanvas::OnMouseDown (wxMouseEvent &event);
    friend void PtermCanvas::OnMouseMotion (wxMouseEvent &event);
    friend void PtermCanvas::OnMouseUp (wxMouseEvent &event);
    
    typedef wxAlphaPixelData PixelData;

public:
    // ctor(s)
    PtermFrame (wxString &host, int port, const wxString& title);
    ~PtermFrame ();

    // event handlers (these functions should _not_ be virtual)
    void OnIdle (wxIdleEvent& event);
    void OnClose (wxCloseEvent& event);
    void OnTimer (wxTimerEvent& event);
    void OnPasteTimer (wxTimerEvent& event);
    void OnShellTimer (wxTimerEvent& event);
    void OnQuit (wxCommandEvent& event);
#if !defined (__WXMAC__)
    void OnToggleMenuBar (wxCommandEvent &event);
#endif
    void OnToggleStatusBar (wxCommandEvent &event);
    void OnToggle2xMode (wxCommandEvent &event);
    void OnToggleStretchMode (wxCommandEvent &event);
    void OnCopyScreen (wxCommandEvent &event);
    void OnCopy (wxCommandEvent &event);
    void OnExec (wxCommandEvent &event);
    void OnMailTo (wxCommandEvent &event);
    void OnSearchThis (wxCommandEvent &event);
    void OnMacro0 (wxCommandEvent &event);
    void OnMacro1 (wxCommandEvent &event);
    void OnMacro2 (wxCommandEvent &event);
    void OnMacro3 (wxCommandEvent &event);
    void OnMacro4 (wxCommandEvent &event);
    void OnMacro5 (wxCommandEvent &event);
    void OnMacro6 (wxCommandEvent &event);
    void OnMacro7 (wxCommandEvent &event);
    void OnMacro8 (wxCommandEvent &event);
    void OnMacro9 (wxCommandEvent &event);
    void OnPaste (wxCommandEvent &event);
    void OnUpdateUIPaste (wxUpdateUIEvent& event);
    void OnSaveScreen (wxCommandEvent &event);
    void OnSaveAudio (wxCommandEvent &event);
    void OnPrint (wxCommandEvent& event);
    void OnPrintPreview (wxCommandEvent& event);
    void OnPageSetup (wxCommandEvent& event);
    void OnPref (wxCommandEvent& event);
    void OnActivate (wxActivateEvent &event);
    void SavePreferences (void);
    void SetColors (wxColour &newfg, wxColour &newbg);
    void OnFullScreen (wxCommandEvent &event);
    void OnResize (wxSizeEvent& event);
    void UpdateDisplayState (void);
#if defined (__WXMSW__)
    void OnIconize (wxIconizeEvent &event);
#endif

    void BuildMenuBar (void);
    void BuildFileMenu (int port);
    void BuildEditMenu (int port);
    void BuildViewMenu (int port);
    void BuildHelpMenu (void);
    void BuildPopupMenu (int port);
	void BuildStatusBar (bool connecting = false);
    void ptermSendKey1 (int key);
    void ptermSendKey (u32 keys);
    void ptermSendKeys (const int key[]);
    void ptermSendTouch (int x, int y);
    void ptermSendExt (int key);
    void ptermSetTrace (bool trace);
    void ProcessPlatoMetaData (void);
    void WriteTraceMessage (wxString);
#if 0
    // The s0ascers spec calls for parity but it isn't really needed
    // and by not doing it we keep things simpler.  For example,
    // that way we don't need to worry about telnet escaping
    // (0xff escaping) in the inbound direction.
    int Parity (int key);
#else
#define Parity(x) (x)
#endif

    bool HasConnection (void) const
    {
        return (m_conn != NULL);
    }
    void ptermSetStatus (wxString &str);
    void ptermShowTrace ();
    void trace (const wxString &) const;
    void trace (const char *, ...) const;
    
    wxMemoryDC  *m_memDC;
    bool        tracePterm;
    int         m_currentWord;      // to be displayed when tracing
    PtermFrame  *m_nextFrame;
    PtermFrame  *m_prevFrame;

    int         m_pendingEcho;

    bool        m_bCancelPaste;
    bool        m_bPasteActive;
    PtermConnection *m_conn;
    bool        m_dumbTty;

    wxMenuBar   *menuBar;           // present even if not displayed
    wxMenu      *menuFile;
    wxMenu      *menuEdit;
    wxMenu      *menuView;
    wxMenu      *menuHelp;
    wxMenu      *menuPopup;
    wxStatusBar *m_statusBar;       // NOT present if not displayed

    // These 3 flags together control how, and to what size, the 
    // PLATO image is displayed.
    bool        m_scale2;
    bool        m_stretch;
    bool        m_fullScreen;
    // These are the X and Y scale factors, derived from the flags above
    // and in the case of stretch mode, the current window or display size.
    float       m_xscale, m_yscale;
    // These are the current margins.  Usually those are simply DisplayMargin,
    // but in full screen mode they account for the entire space beyond the
    // actual display area.  They are in units of PLATO pixels, i.e., the
    // screen margins are this times m_xscale and m_yscale respectively.
    int         m_xmargin, m_ymargin;
    
    //fonts
    wxFont      *m_font;
    bool        m_usefont;
    wxFontFamily m_fontfamily;
    wxString    m_fontface;
    int         m_fontsize;
    bool        m_fontitalic;
    bool        m_fontbold;
    bool        m_fontstrike;
    bool        m_fontunderln;
    wxCoord     m_fontwidth;
    wxCoord     m_fontheight;
    bool        m_fontPMD;
    bool        m_fontinfo;
    //operating system request
    bool        m_osinfo;

    // GSW sound output file handling
    wxString    m_gswFile;
    int         m_gswFFmt;
    
private:
    u32         m_fgpix;
    u32         m_bgpix;
    wxBitmap    *m_bitmap;
    u32         m_maxalpha;
    u32         m_selpixf;
    u32         m_selpixb;
    wxBitmap    *m_selmap;
    u32         m_red;
    u32         m_green;
    u32         m_blue;
    PtermCanvas *m_canvas;
    wxString    m_curProfile;
    wxString    m_ShellFirst;
    wxString    m_hostName;
    long        m_port;
    wxTimer     m_timer;
    int         m_station;
    wxRect      m_rect;         // Used to save window size/pos
    
    // Stuff for pacing Paste operations
    wxTimer     m_pasteTimer;
    wxString    m_pasteText;
    int         m_pasteIndex;
    int         m_pasteNextIndex;
    int         m_pasteLen;
    bool        m_pastePrint;
    int         m_pasteLinePos;

    // The next word to be processed, and its associated delay.
    // We set this if we pick up a word from the connection object, and
    // either it comes with an associated delay, or "true timing" is selected
    // via preferences, or GSW emulation is active.
    int         m_nextword;
    int         m_delay;
    
    // PLATO terminal emulation state
#define mode RAM[M_MODE]
    bool        modexor;
#define wemode (mode & 3)
    int         currentX;
    int         currentY;
#define margin ReadRAMW (M_MARGIN)
    int         memaddr;
    u16         plato_m23[128 * 8];
    int         memlpc;
#define uncover     ((RAM[M_CCR] & 0x80) != 0)
#define reverse     ((RAM[M_CCR] & 0x40) != 0 )
#define large       ((RAM[M_CCR] & 0x20) != 0)
#define currentCharset ((RAM[M_CCR] & 0x0e) >> 1)
#define vertical    ((RAM[M_CCR] & 0x01) != 0)
    int         wc;
    int         seq;
    int         modewords;
    int         mode4start;
    typedef enum { none, ldc, lde, lda, ssf, fg, bg, gsfg, paint,
                   pni_rs, ext, pmd } AscState;
    AscState    m_ascState;
    int         m_ascBytes;
    int         m_assembler;
    int         lastX;
    int         lastY;
    bool        m_flowCtrl;
    bool        m_sendFgt;
    
    wxColour    m_defFg;
    wxColour    m_defBg;
    wxColour    m_currentFg;
    wxColour    m_currentBg;

    bool        m_loadingPMD;
    wxString    m_PMD;

    // CYBIS workstation windowing defines
#define CWS_SAVE        0
#define CWS_RESTORE     1
#define CWS_EXEC        1012
#define CWS_TERMSAVE    2000
#define CWS_TERMRESTORE 2001
    int     cwsmode, cwsfun, cwscnt, cwswin;
    struct  cws
        {
        bool        ok;
        int         data[4];
        wxBitmap    *bm;
        };
    cws cwswindow[10];

    void setMargin (int i)
    {
        RAM[M_MARGIN] = i;
        RAM[M_MARGIN + 1] = i >> 8;
    }
    void setUncover (bool u)
    {
        if (u)
        {
            RAM[M_CCR] |= 0x80;
        }
        else
        {
            RAM[M_CCR] &= ~0x80;
        }
    }
    void setReverse (bool u)
    {
        if (u)
        {
            RAM[M_CCR] |= 0x40;
        }
        else
        {
            RAM[M_CCR] &= ~0x40;
        }
    }
    void setLarge (bool u)
    {
        if (u)
        {
            RAM[M_CCR] |= 0x20;
        }
        else
        {
            RAM[M_CCR] &= ~0x20;
        }
    }
    void setCmem (int i)
    {
        RAM[M_CCR] = (RAM[M_CCR] & ~0x0e) | (i << 1);
    }
    void setVertical (bool u)
    {
        if (u)
        {
            RAM[M_CCR] |= 0x01;
        }
        else
        {
            RAM[M_CCR] &= ~0x01;
        }
    }
    
    // PLATO drawing primitives
    void ptermDrawChar (int x, int y, int snum, int cnum, bool autobs = false);
    void ptermDrawCharInto (int x, int y, const u16 *charp,
                            u32 fpix, u32 bpix, int cmode,
                            bool xor_p, PixelData &pixmap);
    void ptermDrawPoint (int x, int y);
    void ptermUpdatePoint (int x, int y, u32 pixval, bool xor_p,
                           PixelData & pixmap);
    void ptermDrawLine (int x1, int y1, int x2, int y2);
    void ptermFullErase (void);
    void ptermBlockErase (int x1, int y1, int x2, int y2);
    void ptermSetName (wxString &winName);
    void ptermPaint (int pat);
    void ptermPaintWalker (int x, int y, PixelData & pixmap, int pass);
    void ptermSaveWindow (int d);
    void ptermRestoreWindow (int d);
    
    void drawFontChar (int x, int y, int c);
    bool procPlatoWord (u32 d, bool ascii);
    void plotChar (int c);
    void mode0 (u32 d);
    void mode1 (u32 d);
    void mode2 (u32 d);
    void mode3 (u32 d);
    void mode4 (u32 d);
    void mode5 (u32 d);
    void mode6 (u32 d);
    void mode7 (u32 d);
    void progmode (u32 d, int origin);
    void ptermSetStation (int station, bool showtitle, bool showstatus);
    
    bool AssembleCoord (int d);
    int AssemblePaint (int d);
    int AssembleData (int d);
    int AssembleColor (int d);
    int AssembleGrayScale (int d);

    int AssembleAsciiPlatoMetaData (int d);
    bool AssembleClassicPlatoMetaData (int d);
    void SetFontFaceAndFamily (int n);
    void SetFontSize (int n);
    void SetFontFlags (int n);
    void SetFontActive (void);
    
    typedef void (PtermFrame::*mptr)(u32);

    static const mptr modePtr[8];

    // Text-copy support
    bool SaveChar (int x, int y, wxChar c, bool large_p);
    void ClearRegion (void);
    void UpdateRegion (int x, int y, int mousex, int mousey);
    wxString GetRegionText (bool url = false) const;
    
    cmentry textmap[32 * 64];
    int m_regionX;
    int m_regionY;
    int m_regionHeight;
    int m_regionWidth;
    bool m_autobs;
    
    // 8080a emulation support
    u8 input8080a (u8 data);
    void output8080a (u8 data, u8 acc);
    int check_pc8080a (void);

    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE ()
};

const PtermFrame::mptr PtermFrame::modePtr[8] = 
{
    &PtermFrame::mode0, &PtermFrame::mode1,
    &PtermFrame::mode2, &PtermFrame::mode3,
    &PtermFrame::mode4, &PtermFrame::mode5,
    &PtermFrame::mode6, &PtermFrame::mode7
};

// define the preferences dialog
class PtermPrefDialog : public wxDialog
{
public:
    PtermPrefDialog (PtermFrame *parent, wxWindowID id,
                     const wxString &title, wxPoint pos,  wxSize size);

    //event sink handlers
    void OnButton (wxCommandEvent& event);
    void OnCheckbox (wxCommandEvent& event);
    void OnSelect (wxCommandEvent& event);
    void OnDoubleClick (wxCommandEvent& event);
    void OnChange (wxCommandEvent& event);
    void OnComboSelect (wxCommandEvent& event);
    //void OnClose (wxCloseEvent &) { EndModal (wxID_CANCEL); }
    void OnClose (wxCloseEvent &);
    //support routines
    bool ValidProfile (wxString profile);
    bool SaveProfile (wxString profile);
    bool DeleteProfile (wxString profile);
    void SetControlState (void);
    //objects
    wxNotebook* tabPrefsDialog;
    //tab0
    wxListBox* lstProfiles;
    wxButton* btnSave;
    wxButton* btnLoad;
    wxButton* btnDelete;
    wxStaticText* lblProfileStatusMessage;
    wxTextCtrl* txtProfile;
    wxButton* btnAdd;
    //tab1
    wxCheckBox* chkConnectAtStartup;
    wxTextCtrl* txtShellFirst;
    wxTextCtrl* txtDefaultHost;
    wxComboBox* cboDefaultPort;
    //tab2
    wxCheckBox* chkShowSignon;
    wxCheckBox* chkShowSysName;
    wxCheckBox* chkShowHost;
    wxCheckBox* chkShowStation;
    //tab3
    wxCheckBox* chkSimulate1200Baud;
    wxCheckBox* chkEnableGSW;
    wxCheckBox* chkEnableNumericKeyPad;
    wxCheckBox* chkIgnoreCapLock;
    wxCheckBox* chkUsePLATOKeyboard;
    wxCheckBox* chkUseAccelerators;
    wxCheckBox* chkEnableBeep;
    wxCheckBox* chkDisableShiftSpace;
    wxCheckBox* chkDisableMouseDrag;
    //tab4
    //wxCheckBox* chkZoom200;
    //wxCheckBox* chkStatusBar;
    //wxCheckBox* chkMenuBar;
    wxCheckBox* chkDisableColor;
#if defined (_WIN32)
    wxButton* btnFGColor;
    wxButton* btnBGColor;
#else
    wxBitmapButton* btnFGColor;
    wxBitmapButton* btnBGColor;
#endif
    //tab5
    wxTextCtrl* txtCharDelay;
    wxTextCtrl* txtLineDelay;
    wxComboBox* cboAutoLF;
    wxCheckBox* chkSplitWords;
    wxCheckBox* chkSmartPaste;
    wxCheckBox* chkConvertDot7;
    wxCheckBox* chkConvert8Spaces;
    wxCheckBox* chkTutorColor;
    //tab6
    wxTextCtrl* txtEmail;
    wxTextCtrl* txtSearchURL;
    //button bar
    wxButton* btnOK;
    wxButton* btnCancel;
    wxButton* btnDefaults;

    //properties    
    int             m_lastTab;
    //tab0
    wxString        m_curProfile;
    //tab1
    wxString        m_ShellFirst;
    bool            m_connect;
    wxString        m_host;
    long            m_port;
    //tab2
    bool            m_showSignon;
    bool            m_showSysName;
    bool            m_showHost;
    bool            m_showStation;
    //tab3
    bool            m_classicSpeed;
    bool            m_gswEnable;
    bool            m_numpadArrows;
    bool            m_ignoreCapLock;
    bool            m_platoKb;
    bool            m_useAccel;
    bool            m_beepEnable;
    bool            m_DisableShiftSpace;
    bool            m_DisableMouseDrag;
    //tab4
    bool            m_scale2;
    bool            m_stretch;
    bool            m_showStatusBar;
#if !defined (__WXMAC__)
    bool            m_showMenuBar;
#endif
    bool            m_noColor;
    wxColour        m_fgColor;
    wxColour        m_bgColor;
    //tab5
    long            m_charDelay;
    long            m_lineDelay;
    long            m_autoLF;
    bool            m_splitWords;
    bool            m_smartPaste;
    bool            m_convDot7;
    bool            m_conv8Sp;
    bool            m_TutorColor;
    //tab6
    wxString        m_Email;
    wxString        m_SearchURL;
    
private:
    void paintBitmap (wxBitmap &bm, wxColour &color);
    
    DECLARE_EVENT_TABLE ()
};

// define the connection dialog
class PtermConnDialog : public wxDialog
{
public:
    PtermConnDialog (wxWindowID id, const wxString &title, wxPoint pos,
                     wxSize size);
    
    void OnButton (wxCommandEvent& event);
    void OnSelect (wxCommandEvent& event);
    void OnChange (wxCommandEvent& event);
    void OnDoubleClick (wxCommandEvent& event);
    void OnClose (wxCloseEvent &) { EndModal (wxID_CANCEL); }
    
    wxString        m_ShellFirst;
    wxString        m_curProfile;
    wxString        m_host;
    wxString        m_port;
    wxConfig        m_config;

    wxListBox* lstProfiles;
    wxTextCtrl* txtShellFirst;
    wxTextCtrl* txtHost;
    wxComboBox* cboPort;
    wxButton* btnCancel;
    wxButton* btnConnect;
    
private:
    DECLARE_EVENT_TABLE ()
};

// define the connection fail dialog
class PtermConnFailDialog : public wxDialog
{
public:
    PtermConnFailDialog (wxWindowID id, const wxString &title, wxPoint pos,
                         wxSize size);
    
    void OnButton (wxCommandEvent& event);
    void OnClose (wxCloseEvent& event);
    
    wxButton* btnNew;
    wxButton* btnRetry;
    wxButton* btnCancel;
    
private:
    DECLARE_EVENT_TABLE ()
};

// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------

// IDs for the controls and the menu commands
enum
{
    // menu items
    Pterm_ToggleMenuBar = 1,
    Pterm_ToggleMenuBarView,
    Pterm_ToggleStatusBar,
    Pterm_ToggleStatusBarView,
    Pterm_Toggle2xMode,
    Pterm_Toggle2xModeView,
    Pterm_ToggleStretchMode,
    Pterm_ToggleStretchModeView,
    Pterm_CopyScreen,
    Pterm_ConnectAgain,
    Pterm_SaveScreen,
    Pterm_SaveAudio,
    Pterm_HelpKeys,
    Pterm_PastePrint,
    Pterm_FullScreen,
    Pterm_FullScreenView,
    Pterm_Resize,

    // timers
    Pterm_Timer,        // display pacing
    Pterm_PasteTimer,   // paste key generation pacing
    //other items
    Pterm_Exec,         // execute URL
    Pterm_MailTo,       // execute email client
    Pterm_SearchThis,   // execute search URL
    Pterm_Macro0,
    Pterm_Macro1,
    Pterm_Macro2,
    Pterm_Macro3,
    Pterm_Macro4,
    Pterm_Macro5,
    Pterm_Macro6,
    Pterm_Macro7,
    Pterm_Macro8,
    Pterm_Macro9,

    // Menu items with standard ID values
    Pterm_Print = wxID_PRINT,
    Pterm_Page_Setup = wxID_PRINT_SETUP,
    Pterm_Preview = wxID_PREVIEW,
    Pterm_Copy = wxID_COPY,
    Pterm_Paste = wxID_PASTE,
    Pterm_Connect = wxID_NEW,
    Pterm_Quit = wxID_EXIT,
    Pterm_Close = wxID_CLOSE,
    Pterm_Pref = wxID_PREFERENCES,

    // it is important for the id corresponding to the "About" command to have
    // this standard value as otherwise it won't be handled properly under Mac
    // (where it is special and put into the "Apple" menu)
    Pterm_About = wxID_ABOUT
};

/* data for plato font, set 0. */
const unsigned short plato_m0[] = {
    0x0000, 0x0000, 0x0330, 0x0330, 0x0000, 0x0000, 0x0000, 0x0000, // :
    0x0060, 0x0290, 0x0290, 0x0290, 0x0290, 0x01e0, 0x0010, 0x0000, // a
    0x1ff0, 0x0120, 0x0210, 0x0210, 0x0210, 0x0120, 0x00c0, 0x0000, // b
    0x00c0, 0x0120, 0x0210, 0x0210, 0x0210, 0x0210, 0x0120, 0x0000, // c
    0x00c0, 0x0120, 0x0210, 0x0210, 0x0210, 0x0120, 0x1ff0, 0x0000, // d
    0x00c0, 0x01a0, 0x0290, 0x0290, 0x0290, 0x0290, 0x0190, 0x0000, // e
    0x0000, 0x0000, 0x0210, 0x0ff0, 0x1210, 0x1000, 0x0800, 0x0000, // f
    0x01a8, 0x0254, 0x0254, 0x0254, 0x0254, 0x0194, 0x0208, 0x0000, // g
    0x1000, 0x1ff0, 0x0100, 0x0200, 0x0200, 0x0200, 0x01f0, 0x0000, // h
    0x0000, 0x0000, 0x0210, 0x13f0, 0x0010, 0x0000, 0x0000, 0x0000, // i
    0x0000, 0x0002, 0x0202, 0x13fc, 0x0000, 0x0000, 0x0000, 0x0000, // j
    0x1010, 0x1ff0, 0x0080, 0x0140, 0x0220, 0x0210, 0x0010, 0x0000, // k
    0x0000, 0x0000, 0x1010, 0x1ff0, 0x0010, 0x0000, 0x0000, 0x0000, // l
    0x03f0, 0x0200, 0x0200, 0x01f0, 0x0200, 0x0200, 0x01f0, 0x0000, // m
    0x0200, 0x03f0, 0x0100, 0x0200, 0x0200, 0x0200, 0x01f0, 0x0000, // n
    0x00c0, 0x0120, 0x0210, 0x0210, 0x0210, 0x0120, 0x00c0, 0x0000, // o
    0x03fe, 0x0120, 0x0210, 0x0210, 0x0210, 0x0120, 0x00c0, 0x0000, // p
    0x00c0, 0x0120, 0x0210, 0x0210, 0x0210, 0x0120, 0x03fe, 0x0000, // q
    0x0200, 0x03f0, 0x0100, 0x0200, 0x0200, 0x0200, 0x0100, 0x0000, // r
    0x0120, 0x0290, 0x0290, 0x0290, 0x0290, 0x0290, 0x0060, 0x0000, // s
    0x0200, 0x0200, 0x1fe0, 0x0210, 0x0210, 0x0210, 0x0000, 0x0000, // t
    0x03e0, 0x0010, 0x0010, 0x0010, 0x0010, 0x03e0, 0x0010, 0x0000, // u
    0x0200, 0x0300, 0x00c0, 0x0030, 0x00c0, 0x0300, 0x0200, 0x0000, // v
    0x03e0, 0x0010, 0x0020, 0x01c0, 0x0020, 0x0010, 0x03e0, 0x0000, // w
    0x0200, 0x0210, 0x0120, 0x00c0, 0x00c0, 0x0120, 0x0210, 0x0000, // x
    0x0382, 0x0044, 0x0028, 0x0010, 0x0020, 0x0040, 0x0380, 0x0000, // y
    0x0310, 0x0230, 0x0250, 0x0290, 0x0310, 0x0230, 0x0000, 0x0000, // z
    0x0010, 0x07e0, 0x0850, 0x0990, 0x0a10, 0x07e0, 0x0800, 0x0000, // 0
    0x0000, 0x0000, 0x0410, 0x0ff0, 0x0010, 0x0000, 0x0000, 0x0000, // 1
    0x0000, 0x0430, 0x0850, 0x0890, 0x0910, 0x0610, 0x0000, 0x0000, // 2
    0x0000, 0x0420, 0x0810, 0x0910, 0x0910, 0x06e0, 0x0000, 0x0000, // 3
    0x0000, 0x0080, 0x0180, 0x0280, 0x0480, 0x0ff0, 0x0080, 0x0000, // 4
    0x0000, 0x0f10, 0x0910, 0x0910, 0x0920, 0x08c0, 0x0000, 0x0000, // 5
    0x0000, 0x03e0, 0x0510, 0x0910, 0x0910, 0x00e0, 0x0000, 0x0000, // 6
    0x0000, 0x0800, 0x0830, 0x08c0, 0x0b00, 0x0c00, 0x0000, 0x0000, // 7
    0x0000, 0x06e0, 0x0910, 0x0910, 0x0910, 0x06e0, 0x0000, 0x0000, // 8
    0x0000, 0x0700, 0x0890, 0x0890, 0x08a0, 0x07c0, 0x0000, 0x0000, // 9
    0x0000, 0x0080, 0x0080, 0x03e0, 0x0080, 0x0080, 0x0000, 0x0000, // +
    0x0000, 0x0080, 0x0080, 0x0080, 0x0080, 0x0080, 0x0000, 0x0000, // -
    0x0000, 0x0240, 0x0180, 0x0660, 0x0180, 0x0240, 0x0000, 0x0000, // *
    0x0010, 0x0020, 0x0040, 0x0080, 0x0100, 0x0200, 0x0400, 0x0000, // /
    0x0000, 0x0000, 0x0000, 0x0000, 0x07e0, 0x0810, 0x1008, 0x0000, // (
    0x1008, 0x0810, 0x07e0, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, // )
    0x0640, 0x0920, 0x0920, 0x1ff0, 0x0920, 0x0920, 0x04c0, 0x0000, // $
    0x0000, 0x0140, 0x0140, 0x0140, 0x0140, 0x0140, 0x0000, 0x0000, // =
    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, // space
    0x0000, 0x0000, 0x0034, 0x0038, 0x0000, 0x0000, 0x0000, 0x0000, // ,
    0x0000, 0x0000, 0x0030, 0x0030, 0x0000, 0x0000, 0x0000, 0x0000, // .
    0x0000, 0x0080, 0x0080, 0x02a0, 0x0080, 0x0080, 0x0000, 0x0000, // divide
    0x0000, 0x0000, 0x0000, 0x0000, 0x1ff8, 0x1008, 0x1008, 0x0000, // [
    0x1008, 0x1008, 0x1ff8, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, // ]
    0x0c20, 0x1240, 0x0c80, 0x0100, 0x0260, 0x0490, 0x0860, 0x0000, // %
    0x0000, 0x0000, 0x0240, 0x0180, 0x0180, 0x0240, 0x0000, 0x0000, // multiply
    0x0080, 0x0140, 0x0220, 0x0770, 0x0140, 0x0140, 0x0140, 0x0000, // assign
    0x0000, 0x0000, 0x0000, 0x1c00, 0x0000, 0x0000, 0x0000, 0x0000, // '
    0x0000, 0x0000, 0x1c00, 0x0000, 0x1c00, 0x0000, 0x0000, 0x0000, // "
    0x0000, 0x0000, 0x0000, 0x1f90, 0x0000, 0x0000, 0x0000, 0x0000, // !
    0x0000, 0x0000, 0x0334, 0x0338, 0x0000, 0x0000, 0x0000, 0x0000, // ;
    0x0000, 0x0080, 0x0140, 0x0220, 0x0410, 0x0000, 0x0000, 0x0000, // <
    0x0000, 0x0000, 0x0410, 0x0220, 0x0140, 0x0080, 0x0000, 0x0000, // >
    0x0004, 0x0004, 0x0004, 0x0004, 0x0004, 0x0004, 0x0004, 0x0004, // _
    0x0000, 0x0c00, 0x1000, 0x10d0, 0x1100, 0x0e00, 0x0000, 0x0000, // ?
    0x1c1c, 0x1224, 0x0948, 0x0490, 0x0220, 0x0140, 0x0080, 0x0000, // arrow
    0x0000, 0x0000, 0x0000, 0x000a, 0x0006, 0x0000, 0x0000, 0x0000, // cedilla
};

/* data for plato font, set 1. */
const unsigned short plato_m1[] = {
    0x0500, 0x0500, 0x1fc0, 0x0500, 0x1fc0, 0x0500, 0x0500, 0x0000, // #
    0x07f0, 0x0900, 0x1100, 0x1100, 0x1100, 0x0900, 0x07f0, 0x0000, // A
    0x1ff0, 0x1210, 0x1210, 0x1210, 0x1210, 0x0e10, 0x01e0, 0x0000, // B
    0x07c0, 0x0820, 0x1010, 0x1010, 0x1010, 0x1010, 0x0820, 0x0000, // C
    0x1ff0, 0x1010, 0x1010, 0x1010, 0x1010, 0x0820, 0x07c0, 0x0000, // D
    0x1ff0, 0x1110, 0x1110, 0x1110, 0x1010, 0x1010, 0x1010, 0x0000, // E
    0x1ff0, 0x1100, 0x1100, 0x1100, 0x1000, 0x1000, 0x1000, 0x0000, // F
    0x07c0, 0x0820, 0x1010, 0x1010, 0x1090, 0x1090, 0x08e0, 0x0000, // G
    0x1ff0, 0x0100, 0x0100, 0x0100, 0x0100, 0x0100, 0x1ff0, 0x0000, // H
    0x0000, 0x1010, 0x1010, 0x1ff0, 0x1010, 0x1010, 0x0000, 0x0000, // I
    0x0020, 0x0010, 0x1010, 0x1010, 0x1fe0, 0x1000, 0x1000, 0x0000, // J
    0x1ff0, 0x0080, 0x0100, 0x0280, 0x0440, 0x0820, 0x1010, 0x0000, // K
    0x1ff0, 0x0010, 0x0010, 0x0010, 0x0010, 0x0010, 0x0010, 0x0000, // L
    0x1ff0, 0x0800, 0x0400, 0x0200, 0x0400, 0x0800, 0x1ff0, 0x0000, // M
    0x1ff0, 0x0800, 0x0600, 0x0100, 0x00c0, 0x0020, 0x1ff0, 0x0000, // N
    0x07c0, 0x0820, 0x1010, 0x1010, 0x1010, 0x0820, 0x07c0, 0x0000, // O
    0x1ff0, 0x1100, 0x1100, 0x1100, 0x1100, 0x1100, 0x0e00, 0x0000, // P
    0x07c0, 0x0820, 0x1010, 0x1018, 0x1014, 0x0824, 0x07c0, 0x0000, // Q
    0x1ff0, 0x1100, 0x1100, 0x1180, 0x1140, 0x1120, 0x0e10, 0x0000, // R
    0x0e20, 0x1110, 0x1110, 0x1110, 0x1110, 0x1110, 0x08e0, 0x0000, // S
    0x1000, 0x1000, 0x1000, 0x1ff0, 0x1000, 0x1000, 0x1000, 0x0000, // T
    0x1fe0, 0x0010, 0x0010, 0x0010, 0x0010, 0x0010, 0x1fe0, 0x0000, // U
    0x1800, 0x0700, 0x00c0, 0x0030, 0x00c0, 0x0700, 0x1800, 0x0000, // V
    0x1fe0, 0x0010, 0x0020, 0x03c0, 0x0020, 0x0010, 0x1fe0, 0x0000, // W
    0x1830, 0x0440, 0x0280, 0x0100, 0x0280, 0x0440, 0x1830, 0x0000, // X
    0x1800, 0x0400, 0x0200, 0x01f0, 0x0200, 0x0400, 0x1800, 0x0000, // Y
    0x1830, 0x1050, 0x1090, 0x1110, 0x1210, 0x1410, 0x1830, 0x0000, // Z
    0x0000, 0x1000, 0x2000, 0x2000, 0x1000, 0x1000, 0x2000, 0x0000, // ~
    0x0000, 0x0000, 0x1000, 0x0000, 0x1000, 0x0000, 0x0000, 0x0000, // dieresis
    0x0000, 0x1000, 0x2000, 0x4000, 0x2000, 0x1000, 0x0000, 0x0000, // circumflex
    0x0000, 0x0000, 0x0000, 0x1000, 0x2000, 0x4000, 0x0000, 0x0000, // acute
    0x0000, 0x4000, 0x2000, 0x1000, 0x0000, 0x0000, 0x0000, 0x0000, // grave
    0x0000, 0x0100, 0x0300, 0x07f0, 0x0300, 0x0100, 0x0000, 0x0000, // uparrow
    0x0080, 0x0080, 0x0080, 0x0080, 0x03e0, 0x01c0, 0x0080, 0x0000, // rtarrow 
    0x0000, 0x0040, 0x0060, 0x07f0, 0x0060, 0x0040, 0x0000, 0x0000, // downarrow
    0x0080, 0x01c0, 0x03e0, 0x0080, 0x0080, 0x0080, 0x0080, 0x0000, // leftarrow
    0x0000, 0x0080, 0x0100, 0x0100, 0x0080, 0x0080, 0x0100, 0x0000, // low tilde
    0x1010, 0x1830, 0x1450, 0x1290, 0x1110, 0x1010, 0x1010, 0x0000, // Sigma
    0x0030, 0x00d0, 0x0310, 0x0c10, 0x0310, 0x00d0, 0x0030, 0x0000, // Delta
    0x0000, 0x0380, 0x0040, 0x0040, 0x0040, 0x0380, 0x0000, 0x0000, // union
    0x0000, 0x01c0, 0x0200, 0x0200, 0x0200, 0x01c0, 0x0000, 0x0000, // intersect
    0x0000, 0x0000, 0x0000, 0x0080, 0x0f78, 0x1004, 0x1004, 0x0000, // {
    0x1004, 0x1004, 0x0f78, 0x0080, 0x0000, 0x0000, 0x0000, 0x0000, // }
    0x00e0, 0x0d10, 0x1310, 0x0c90, 0x0060, 0x0060, 0x0190, 0x0000, // &
    0x0150, 0x0160, 0x0140, 0x01c0, 0x0140, 0x0340, 0x0540, 0x0000, // not equal
    0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, // space
    0x0000, 0x0000, 0x0000, 0x1ff0, 0x0000, 0x0000, 0x0000, 0x0000, // |
    0x0000, 0x0c00, 0x1200, 0x1200, 0x0c00, 0x0000, 0x0000, 0x0000, // degree
    0x0000, 0x02a0, 0x02a0, 0x02a0, 0x02a0, 0x02a0, 0x0000, 0x0000, // equiv
    0x01e0, 0x0210, 0x0210, 0x01a0, 0x0060, 0x0090, 0x0310, 0x0000, // alpha
    0x0002, 0x03fc, 0x0510, 0x0910, 0x0910, 0x0690, 0x0060, 0x0000, // beta
    0x0000, 0x0ce0, 0x1310, 0x1110, 0x0890, 0x0460, 0x0000, 0x0000, // delta
    0x0000, 0x1030, 0x0cc0, 0x0300, 0x00c0, 0x0030, 0x0000, 0x0000, // lambda
    0x0002, 0x0002, 0x03fc, 0x0010, 0x0010, 0x03e0, 0x0010, 0x0000, // mu
    0x0100, 0x0200, 0x03f0, 0x0200, 0x03f0, 0x0200, 0x0400, 0x0000, // pi
    0x0006, 0x0038, 0x00e0, 0x0110, 0x0210, 0x0220, 0x01c0, 0x0000, // rho
    0x00e0, 0x0110, 0x0210, 0x0310, 0x02e0, 0x0200, 0x0200, 0x0000, // sigma
    0x01e0, 0x0210, 0x0010, 0x00e0, 0x0010, 0x0210, 0x01e0, 0x0000, // omega
    0x0220, 0x0220, 0x0520, 0x0520, 0x08a0, 0x08a0, 0x0000, 0x0000, // less/equal
    0x0000, 0x08a0, 0x08a0, 0x0520, 0x0520, 0x0220, 0x0220, 0x0000, // greater/equal
    0x07c0, 0x0920, 0x1110, 0x1110, 0x1110, 0x0920, 0x07c0, 0x0000, // theta
    0x01e0, 0x0210, 0x04c8, 0x0528, 0x05e8, 0x0220, 0x01c0, 0x0000, // @
    0x0400, 0x0200, 0x0100, 0x0080, 0x0040, 0x0020, 0x0010, 0x0000, /* \ */
    0x01e0, 0x0210, 0x0210, 0x01e0, 0x0290, 0x0290, 0x01a0, 0x0000, // oe

    // "Special" character patterns: these are beyond the regular
    // 6-bit character indices, and are used when "special" ASCII
    // mode characters are encountered.  Rather than display them
    // from pieces, we just treat them as additional character
    // patterns.
    0x0000, 0x0080, 0x0140, 0x0220, 0x07f0, 0x0810, 0x1008, 0x0000, // l-embed
    0x1008, 0x0810, 0x07f0, 0x0220, 0x0140, 0x0080, 0x0000, 0x0000, // r-embed
    0x2184, 0x2244, 0x2424, 0x2424, 0x2424, 0x2424, 0x2244, 0x2004, // copyright
    0x0000, 0x03c0, 0x0240, 0x0240, 0x0240, 0x03c0, 0x0000, 0x0000, // box
    0x0080, 0x01c0, 0x03e0, 0x07f0, 0x03e0, 0x01c0, 0x0080, 0x0000, // diamond
    0x0410, 0x0220, 0x0140, 0x0080, 0x0140, 0x0220, 0x0410, 0x0000, // cross
    0x0000, 0x4000, 0x2000, 0x1000, 0x2000, 0x4000, 0x0000, 0x0000, // hacek
    0x0000, 0x0140, 0x0360, 0x07f0, 0x0360, 0x0140, 0x0000, 0x0000, // delim
    0x0000, 0x0180, 0x0240, 0x0240, 0x0180, 0x0000, 0x0000, 0x0000, // dot prod
    0x0000, 0x0000, 0x0000, 0x0002, 0x0004, 0x0008, 0x0000, 0x0000, // cedilla
};


// ----------------------------------------------------------------------------
// event tables and other macros for wxWindows
// ----------------------------------------------------------------------------

// the event tables connect the wxWindows events with the functions (event
// handlers) which process them. It can be also done at run-time, but for the
// simple menu events like this the static method is much simpler.
BEGIN_EVENT_TABLE (PtermFrame, wxFrame)
    EVT_IDLE (PtermFrame::OnIdle)
    EVT_CLOSE (PtermFrame::OnClose)
    EVT_TIMER (Pterm_Timer, PtermFrame::OnTimer)
    EVT_TIMER (Pterm_PasteTimer, PtermFrame::OnPasteTimer)
    EVT_ACTIVATE (PtermFrame::OnActivate)
    EVT_MENU (Pterm_Close, PtermFrame::OnQuit)
#if !defined (__WXMAC__)
    EVT_MENU (Pterm_ToggleMenuBar, PtermFrame::OnToggleMenuBar)
    EVT_MENU (Pterm_ToggleMenuBarView, PtermFrame::OnToggleMenuBar)
#endif
    EVT_MENU (Pterm_ToggleStatusBar, PtermFrame::OnToggleStatusBar)
    EVT_MENU (Pterm_ToggleStatusBarView, PtermFrame::OnToggleStatusBar)
    EVT_MENU (Pterm_Toggle2xMode, PtermFrame::OnToggle2xMode)
    EVT_MENU (Pterm_Toggle2xModeView, PtermFrame::OnToggle2xMode)
    EVT_MENU (Pterm_ToggleStretchMode, PtermFrame::OnToggleStretchMode)
    EVT_MENU (Pterm_ToggleStretchModeView, PtermFrame::OnToggleStretchMode)
    EVT_MENU (Pterm_CopyScreen, PtermFrame::OnCopyScreen)
    EVT_MENU (Pterm_Copy, PtermFrame::OnCopy)
    EVT_MENU (Pterm_Exec, PtermFrame::OnExec)    
    EVT_MENU (Pterm_MailTo, PtermFrame::OnMailTo)    
    EVT_MENU (Pterm_SearchThis, PtermFrame::OnSearchThis)    
    EVT_MENU (Pterm_Macro0, PtermFrame::OnMacro0)    
    EVT_MENU (Pterm_Macro1, PtermFrame::OnMacro1)    
    EVT_MENU (Pterm_Macro2, PtermFrame::OnMacro2)    
    EVT_MENU (Pterm_Macro3, PtermFrame::OnMacro3)    
    EVT_MENU (Pterm_Macro4, PtermFrame::OnMacro4)    
    EVT_MENU (Pterm_Macro5, PtermFrame::OnMacro5)    
    EVT_MENU (Pterm_Macro6, PtermFrame::OnMacro6)    
    EVT_MENU (Pterm_Macro7, PtermFrame::OnMacro7)    
    EVT_MENU (Pterm_Macro8, PtermFrame::OnMacro8)    
    EVT_MENU (Pterm_Macro9, PtermFrame::OnMacro9)    
    EVT_MENU (Pterm_Paste, PtermFrame::OnPaste)
    EVT_MENU (Pterm_PastePrint, PtermFrame::OnPaste)
    EVT_UPDATE_UI (Pterm_Paste, PtermFrame::OnUpdateUIPaste)
    EVT_MENU (Pterm_SaveScreen, PtermFrame::OnSaveScreen)
    EVT_MENU (Pterm_SaveAudio, PtermFrame::OnSaveAudio)
    EVT_MENU (Pterm_Print, PtermFrame::OnPrint)
    EVT_MENU (Pterm_Preview, PtermFrame::OnPrintPreview)
    EVT_MENU (Pterm_Page_Setup, PtermFrame::OnPageSetup)
    EVT_MENU (Pterm_Pref,    PtermFrame::OnPref)
    EVT_MENU (Pterm_FullScreen, PtermFrame::OnFullScreen)
    EVT_MENU (Pterm_FullScreenView, PtermFrame::OnFullScreen)
    EVT_SIZE (PtermFrame::OnResize)
#if defined (__WXMSW__)
    EVT_ICONIZE (PtermFrame::OnIconize)
#endif
    END_EVENT_TABLE ();

BEGIN_EVENT_TABLE (PtermApp, wxApp)
    EVT_MENU (Pterm_Connect, PtermApp::OnConnect)
    EVT_MENU (Pterm_ConnectAgain, PtermApp::OnConnect)
    EVT_MENU (Pterm_Quit,    PtermApp::OnQuit)
    EVT_MENU (Pterm_HelpKeys, PtermApp::OnHelpKeys)
    EVT_MENU (Pterm_About, PtermApp::OnAbout)
    END_EVENT_TABLE ();

// Create a new application object: this macro will allow wxWindows to create
// the application object during program execution (it's better than using a
// static object for many reasons) and also implements the accessor function
// wxGetApp () which will return the reference of the right type (i.e. PtermApp
// and not wxApp)
IMPLEMENT_APP (PtermApp);

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// the application class
// ----------------------------------------------------------------------------

// 'Main program' equivalent: the program execution "starts" here
bool PtermApp::OnInit (void)
{
    int r, g, b, dspi, sz;
    wxString rgb;
    wxString str;
    wxString filename;
    bool skipinit = false;
    const char *s;

    m_testascii = false;
    m_testreq = false;
    m_termType = 0;
    
    ptermApp = this;
    m_firstFrame = m_helpFrame = NULL;
    g_printData = new wxPrintData;
    g_pageSetupData = new wxPageSetupDialogData;
#ifdef DEBUG
    debugF.Open ();
#endif
#if defined (__WXMSW__)
    const int pid = GetCurrentProcessId ();
    g_beep = new wxSound (wxT ("touch-beep.wav"), TRUE);
    if (!g_beep->IsOk ()) printf ("beep load failed\n");
#else
    const int pid = getpid ();
#endif
    
    // File name to use for tracing, if we enable tracing
    sprintf (traceFn, "pterm%d.trc", pid);

    srand (time (NULL)); 
    m_locale.Init (wxLANGUAGE_DEFAULT);
    m_locale.AddCatalog (wxT ("pterm"));

    if (argc > 4)
    {
        printf ("usage: pterm [ hostname [ portnum [ termtype ]]]\n"
                "   or: pterm [ filename.ppf ]\n");
        exit (1);
    }

    m_config = new wxConfig (wxT ("Pterm"));

    //check for PPF file specified on command line
    if (argc > 1)
    {
        str = argv[argc - 1];
        if (str.Right (4).CmpNoCase (wxT (".ppf")) == 0 ||
            str.Right (5).CmpNoCase (wxT (".ppf\"")) == 0)
        {
            wxString filename;
            wxString profile;
            filename.Clear ();
            for (int i=1; i < argc; i++)
            {
                if (i > 1)
                    filename.Append (wxT (" "));
                filename.Append (argv[i]);
            }
            //strip double quotes from ends
            if (filename.Left (1)==wxT ("\"") &&
                filename.Right (1)==wxT ("\""))
                filename.Mid (2, filename.Len () - 2);
            //get just the filename (profile)
            profile = filename.BeforeLast ('.');
#if defined (_WIN32)
                profile = profile.AfterLast ('\\');
#else
                profile = profile.AfterLast ('/');
#endif
            //load profile
            skipinit = ptermApp->LoadProfile (profile, filename);
        }
        else if (str.Right (4).CmpNoCase (wxT (".dat")) == 0 ||
                 str.Right (4).CmpNoCase (wxT (".trc")) == 0)
        {
            // test data file
            m_testdata = fopen (str, "r");
            if (m_testdata == NULL)
            {
                perror ("Error opening test data file");
                exit (99);
            }
            str = argv[1];
            if (argc > 2 && str.Left (2) == wxT ("-a"))
            {
                m_testascii = true;
            }
            m_testreq = true;
        }
    }

    // check if skipping initial read of settings due to successful load of
    // settings from profile filename specified above
    if (!skipinit)
    {
        //notebook
        m_lastTab = m_config->Read (wxT (PREF_LASTTAB), 0L);
        //tab0
        m_config->Read (wxT (PREF_CURPROFILE), &m_curProfile, CURRENT_PROFILE);
        //tab1
        m_config->Read (wxT (PREF_SHELLFIRST), &m_ShellFirst, wxT (""));
        m_connect = (m_config->Read (wxT (PREF_CONNECT), 1) != 0);
        if (argc > 1)
        {
            m_hostName = argv[1];
        }
        else
        {
            m_config->Read (wxT (PREF_HOST), &m_hostName, DEFAULTHOST);
        }
        if (argc > 2)
        {
            argv[2].ToCLong (&m_port);
            if (argc > 3)
            {
                argv[3].ToCLong (&m_termType);
                printf ("terminal type override %ld\n", m_termType);
            }
        }
        else
        {
            m_config->Read (wxT (PREF_PORT), &m_port, DefNiuPort);
        }
        //tab2
        m_showSignon = (m_config->Read (wxT (PREF_SHOWSIGNON), 0L) != 0);
        m_showSysName = (m_config->Read (wxT (PREF_SHOWSYSNAME), 0L) != 0);
        m_showHost = (m_config->Read (wxT (PREF_SHOWHOST), 1) != 0);
        m_showStation = (m_config->Read (wxT (PREF_SHOWSTATION), 1) != 0);
        //tab3
        m_classicSpeed = (m_config->Read (wxT (PREF_1200BAUD), 0L) != 0);
        m_gswEnable = (m_config->Read (wxT (PREF_GSW), 1) != 0);
        m_numpadArrows = (m_config->Read (wxT (PREF_ARROWS), 1) != 0);
        m_ignoreCapLock = (m_config->Read (wxT (PREF_IGNORECAP), 0L) != 0);
        m_platoKb = (m_config->Read (wxT (PREF_PLATOKB), 0L) != 0);
#if defined (__WXMAC__)
        m_useAccel = true;
#else
        m_useAccel = (m_config->Read (wxT (PREF_ACCEL), 0L) != 0);
#endif
        m_beepEnable = (m_config->Read (wxT (PREF_BEEP), 1) != 0);
        m_DisableShiftSpace = (m_config->Read (wxT (PREF_SHIFTSPACE), 0L) != 0);
        m_DisableMouseDrag = (m_config->Read (wxT (PREF_MOUSEDRAG), 0L) != 0);
        //tab4
        m_scale2 = (m_config->Read (wxT (PREF_SCALE), 1) == 2);
        m_stretch = (m_config->Read (wxT (PREF_STRETCH), 0L) != 0);
        m_showStatusBar = (m_config->Read (wxT (PREF_STATUSBAR), 1) != 0);
#if !defined (__WXMAC__)
        m_showMenuBar = (m_config->Read (wxT (PREF_MENUBAR), 1) != 0);
#endif
        m_noColor = (m_config->Read (wxT (PREF_NOCOLOR), 0L) != 0);
        // 255 144 0 is RGB for Plato Orange
        m_config->Read (wxT (PREF_FOREGROUND), &rgb, wxT ("255 144 0"));
        s = rgb.c_str ();
        sscanf (s, "%d %d %d", &r, &g, &b);
        m_fgColor = wxColour (r, g, b);
        m_config->Read (wxT (PREF_BACKGROUND), &rgb, wxT ("0 0 0"));
        s = rgb.c_str ();
        sscanf (s, "%d %d %d", &r, &g, &b);
        m_bgColor = wxColour (r, g, b);
        //tab5
        m_config->Read (wxT (PREF_CHARDELAY), &m_charDelay, PASTE_CHARDELAY);
        m_config->Read (wxT (PREF_LINEDELAY), &m_lineDelay, PASTE_LINEDELAY);
        m_config->Read (wxT (PREF_AUTOLF), &m_autoLF, 0L);
        m_splitWords = (m_config->Read (wxT (PREF_SPLITWORDS), 0L) != 0);
        m_smartPaste = (m_config->Read (wxT (PREF_SMARTPASTE), 0L) != 0);
        m_convDot7 = (m_config->Read (wxT (PREF_CONVDOT7), 0L) != 0);
        m_conv8Sp = (m_config->Read (wxT (PREF_CONV8SP), 0L) != 0);
        m_TutorColor = (m_config->Read (wxT (PREF_TUTORCOLOR), 0L) != 0);
        //tab6
        m_config->Read (wxT (PREF_EMAIL), &m_Email, wxT (""));
        m_config->Read (wxT (PREF_SEARCHURL), &m_SearchURL, DEFAULTSEARCH);
    }

#if PTERM_MDI
    // On Mac, the style rule is that the application keeps running even
    // if all its windows are closed.
//    SetExitOnFrameDelete (false);
    PtermFrameParent = new PtermMainFrame ();
    PtermFrameParent->Show (true);
#endif

    // Pick up the preferred x/y (saved from last time).  Adjust these
    // to make sure that the top left corner of the window is on screen,
    // and second (if possible) that the whole window is.
    m_config->Read (wxT (PREF_XPOS), &prefX, 0L);
    m_config->Read (wxT (PREF_YPOS), &prefY, 0L);
    debug ("saved x/y is %ld, %ld", prefX, prefY);
    dspi = wxDisplay::GetFromPoint (wxPoint (prefX, prefY));
    if (dspi == wxNOT_FOUND)
    {
        dspi = 0;
    }
    
    wxDisplay dsp (dspi);
    wxRect ca = dsp.GetClientArea ();
    
    debug ("display %d client area pos %d, %d size %d, %d",
           dspi, ca.x, ca.y, ca.width, ca.height);
    
    // Adjust to make the bottom right corner be on-screen
    sz = 512 + 2 * DisplayMargin;
    if (m_scale2)
    {
        sz *= 2;
    }
    prefX += sz;
    prefY += sz;
    // width and height have some adjustment added to it to allow for
    // window decoration.  
    if (prefX > ca.x + ca.width - 5)
    {
        prefX = ca.x + ca.width - 5;
    }
    if (prefY > ca.y + ca.height - 25)
    {
        prefY = ca.y + ca.height - 25;
    }
    // From the adjusted bottom right, get the adjusted top left
    prefX -= sz;
    prefY -= sz;
    if (prefX < ca.x)
    {
        prefX = ca.x;
    }
    if (prefY < ca.y)
    {
        prefY = ca.y;
    }
    debug ("adjusted x/y is %ld, %ld", prefX, prefY);
    
    // create the main application window
    // If arguments are present, always connect without asking
    if (!m_testreq && (!DoConnect (!(m_connect || argc > 1))))
    {
        return false;
    }
    
    if (m_testreq)
    {
        char tline[200], *p;
        int w, seq, pseq = -1;
        PtermFrame *frame;
        
        frame = new PtermFrame (str, -1, _("Test execution"));

        if (frame != NULL)
        {
            if (m_firstFrame != NULL)
            {
                m_firstFrame->m_prevFrame = frame;
            }
            frame->m_nextFrame = m_firstFrame;
            frame->tracePterm = true;
            traceFn[0] = '\0';
            traceF.Open ();
            m_firstFrame = frame;
            if (m_testascii)
            {
                printf ("reading test data for ascii mode.,,\n");
                frame->m_dumbTty = false;
            }
            else
            {
                printf ("reading test data for classic mode...\n");
            }
            for (;;)
            {
                tline[0] = '\0';
                fgets (tline, 199, m_testdata);
                if (tline[0] == '\0')
                {
                    fclose (m_testdata);
                    m_testdata = NULL;
                    break;
                }
                p = tline;
                if (p[2] == ':')
                {
                    // Timestamp at start of line, skip it
                    p += 14;
                }
                if (sscanf (p, "%o seq %d", &w, &seq) != 0 &&
                    seq != pseq)
                {
                    // Successful conversion, process the word,
                    // provided it is new (different sequence number
                    // than before)
                    pseq = seq;
                    frame->procPlatoWord (w, m_testascii);
                }
            }
            frame->Refresh ();
            
            printf ("done with test data\n");
            
            m_helpFrame = frame;
        }
    }
    
    // Add some handlers so we can save the screen in various formats
    // Note that the BMP handler is always loaded, don't do it again.
    wxImage::AddHandler (new wxPNGHandler);
    wxImage::AddHandler (new wxPNMHandler);
    wxImage::AddHandler (new wxTIFFHandler);
    wxImage::AddHandler (new wxXPMHandler);

    // success: wxApp::OnRun () will be called which will enter the main message
    // loop and the application will run. If we returned false here, the
    // application would exit immediately.
    return true;
}

int PtermApp::OnExit (void)
{
    wxTheClipboard->Flush ();

    m_config->Write (wxT (PREF_XPOS), prefX);
    m_config->Write (wxT (PREF_YPOS), prefY);
    m_config->Flush ();

    delete g_printData;
    delete g_pageSetupData;

    return 0;
}

void PtermApp::OnConnect (wxCommandEvent &event)
{
    DoConnect (event.GetId () == Pterm_Connect);
}

bool PtermApp::DoConnect (bool ask)
{
    PtermFrame *frame;

    if (ask)
    {
        PtermConnDialog dlg (wxID_ANY, _("Connect to PLATO"),
                             wxDefaultPosition, wxSize (-1, -1)); //450, 355));
    
        dlg.CenterOnScreen ();
        
        if (dlg.ShowModal () == wxID_OK)
        {
            if (dlg.m_ShellFirst.IsEmpty ())
            {
                m_ShellFirst = wxT ("");
            }
            else
            {
                m_ShellFirst = dlg.m_ShellFirst;
            }
            if (dlg.m_host.IsEmpty ())
            {
                m_hostName = DEFAULTHOST;
            }
            else
            {
                m_hostName = dlg.m_host;
            }
            if (dlg.m_port.IsEmpty ())
            {
                m_port = DefNiuPort;
            }
            else
            {
                dlg.m_port.ToCLong (&m_port);
            }
            //save selection to current
            m_config = new wxConfig (wxT ("Pterm"));
            m_config->Write (wxT (PREF_SHELLFIRST), m_ShellFirst);
            m_config->Write (wxT (PREF_CURPROFILE), m_curProfile);
            m_config->Write (wxT (PREF_HOST), m_hostName);
            m_config->Write (wxT (PREF_PORT), m_port);
            m_config->Flush ();
        }
        else
        {
            return false;     // connect canceled
        }
    }
    
    // create the main application window
    frame = new PtermFrame (m_hostName, m_port, wxT ("Pterm"));

    if (frame != NULL)
    {
        if (m_firstFrame != NULL)
        {
            m_firstFrame->m_prevFrame = frame;
        }
        frame->m_nextFrame = m_firstFrame;
        m_firstFrame = frame;
    }
    
    return (frame != NULL);
}

bool PtermApp::LoadProfile (wxString profile, wxString filename)
{
    wxString buffer;
    wxString token;
    wxString value;
    wxString str;
    wxString rgb;
    int r, g, b;
    const char *s;
    
    //open file
    if (filename.IsEmpty ())
        filename = ptermApp->ProfileFileName (profile);
    wxTextFile file (filename);
    if (!file.Open ())
        return false;

    //read file
    for (buffer = file.GetFirstLine (); ; buffer = file.GetNextLine ())
    {
        if (buffer.Contains (wxT ("=")))
        {
            token = buffer.BeforeFirst ('=');
            token.Trim (true);
            token.Trim (false);
            value = buffer.AfterFirst (wxT ('='));
            value.Trim (true);
            value.Trim (false);
            //tab0
            if     (token.Cmp (wxT (PREF_CURPROFILE)) == 0)
                m_curProfile    = profile;
            //tab1
            else if (token.Cmp (wxT (PREF_SHELLFIRST)) == 0)
                m_ShellFirst    = value;
            else if (token.Cmp (wxT (PREF_CONNECT)) == 0)
                m_connect       = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_HOST)) == 0)
                m_hostName      = value;
            else if (token.Cmp (wxT (PREF_PORT)) == 0)
            {
                value.ToCLong (&m_port);
            }
            //tab2
            else if (token.Cmp (wxT (PREF_SHOWSIGNON)) == 0)
                m_showSignon    = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_SHOWSYSNAME)) == 0)
                m_showSysName   = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_SHOWHOST)) == 0)
                m_showHost      = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_SHOWSTATION)) == 0)
                m_showStation   = (value.Cmp (wxT ("1")) == 0);

            //tab3
            else if (token.Cmp (wxT (PREF_1200BAUD)) == 0)
                m_classicSpeed  = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_GSW)) == 0)
                m_gswEnable     = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_ARROWS)) == 0)
                m_numpadArrows  = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_IGNORECAP)) == 0)
                m_ignoreCapLock = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_PLATOKB)) == 0)
                m_platoKb       = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_ACCEL)) == 0)
            {
#if defined (__WXMAC__)
                m_useAccel      = true;
#else
                m_useAccel      = (value.Cmp (wxT ("1")) == 0);
#endif
            }
            else if (token.Cmp (wxT (PREF_BEEP)) == 0)
                m_beepEnable    = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_SHIFTSPACE)) == 0)
                m_DisableShiftSpace = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_MOUSEDRAG)) == 0)
                m_DisableMouseDrag  = (value.Cmp (wxT ("1")) == 0);

            //tab4
            else if (token.Cmp (wxT (PREF_SCALE)) == 0)
                m_scale2        = (value.Cmp (wxT ("2")) == 0);
            else if (token.Cmp (wxT (PREF_STRETCH)) == 0)
                m_stretch       = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_STATUSBAR)) == 0)
                m_showStatusBar = (value.Cmp (wxT ("1")) == 0);
#if !defined (__WXMAC__)
            else if (token.Cmp (wxT (PREF_MENUBAR)) == 0)
                m_showMenuBar   = (value.Cmp (wxT ("1")) == 0);
#endif
            else if (token.Cmp (wxT (PREF_NOCOLOR)) == 0)
                m_noColor       = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_FOREGROUND)) == 0)
            {
                s = value.c_str ();
                sscanf (s, "%d %d %d", &r, &g, &b);
                m_fgColor       = wxColour (r, g, b);
            }
            else if (token.Cmp (wxT (PREF_BACKGROUND)) == 0)
            {
                s = value.c_str ();
                sscanf (s, "%d %d %d", &r, &g, &b);
                m_bgColor       = wxColour (r, g, b);
            }
            //tab5
            else if (token.Cmp (wxT (PREF_CHARDELAY)) == 0)
                value.ToCLong (&m_charDelay);
            else if (token.Cmp (wxT (PREF_LINEDELAY)) == 0)
                value.ToCLong (&m_lineDelay);
            else if (token.Cmp (wxT (PREF_AUTOLF)) == 0)
                value.ToCLong (&m_autoLF);
            else if (token.Cmp (wxT (PREF_SPLITWORDS)) == 0)
                m_splitWords    = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_SMARTPASTE)) == 0)
                m_smartPaste    = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_CONVDOT7)) == 0)
                m_convDot7      = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_CONV8SP)) == 0)
                m_conv8Sp       = (value.Cmp (wxT ("1")) == 0);
            else if (token.Cmp (wxT (PREF_TUTORCOLOR)) == 0)
                m_TutorColor    = (value.Cmp (wxT ("1")) == 0);

            //tab6
            else if (token.Cmp (wxT (PREF_EMAIL)) == 0)
                m_Email         = value;
            else if (token.Cmp (wxT (PREF_SEARCHURL)) == 0)
                m_SearchURL     = value;
        }
        if (file.Eof ())
        {
            break;
        }
    }
    file.Close ();

    //write prefs
    m_config = new wxConfig (wxT ("Pterm"));
    //tab0
    m_config->Write (wxT (PREF_CURPROFILE), profile);
    //tab1
    m_config->Write (wxT (PREF_SHOWSIGNON), (m_showSignon) ? 1 : 0);
    m_config->Write (wxT (PREF_SHOWSYSNAME), (m_showSysName) ? 1 : 0);
    m_config->Write (wxT (PREF_SHOWHOST), (m_showHost) ? 1 : 0);
    m_config->Write (wxT (PREF_SHOWSTATION), (m_showStation) ? 1 : 0);
    //tab2
    m_config->Write (wxT (PREF_1200BAUD), (m_classicSpeed) ? 1 : 0);
    m_config->Write (wxT (PREF_GSW), (m_gswEnable) ? 1 : 0);
    m_config->Write (wxT (PREF_ARROWS), (m_numpadArrows) ? 1 : 0);
    m_config->Write (wxT (PREF_IGNORECAP), (m_ignoreCapLock) ? 1 : 0);
    m_config->Write (wxT (PREF_PLATOKB), (m_platoKb) ? 1 : 0);
    m_config->Write (wxT (PREF_ACCEL), (m_useAccel) ? 1 : 0);
    m_config->Write (wxT (PREF_BEEP), (m_beepEnable) ? 1 : 0);
    m_config->Write (wxT (PREF_SHIFTSPACE), (m_DisableShiftSpace) ? 1 : 0);
    m_config->Write (wxT (PREF_MOUSEDRAG), (m_DisableMouseDrag) ? 1 : 0);
    //tab3
    m_config->Write (wxT (PREF_CONNECT), (m_connect) ? 1 : 0);
    m_config->Write (wxT (PREF_HOST), m_hostName);
    m_config->Write (wxT (PREF_PORT), m_port);
    //tab4
    m_config->Write (wxT (PREF_SCALE), (m_scale2) ? 2 : 1);
    m_config->Write (wxT (PREF_STRETCH), (m_stretch) ? 1 : 0);
    m_config->Write (wxT (PREF_STATUSBAR), (m_showStatusBar) ? 1 : 0);
#if !defined (__WXMAC__)
    m_config->Write (wxT (PREF_MENUBAR), (m_showMenuBar) ? 1 : 0);
#endif
    m_config->Write (wxT (PREF_NOCOLOR), (m_noColor) ? 1 : 0);
    rgb.Printf (wxT ("%d %d %d"), m_fgColor.Red (), m_fgColor.Green (),
                m_fgColor.Blue ());
    m_config->Write (wxT (PREF_FOREGROUND), rgb);
    rgb.Printf (wxT ("%d %d %d"), m_bgColor.Red (), m_bgColor.Green (),
                m_bgColor.Blue ());
    m_config->Write (wxT (PREF_BACKGROUND), rgb);
    //tab5
    m_config->Write (wxT (PREF_CHARDELAY), m_charDelay);
    m_config->Write (wxT (PREF_LINEDELAY), m_lineDelay);
    m_config->Write (wxT (PREF_AUTOLF), m_autoLF);
    m_config->Write (wxT (PREF_SPLITWORDS), (m_splitWords) ? 1 : 0);
    m_config->Write (wxT (PREF_SMARTPASTE), (m_smartPaste) ? 1 : 0);
    m_config->Write (wxT (PREF_CONVDOT7), (m_convDot7) ? 1 : 0);
    m_config->Write (wxT (PREF_CONV8SP), (m_conv8Sp) ? 1 : 0);
    m_config->Write (wxT (PREF_TUTORCOLOR), (m_TutorColor) ? 1 : 0);
    //tab6
    m_config->Write (wxT (PREF_EMAIL), m_Email);
    m_config->Write (wxT (PREF_SEARCHURL), m_SearchURL);
    m_config->Flush ();

    return true;
}

wxString PtermApp::ProfileFileName (wxString profile)
{
    wxString filename;
    filename = wxGetCwd ();
    if (filename.Right (1) != wxT ("/") && filename.Right (1) != wxT ("\\"))
    {
#if defined (_WIN32)
        filename.Append (wxT ("\\"));
#else
        filename.Append (wxT ("/"));
#endif
    }
    filename.Append (profile);
    filename.Append (wxT (".ppf"));
    return filename;
}

void PtermApp::OnAbout (wxCommandEvent&)
{
    wxMessageBox (wxT (STRPRODUCTNAME) L" V" wxT (STRFILEVER)
                      L"\n  built with wxWidgets V" wxT (WXVERSION)
                      L"\n  build date " wxT (PTERMBUILDDATE)
#ifdef PTERMSVNREV
                      L"\n  SVN revision " wxT (PTERMSVNREV)
#endif
                      L"\n" wxT (STRLEGALCOPYRIGHT),
                      _(L"About Pterm"), wxOK | wxICON_INFORMATION, NULL);
}

void PtermApp::OnHelpKeys (wxCommandEvent &)
{
    PtermFrame *frame;
    wxString str;
    unsigned int i;


    if (m_helpFrame == NULL)
    {
        // If there isn't one yet, create a help window -- same as a
        // regular frame except that the data comes from here, not
        // from a connection.
        frame = new PtermFrame (str, -1, _("Keyboard Help"));

        if (frame != NULL)
        {
            if (m_firstFrame != NULL)
            {
                m_firstFrame->m_prevFrame = frame;
            }
            frame->m_nextFrame = m_firstFrame;
            m_firstFrame = frame;
            for (i = 0;
                 i < sizeof (keyboardhelp) / sizeof (keyboardhelp[0]);
                 i++)
            {
                frame->procPlatoWord (keyboardhelp[i], false);
            }
            m_helpFrame = frame;
            frame->Refresh ();
        }
    }
    else
    {
        m_helpFrame->Show (true);
        m_helpFrame->Raise ();
    }
}

wxColour PtermApp::SelectColor (wxWindow &parent, 
                                 const wxChar *title, wxColour &initcol)
{
    wxColour col (initcol);
    wxColour orange (255, 144, 0);
    wxColour vikingGreen (0, 220, 0);
    wxColourData data;

    data.SetColour (initcol);
    data.SetCustomColour (0, orange);
    data.SetCustomColour (1, *wxBLACK);
    data.SetCustomColour (2, vikingGreen);
    
    wxColourDialog dialog (&parent, &data);

    dialog.CentreOnParent ();
    dialog.SetTitle (title);

    if (dialog.ShowModal () == wxID_OK)
    {
        col = dialog.GetColourData ().GetColour ();
    }

    return col;
}

void PtermApp::OnQuit (wxCommandEvent&)
{
    PtermFrame *frame, *nextframe;

    frame = m_firstFrame;
    while (frame != NULL)
    {
        nextframe = frame->m_nextFrame;
        frame->Close (true);
        frame = nextframe;
    }
#if PTERM_MDI // defined (__WXMAC__)
    // On the Mac, deleting all the windows doesn't terminate the
    // program, so we make it stop this way.
    ExitMainLoop ();
#endif
}


#if PTERM_MDI
// MDI parent frame
PtermMainFrame::PtermMainFrame (void)
    : wxMDIParentFrame (NULL, wxID_ANY, wxT ("Pterm"),
                        wxDefaultPosition, wxDefaultSize, 0)
{
    // create a menu bar

    menuFile = new wxMenu;
    menuFile->Append (Pterm_Connect, _("New Connection...")
                      ACCELERATOR ("\tCtrl-N"), _("Connect to a PLATO host"));
    menuFile->Append (Pterm_Pref, _("Preferences...")
                      MACACCEL ("\tCtrl-,"),
                      _("Set program configuration"));
    menuFile->AppendSeparator ();
    menuFile->Append (Pterm_Quit, _("Exit"), _("Quit this program"));

    // the "About" item should be in the help menu.
    // Well, on the Mac it actually doesn't show up there, but for that magic
    // to work it has to be presented to wx in the help menu.  So the help
    // menu ends up empty.  Sigh.
    menuHelp = new wxMenu;

    menuHelp->Append (Pterm_About, _("About Pterm"), _("Show about dialog"));
    menuHelp->Append (Pterm_HelpKeys, _("Pterm keyboard"),
                      _("Show keyboard description"));
    
    // now append the freshly created menu to the menu bar...
    menuBar = new wxMenuBar ();
    menuBar->Append (menuFile, _("File"));
    menuBar->Append (menuHelp, _("Help"));

    // ... and attach this menu bar to the frame
#if !defined (__WXMAC__)
    if (!m_fullScreen && ptermApp->m_showMenuBar)
#endif
        SetMenuBar (menuBar);
}
#endif

// ----------------------------------------------------------------------------
// main frame
// ----------------------------------------------------------------------------

// frame constructor
PtermFrame::PtermFrame (wxString &host, int port, const wxString& title)
    : PtermFrameBase (PtermFrameParent, -1, title, 
                      wxPoint (ptermApp->prefX, ptermApp->prefY),
                      wxDefaultSize),
      // m_memDC not initialized
      tracePterm (false),
      m_currentWord (0),
      m_nextFrame (NULL),
      m_prevFrame (NULL),
      m_pendingEcho (-1),
      m_bCancelPaste (false),
      m_bPasteActive (false),
      m_conn (NULL),
      m_dumbTty (true),
      // menuBar not initialized
      // menuFile not initialized
      // menuEdit not initialized
      // menuView not initialized
      // menuHelp not initialized
      // menuPopup not initialized
      m_statusBar (NULL),
      m_scale2 (ptermApp->m_scale2),
      m_stretch (ptermApp->m_stretch),
      m_fullScreen (false),
      m_xscale ((m_scale2) ? 2 : 1),
      m_yscale ((m_scale2) ? 2 : 1),
      // m_font not initialized
      m_usefont (false),
      // m_fontfamily not initialized
      // m_fontface not initialized
      // m_fontsize not initialized
      // m_fontitalic not initialized
      // m_fontbold not initialized
      // m_fontstrike not initialized
      // m_fontunderln not initialized
      // m_fontwidth not initialized
      // m_fontheight not initialized
      m_fontPMD (false),
      m_fontinfo (false),
      m_osinfo (false),
      // m_gswFile not initialized
      m_gswFFmt (0),
      // m_fgpix not initialized
      // m_bgpix not initialized
      // m_bitmap not initialized
	  m_canvas (NULL),
      // m_curProfile not initialized
      // m_ShellFirst not initialized
      // m_hostName not initialized
      m_port (port),
      m_timer (this, Pterm_Timer),
      m_station (0),
      // m_rect not initialized
      m_pasteTimer (this, Pterm_PasteTimer),
      // m_pasteText not initialized
      m_pasteIndex (-1),
      // m_pasteNextIndex not initialized
      // m_pasteLen not initialized
      // m_pastePrint not initialized
      // m_pasteLinePos not initialized
      m_nextword (0),
      m_delay (0),
      modexor (false),
      currentX (0),
      currentY (496),
      memaddr (0),
      // plato_m23 not initialized
      memlpc (0),
      wc (0),
      seq (0),
      modewords (0),
      m_ascState (none),
      m_ascBytes (0),
      // m_assembler not initialized
      lastX (0),
      lastY (0),
      m_flowCtrl (false),
      m_sendFgt (false),
      m_defFg (ptermApp->m_fgColor),
      m_defBg (ptermApp->m_bgColor),
      m_currentFg (ptermApp->m_fgColor),
      m_currentBg (ptermApp->m_bgColor),
      // m_loadingPMD not initialized
      // m_PMD not initialized
      cwsmode (0),
      cwsfun (0),
      cwscnt (0),
      cwswin (0),
      // cwswindow not initialized
      m_regionX (0),
      m_regionY (0),
      m_regionHeight (0),
      m_regionWidth (0)
{
    int i;

    m_hostName = host;
    mode = 017;             // default to character mode, rewrite
    modexor = false;
    setMargin (0);
    RAM[M_CCR] = 0;
    
    // Set default character set origins (for PPT that is; ASCII is different)
    RAM[C2ORIGIN] = M2ADDR & 0xff;
    RAM[C2ORIGIN + 1] = M2ADDR >> 8;
    RAM[C3ORIGIN] = M3ADDR & 0xff;
    RAM[C3ORIGIN + 1] = M3ADDR >> 8;
    
    // set the frame icon
    SetIcon (wxICON (pterm_32));


#if  defined (__WXGTK20__)
    // A rather ugly hack here.  GTK V2 insists that F10 should be the
    // accelerator for the menu bar.  We don't want that.  There is
    // no sane way to turn this off, but we *can* get the same effect
    // by setting the "menu bar accelerator" property to the name of a
    // function key that is apparently legal, but doesn't really exist.
    // (Or if it does, it certainly isn't a key we use.)
    gtk_settings_set_string_property (gtk_settings_get_default (),
                                      "gtk-menu-bar-accel", "F15", "foo");

#endif

    // create a menu bar
    BuildMenuBar ();

    // Build the popup menu
    BuildPopupMenu (port);

    if (port > 0)
    {
        // If we're supposed to run a shell program, do so; then wait 2 seconds
        if (!ptermApp->m_ShellFirst.IsEmpty ())
        {
            if (wxExecute (ptermApp->m_ShellFirst)!=0)
            {
#if defined (_WIN32)
                Sleep (2000);
#else
                sleep (2);
#endif
            }
        }
        SetCursor (*wxHOURGLASS_CURSOR);
    }
    
    // Status bar always exists, but we may or may not display it
    BuildStatusBar (true);

    m_bitmap = new wxBitmap (512, 512, 32);
    {
         PixelData pixmap (*m_bitmap);

         if (!pixmap)
         {
             // ... raw access to bitmap data unavailable, do something else ...
             printf ("no raw bitmap access\n");
             exit (1);
         }

         // Find out how RGBA map to the parts of a 32 bit value, so we can access
         // things as 32 bit arrays and still be implementation independent.
         PixelData::Iterator p (pixmap);
         u32 *pmap = (u32 *) (p.m_ptr);
         u8 *pb = (u8 *) pmap;
         u32 t;
    
         t = *pmap;
         *pmap = 0;
         p.Alpha () = 255;
         m_maxalpha = *pmap;
         *pmap = 0;
         p.Red () = 1;
         for (i = 0; i < 4; i++)
         {
             if (pb[i]) m_red = i;
         }
         *pmap = 0;
         p.Green () = 1;
         for (i = 0; i < 4; i++)
         {
             if (pb[i]) m_green = i;
         }
         *pmap = 0;
         p.Blue () = 1;
         for (i = 0; i < 4; i++)
         {
             if (pb[i]) m_blue = i;
         }
         // Pixel value for selected text is white on 
         // translucent gray background.
         p.Alpha () = 255;
         p.Red () = p.Green () = p.Blue () = 255;
         m_selpixf = *pmap;
         p.Alpha () = 140;
         m_selpixb = *pmap;
         *pmap = t;
    }
    m_memDC = new wxMemoryDC ();
    m_selmap = new wxBitmap (512, 512, 32);
    
    m_canvas = new PtermCanvas (this);

    SetColors (m_currentFg, m_currentBg);    
    SetClientSize (XSize, YSize);
    ptermFullErase ();
    UpdateDisplayState ();

    if (port > 0)
    {
        // Create and start the network processing thread
        tracex ("Connecting to: %s:%d", host.mb_str ().data (), port);
        m_conn = new PtermConnection (this, host, port);
    }
    Show (true);
}

void PtermFrame::trace (const wxString &msg) const
{
    wxString hdr;
    
    if (traceF.Active ())
    {
        hdr.Printf ("%07o seq %6d wc %3d ", m_currentWord, seq, wc);
        hdr.Append (msg);
        traceF.Log (hdr);
    }
}

void PtermFrame::trace (const char *fmt, ...) const
{
    va_list v;
    wxString msg;
    
    if (traceF.Active ())
    {
        va_start (v, fmt);
        msg.PrintfV (fmt, v);
        va_end (v);
        trace (msg);
    }
}

PtermFrame::~PtermFrame ()
{
    if (m_conn != NULL)
    {
        delete m_conn;
    }
    delete m_bitmap;

    // Remove this frame from the app's frame list
    if (m_nextFrame != NULL)
    {
        m_nextFrame->m_prevFrame = m_prevFrame;
    }
    if (m_prevFrame != NULL)
    {
        m_prevFrame->m_nextFrame = m_nextFrame;
    }
    else
    {
        ptermApp->m_firstFrame = m_nextFrame;
    }
}

void PtermFrame::BuildMenuBar (void)
{
    menuBar = new wxMenuBar ();

    // First build the menus:
    // File menu options
    BuildFileMenu (m_port);

    //edit menu options
    BuildEditMenu (m_port);

    //view menu options
    BuildViewMenu (m_port);

    //help menu options
    BuildHelpMenu ();
    
    // Now put them on the bar
    menuBar->Append (menuFile, _("File"));
    menuBar->Append (menuEdit, _("Edit"));
    menuBar->Append (menuView, _("View"));
    menuBar->Append (menuHelp, _("Help"));

#if !defined (__WXMAC__)
    if (!m_fullScreen && ptermApp->m_showMenuBar)
#endif
        SetMenuBar (menuBar);
}

// NOTE: Accelerators are actually Command-xxx on the Mac
// on other platforms they are controlled by a prefs setting.

void PtermFrame::BuildFileMenu (int port)
{
    menuFile = new wxMenu;
    menuFile->Append (Pterm_Connect, _("New Connection...")
                      ACCELERATOR ("\tCtrl-N"), _("Connect to a PLATO host"));
    if (port > 0)
    {
        // No "connect again" for the help window because that
        // doesn't own a connection.
        menuFile->Append (Pterm_ConnectAgain, _("Connect Again"),
                          _("Connect to the same host"));
        menuFile->AppendSeparator ();
    }
    menuFile->Append (Pterm_SaveScreen, _("Save Screen")
                      ACCELERATOR ("\tCtrl-S"), _("Save screen image to file"));
    if (port > 0)
    {
        // Save audio is disabled until we know that we have a classic connection
        menuFile->Append (Pterm_SaveAudio, _("Save GSW Audio"),
                          _("Save GSW sound to audio file"));
        menuFile->Enable (Pterm_SaveAudio, false);
    }
    menuFile->Append (Pterm_Print, _("Print...")
                      ACCELERATOR ("\tCtrl-P"), _("Print screen content"));
    menuFile->Append (Pterm_Page_Setup, _("Page Setup..."),
                      _("Printout page setup"));
    menuFile->Append (Pterm_Preview, _("Print Preview"),
                      _("Preview screen print"));
    menuFile->AppendSeparator ();
    menuFile->Append (Pterm_Pref, _("Preferences...")
                      MACACCEL ("\tCtrl-,"),
                      _("Set program configuration"));
    menuFile->AppendSeparator ();
    menuFile->Append (Pterm_Close, _("Close") ACCELERATOR ("\tCtrl-W"),
                      _("Close this window"));
    menuFile->Append (Pterm_Quit, _("Exit"), _("Quit this program"));
}

void PtermFrame::BuildEditMenu (int port)
{
    menuEdit = new wxMenu;
    menuEdit->Append (Pterm_CopyScreen, _("Copy Screen"),
                      _("Copy screen to clipboard"));
    menuEdit->Append (Pterm_Copy, _("Copy text") ACCELERATOR ("\tCtrl-C"),
                      _("Copy text only to clipboard"));
    if (port > 0)
    {
        menuEdit->Append (Pterm_Paste, _("Paste plain text")
                          ACCELERATOR ("\tCtrl-V"), _("Paste plain text"));
        // No "paste" for help window because it doesn't do input.
        menuEdit->Append (Pterm_PastePrint, _("Paste Printout"),
                          _("Paste Cyber printout format"));
    }
    menuEdit->AppendSeparator ();                   
    menuEdit->Append (Pterm_Exec, _("Execute URL") ACCELERATOR ("\tCtrl-X"),
                      _("Execute URL"));
    menuEdit->Append (Pterm_MailTo, _("Mail to...") ACCELERATOR ("\tCtrl-M"),
                      _("Mail to..."));
    menuEdit->Append (Pterm_SearchThis, _("Search this...")
                      ACCELERATOR ("\tCtrl-G"),
                      _("Search this...")); // g=google this?
    if (ptermApp->m_TutorColor && port > 0)
    {
        menuEdit->AppendSeparator ();
        menuEdit->Append (Pterm_Macro0, wxT ("Box 8x")
                          ACCELERATOR ("\tCtrl-0"), wxT ("Box 8x"));
        menuEdit->Append (Pterm_Macro1, wxT ("<c,zc.errf>")
                          ACCELERATOR ("\tCtrl-1"), wxT ("<c,zc.errf>"));
        menuEdit->Append (Pterm_Macro2, wxT ("<c,zc.info>")
                          ACCELERATOR ("\tCtrl-2"), wxT ("<c,zc.info>"));
        menuEdit->Append (Pterm_Macro3, wxT ("<c,zc.keys>")
                          ACCELERATOR ("\tCtrl-3"), wxT ("<c,zc.keys>"));
        menuEdit->Append (Pterm_Macro4, wxT ("<c,zc.text>")
                          ACCELERATOR ("\tCtrl-4"), wxT ("<c,zc.text>"));
        menuEdit->Append (Pterm_Macro5, wxT ("color zc.errf")
                          ACCELERATOR ("\tCtrl-5"), wxT ("color zc.errf"));
        menuEdit->Append (Pterm_Macro6, wxT ("color zc.info")
                          ACCELERATOR ("\tCtrl-6"), wxT ("color zc.info"));
        menuEdit->Append (Pterm_Macro7, wxT ("color zc.keys")
                          ACCELERATOR ("\tCtrl-7"), wxT ("color zc.keys"));
        menuEdit->Append (Pterm_Macro8, wxT ("color zc.text")
                          ACCELERATOR ("\tCtrl-8"), wxT ("color zc.text"));
        menuEdit->Append (Pterm_Macro9, wxT ("Menu colorization")
                          ACCELERATOR ("\tCtrl-9"), wxT ("Menu colorization"));
    }
    // screen-region related options are disabled until a region is selected
    menuEdit->Enable (Pterm_Copy, false);
    menuEdit->Enable (Pterm_Exec, false);
    menuEdit->Enable (Pterm_MailTo, false);
    menuEdit->Enable (Pterm_SearchThis, false);
}

void PtermFrame::BuildViewMenu (int port)
{
    menuView = new wxMenu;
#if !defined (__WXMAC__)
    menuView->AppendCheckItem (Pterm_ToggleMenuBarView,
                               _("Display menu bar"), _("Display menu bar"));
    menuView->Check (Pterm_ToggleMenuBarView, ptermApp->m_showMenuBar);
#endif
    if (port > 0)
    {
        menuView->AppendCheckItem (Pterm_ToggleStatusBarView,
                                   _("Display status bar"),
                                   _("Display status bar"));
        menuView->Check (Pterm_ToggleStatusBarView, ptermApp->m_showStatusBar);
    }
    menuView->AppendCheckItem (Pterm_Toggle2xModeView, _("Zoom display 200%"),
                               _("Zoom display 200%"));
    menuView->Check (Pterm_Toggle2xModeView, (ptermApp->m_scale2));
    menuView->AppendCheckItem (Pterm_ToggleStretchModeView,
                               _("Stretch display"), _("Stretch display"));
    menuView->Check (Pterm_ToggleStretchModeView, ptermApp->m_stretch);
    menuView->AppendSeparator ();
    menuView->Append (Pterm_FullScreenView, _("Full Screen")
                      ACCELERATOR ("\tCtrl-U"),
                      _("Display in full screen mode"));
}

void PtermFrame::BuildHelpMenu (void)
{
    // the "About" item should be in the help menu.
    // Well, on the Mac it actually doesn't show up there, but for that magic
    // to work it has to be presented to wx in the help menu.  So the help
    // menu ends up empty.  Sigh.
    menuHelp = new wxMenu;
    menuHelp->Append (Pterm_About, _("About Pterm"), _("Show about dialog"));
    menuHelp->Append (Pterm_HelpKeys, _("Pterm keyboard"),
                      _("Show keyboard description"));
}

void PtermFrame::BuildPopupMenu (int port)
{
    menuPopup = new wxMenu;
    menuPopup->Append (Pterm_CopyScreen, _("Copy Screen"),
                       _("Copy screen to clipboard"));
    menuPopup->Append (Pterm_Copy, _("Copy text") ACCELERATOR ("\tCtrl-C"),
                       _("Copy text only to clipboard"));
    if (port > 0)
    {
        menuPopup->Append (Pterm_Paste, _("Paste ASCII")
                           ACCELERATOR ("\tCtrl-V"), _("Paste plain text"));
        // No "paste" for help window because it doesn't do input.
        menuPopup->Append (Pterm_PastePrint, _("Paste Printout"),
                           _("Paste Cyber printout format"));
    }
    menuPopup->AppendSeparator ();                  
    menuPopup->Append (Pterm_Exec, _("Execute URL") ACCELERATOR ("\tCtrl-X"),
                       _("Execute URL"));
    menuPopup->Append (Pterm_MailTo, _("Mail to...") ACCELERATOR ("\tCtrl-M"),
                       _("Mail to..."));
    menuPopup->Append (Pterm_SearchThis, _("Search this...")
                       ACCELERATOR ("\tCtrl-G"),
                       _("Search this...")); // g=google this?
    menuPopup->AppendSeparator ();
#if !defined (__WXMAC__)
    menuPopup->AppendCheckItem (Pterm_ToggleMenuBar, _("Display menu bar"),
                                _("Display menu bar"));
    menuPopup->Check (Pterm_ToggleMenuBar, ptermApp->m_showMenuBar);
#endif
    if (port > 0)
    {
        menuPopup->AppendCheckItem (Pterm_ToggleStatusBar,
                                    _("Display status bar"),
                                    _("Display status bar"));
        menuPopup->Check (Pterm_ToggleStatusBar, ptermApp->m_showStatusBar);
    }
    menuPopup->AppendCheckItem (Pterm_Toggle2xMode, _("Zoom display 200%"),
                                _("Zoom display 200%"));
    menuPopup->Check (Pterm_Toggle2xMode, (ptermApp->m_scale2));
    menuPopup->AppendCheckItem (Pterm_ToggleStretchMode, _("Stretch display"),
                                _("Stretch display"));
    menuPopup->Check (Pterm_ToggleStretchMode, ptermApp->m_stretch);
    menuPopup->AppendSeparator ();
    menuPopup->AppendCheckItem (Pterm_FullScreen, _("Full Screen")
                                ACCELERATOR ("\tCtrl-U"),
                                _("Display full screen"));
    if (ptermApp->m_TutorColor && port > 0)
    {
        menuPopup->AppendSeparator ();
        menuPopup->Append (Pterm_Macro0, wxT ("Box 8x")
                           ACCELERATOR ("\tCtrl-0"), wxT ("Box 8x"));
        menuPopup->Append (Pterm_Macro1, wxT ("<c,zc.errf>")
                           ACCELERATOR ("\tCtrl-1"), wxT ("<c,zc.errf>"));
        menuPopup->Append (Pterm_Macro2, wxT ("<c,zc.info>")
                           ACCELERATOR ("\tCtrl-2"), wxT ("<c,zc.info>"));
        menuPopup->Append (Pterm_Macro3, wxT ("<c,zc.keys>")
                           ACCELERATOR ("\tCtrl-3"), wxT ("<c,zc.keys>"));
        menuPopup->Append (Pterm_Macro4, wxT ("<c,zc.text>")
                           ACCELERATOR ("\tCtrl-4"), wxT ("<c,zc.text>"));
        menuPopup->Append (Pterm_Macro5, wxT ("color zc.errf")
                           ACCELERATOR ("\tCtrl-5"), wxT ("color zc.errf"));
        menuPopup->Append (Pterm_Macro6, wxT ("color zc.info")
                           ACCELERATOR ("\tCtrl-6"), wxT ("color zc.info"));
        menuPopup->Append (Pterm_Macro7, wxT ("color zc.keys")
                           ACCELERATOR ("\tCtrl-7"), wxT ("color zc.keys"));
        menuPopup->Append (Pterm_Macro8, wxT ("color zc.text")
                           ACCELERATOR ("\tCtrl-8"), wxT ("color zc.text"));
        menuPopup->Append (Pterm_Macro9, wxT ("Menu colorization")
                           ACCELERATOR ("\tCtrl-9"), wxT ("Menu colorization"));
    }
    // screen-region related options are disabled until a region is selected
    menuPopup->Enable (Pterm_Copy, false);
    menuPopup->Enable (Pterm_Exec, false);
    menuPopup->Enable (Pterm_MailTo, false);
    menuPopup->Enable (Pterm_SearchThis, false);
}

// For some strange reason it seems to be necessary to delete 
// the statusbar (not just unlink it) if we don't want to see it.
// If it exists as an object but isn't tied to the window, it displays
// at the top of the window !
void PtermFrame::BuildStatusBar (bool connecting)
{
    const bool showstatusbar = (!m_fullScreen &&
                                ptermApp->m_showStatusBar &&
                                m_port > 0);
    const bool havestatusbar = (m_statusBar != NULL);
	int ww,wh,ow,oh,nw,nh;

	// get window parameters before changing things
	GetSize(&ww, &wh);
	GetClientSize (&ow, &oh);
    
    if (!showstatusbar && havestatusbar)
    {
        // We have one right now but don't want it
        SetStatusBar (NULL);
        delete m_statusBar;
        m_statusBar = NULL;
        debug ("Turning off status bar");
    }
    else if (showstatusbar && !havestatusbar)
    {
        // We don't have one, but want one
        m_statusBar = new wxStatusBar (this, wxID_ANY);
        m_statusBar->SetFieldsCount (STATUSPANES);

        if (connecting)
        {
            m_statusBar->SetStatusText (_(" Connecting..."), STATUS_CONN);
        }
        else if (m_conn == NULL)
        {
            m_statusBar->SetStatusText (_(" Not connected"), STATUS_CONN);
        }
        else
        {
            ptermSetStation (m_station, true, true);
        }
        ptermShowTrace ();
        SetStatusBar (m_statusBar);
        debug ("Turning on status bar");
    }

	//check if size changed
	GetClientSize (&nw, &nh);
	SetSize (ww, wh - (nh - oh));
}

// event handlers

void PtermFrame::OnIdle (wxIdleEvent& event)
{
    int word;
    bool refresh = false;
    
    // In every case, let others see this event too.
    event.Skip ();

    // Do nothing for the help window or other connection-less windows
    // If our timer is running, we're using the timer event to drive
    // the display, so ignore idle events.
    if (m_conn == NULL || m_timer.IsRunning ())
    {
        return;
    }

    if (m_nextword != 0)
    {
        refresh |= procPlatoWord (m_nextword, m_conn->Ascii ());
        m_nextword = 0;
    }

    for (;;)
    {
        /*
        **  Process words until there is nothing left.
        */
        word = m_conn->NextWord ();
    
        if (word == C_NODATA || word == C_CONNFAIL)
        {
            break;
        }

        // See if we're supposed to delay (but it's not an internal
        // code such as NODATA)
        if ((int) word >= 0 && (word >> 19) != 0)
        {
            m_delay = word >> 19;
            m_nextword = word & 01777777;
            if (m_conn->Ascii ())
            {
                m_timer.Start (8);  // 16.67 / 2.2, rounded
            }
            else
            {
                m_timer.Start (17);
            }
            
            if (refresh)
            {
                Refresh ();
            }

            return;
        }
        
        debug ("processing data from plato %07o", word);
        refresh |= procPlatoWord (word, m_conn->Ascii ());
    }

    if (refresh)
    {
        Refresh ();
    }
    
    switch (word)
    {
        case C_NODATA:
            break;
        case C_CONNFAIL:
            // restart the network processing thread
            if (m_port > 0)
            {
                m_conn = new PtermConnection (this, m_hostName, m_port);
            }
            break;
        default:
            event.RequestMore ();
    }
}

void PtermFrame::OnTimer (wxTimerEvent &)
{
    int word;
    bool refresh = false;
    
    if (--m_delay > 0)
    {
        return;
    }

    debug ("processing data from plato %07o", m_nextword);
    refresh = procPlatoWord (m_nextword, m_conn->Ascii ());
    
    // See what's next.  If no delay is called for, just process it, keep
    // going until no more data or we get a word with delay.
    for (;;)
    {
        word = m_conn->NextWord ();
    
        if (word == C_NODATA)
        {
            m_nextword = 0;
            m_timer.Stop ();

            break;
        }
        m_delay = (word >> 19);
        m_nextword = word & 01777777;
        if (m_delay != 0)
        {
            break;
        }
        debug ("processing data from plato %07o", word);
        refresh |= procPlatoWord (word, m_conn->Ascii ());
    }

    if (refresh)
    {
        Refresh ();
    }
}

// This method is invoked by the paste character pacing timer, once
// for each character to be pasted.  It sends the key code(s) for
// the character and re-arms the timer, if more needs to be done.
void PtermFrame::OnPasteTimer (wxTimerEvent &)
{
    wxChar c = 0, c2 = 0;
    u32 p;
    int pp, i, nextindex, nextpos;
    int delay = 0;

    if (m_bCancelPaste || m_pasteIndex < 0 || m_pasteIndex >= m_pasteLen)
    {
        //reset flags
        m_bCancelPaste = false;
        m_bPasteActive = false;
        return;
    }

    // Before doing the next character, see if we're supposed to be
    // doing auto-newline and we haven't found the break point yet.
    // Note that auto-newline interacts poorly with tab conversion, because
    // it works by looking ahead some number of characters in the 
    // paste string.
    if (m_pasteNextIndex == 0)
    {
        i = m_pasteText.find_last_of (" -", m_pasteIndex + ptermApp->m_autoLF - 1);
        if (i < m_pasteIndex || i == wxString::npos)
        {
            // If there is no good break point, break at the limit.
            i = m_pasteNextIndex + ptermApp->m_autoLF - 1;
        }
        m_pasteNextIndex = i;
    }
    
    nextindex = m_pasteIndex;
    // Pick up the next two characters, advancing the index past the
    // first one only.  Sometimes we need the extra character lookahead.
    c = m_pasteText[nextindex++];
    if (nextindex < m_pasteLen)
    {
        c2 = m_pasteText[nextindex];
    }
    
    // Check if we're at the line break position, and we have a space.
    // If yes, remove it (the newline will be inserted at the end)
    if (m_pasteIndex == m_pasteNextIndex && c == ' ')
    {
        c = '\0';
    }
    
    if (m_pastePrint)
    {
        // Pasting a printout string, with ' for shift and other fun stuff.
        if (c < sizeof (printoutToPlato) / sizeof (printoutToPlato[0]))
        {
            pp = printoutToPlato[c];

            if (pp == -2)
            {
                // Shift code.  Look up the next character.
                pp = printoutToPlato[c2];
                // Look for a shift code preceding a character that is
                // a shifted character (like $), or an unshifted
                // character whose shifted form corresponds to a
                // different printable character (like 4).  For those
                // cases, if we see a shift code, send that
                // separately.  For example, '4 does not mean $, it
                // means a shift code then 4 (which is an embedded
                // mode change).  Note that we also do this if the 
                // character after the ' is not recognized.
                if ((pp & 040) != 0 || c2 == '=' ||
                    (c2 >= '0' && c2 <= '9'))
                {
                    ptermSendKey1 (055);     // shift-Assign is shift code
                }
                else
                {
                    // Regular shifted key, send that and skip
                    ptermSendKey1 (pp | 040);
                    nextindex++;
                }
            }
            else if (pp != -1)
            {
                ptermSendKey1 (pp);
            }
        }
    }
    else
    {
        // Regular paste.  This is a lot harder because we have to
        // translate Unicode characters to PLATO.

        // By default, each character pasted is taken to use one
        // display position.  This is how we do tab handling and
        // auto-newline.
        nextpos = m_pasteLinePos + 1;
        
        // Smart paste processing.  This recognizes spaces that go to 
        // the next multiple of 8 tab stop, and the sequences <( and )>
        // for embed open and close.
        if (ptermApp->m_smartPaste)
        {
            if (c == '<' && c2 == '(')
            {
                // open embed
                c = L'\u2993';
                nextindex++;
            }
            else if (c == ')' && c2 == '>')
            {
                // close embed
                c = L'\u2994';
                nextindex++;
            }
            else if (c == ' ' && c2 == ' ')
            {
                // We only substitute tab for at least two spaces.
                // We have two now; see if there are enough additional
                // ones to get to the next tab stop.
                const int ts = ((m_pasteLinePos + 8) & (~7)) - m_pasteLinePos;
                if (ts == 2)
                {
                    // Just those two gets us to the tabstop.  Do it.
                    c = '\t';
                    nextindex++;
                    nextpos++;
                }
                else if (m_pasteIndex + ts < m_pasteLen)
                {
                    // Need more, and we're not at end of string.  Check
                    // for more spaces.
                    for (i = 2; i < ts; i++)
                    {
                        if (m_pasteText[m_pasteIndex + i] != ' ')
                        {
                            break;
                        }
                    }
                    if (i == ts)
                    {
                        // Spaces all the way to the next tab.
                        c = '\t';
                        nextindex = m_pasteIndex + ts;
                        nextpos = m_pasteLinePos + ts;
                    }
                }
            }
        }
        
        // One more special case to handle: A with ring can appear either
        // as combined or as separated.  It's a single character in PLATO,
        // represented in the unicodeToPlato table by its combined encoding.
        // If we see the separate code, form the combined.
        if (c2 == L'\u030a')
        {
            // Found "combining ring above" after character c.  The only
            // ones we handle now are a and A, so check.
            if (c == 'a')
            {
                c = L'\u00e5';
                nextindex++;
            }
            else if (c == 'A')
            {
                c = L'\u00c5';
                nextindex++;
            }
        }

        // Combining accents don't take up a column
        if (c >= L'\u0300' && c <= L'\u033f')
        {
            nextpos = m_pasteLinePos;
        }

        if (c < (sizeof (asciiToPlato) / sizeof (asciiToPlato[0])))
        {
            p = pastedAsciiToPlato[c];
        }
        else
        {
            // Out of ASCII range.  Look for a match in the Unicode table.
            p = None;
            for (i = 0;
                 i < sizeof (unicodeToPlato) / sizeof (unicodeToPlato[0]);
                 i++)
            {
                if (unicodeToPlato[i].u == c)
                {
                    p = unicodeToPlato[i].p;
                    break;
                }
            }
        }
        
        // Send what we found, if anything.  Note that we don't change
        // the current position if we don't send a key.
        if (p != None)
        {
            ptermSendKey (p);
            m_pasteLinePos = nextpos;
        }
    }

    if (c == '\n' && ptermApp->m_splitWords && ptermApp->m_autoLF != 0)
    {
        // Newline and splitting lines, next time we'll find the next
        // break point.
        m_pasteNextIndex = 0;
    }
    else if (m_pasteIndex == m_pasteNextIndex)
    {
        // This is the spot where we want to break.  Send a NEXT,
        // then skip spaces.
        ptermSendKey1 (026);
        while (nextindex < m_pasteLen && m_pasteText[nextindex] == ' ')
        {
            nextindex++;
        }
    }
    
    if (nextindex < m_pasteLen)
    {
        // Still more to do.  Update the index (which is cleared to -1
        // by ptermSendKey), then restart the timer with the
        // appropriate delay (char delay or line delay).
        m_pasteIndex = nextindex;
        if (c == '\n')
        {
            delay = ptermApp->m_lineDelay;
            m_pasteLinePos = 0;
        }
        else
        {
            delay = ptermApp->m_charDelay;
            m_pasteLinePos++;
        }
        m_pasteTimer.Start (delay, true);
        m_bPasteActive = true;
    }
    else
    {
        // reset flags
        m_bCancelPaste = false;
        m_bPasteActive = false;
    }
}

void PtermFrame::OnClose (wxCloseEvent &)
{
    int x, y;
    
    if (m_conn != NULL)
    {
        delete m_conn;
        m_conn = NULL;
    }

    if (this == ptermApp->m_helpFrame)
    {
        ptermApp->m_helpFrame = NULL;
    }

    // Save the position of this window as our preferred position
    GetPosition (&x, &y);
    ptermApp->prefX = x;
    ptermApp->prefY = y;
    debug ("Window position on exit is %d, %d", x, y);

    if (m_nextFrame != NULL && IsActive ())
    {
        m_nextFrame->Raise ();
    }
    else if (m_prevFrame != NULL && IsActive ())
    {
        m_prevFrame->Raise ();
    }
    
    Destroy ();
}

void PtermFrame::OnQuit (wxCommandEvent&)
{
    // true is to force the frame to close
    Close (true);
}

void PtermFrame::OnActivate (wxActivateEvent &event)
{
    if (m_canvas != NULL)
    {
        m_canvas->SetFocus ();
    }
    event.Skip ();        // let others see the event, too
}

void PtermFrame::OnCopyScreen (wxCommandEvent &)
{
    wxBitmapDataObject *screen;

    screen = new wxBitmapDataObject (*m_bitmap);

    if (wxTheClipboard->Open ())
    {
        if (!wxTheClipboard->SetData (screen))
        {
            wxLogError (_("Can't copy image to the clipboard"));
        }
        wxTheClipboard->Close ();
    }
    else
    {
        wxLogError (_("Can't open clipboard."));
    }
}

#if !defined (__WXMAC__)
void PtermFrame::OnToggleMenuBar (wxCommandEvent &)
{
    //toggle menu
    ptermApp->m_showMenuBar = !ptermApp->m_showMenuBar;
    SavePreferences ();
    if (!ptermApp->m_showMenuBar)
    {
        SetMenuBar (NULL);
    }
    else
    {
        SetMenuBar (menuBar);
    }
    UpdateDisplayState ();
}
#endif

void PtermFrame::OnToggleStatusBar (wxCommandEvent &)
{
    //toggle status
    ptermApp->m_showStatusBar = !ptermApp->m_showStatusBar;
    SavePreferences ();

    BuildStatusBar ();

    UpdateDisplayState ();
}

void PtermFrame::OnToggle2xMode (wxCommandEvent &)
{
    //toggle status
    m_scale2 = ptermApp->m_scale2 = !ptermApp->m_scale2;
    SavePreferences ();

    UpdateDisplayState ();
}

void PtermFrame::OnToggleStretchMode (wxCommandEvent &)
{

    //toggle status
    m_stretch = ptermApp->m_stretch = !ptermApp->m_stretch;
    SavePreferences ();

    //refit
    UpdateDisplayState ();

}
void PtermFrame::OnCopy (wxCommandEvent &event)
{
    wxString text = GetRegionText ();
    
    if (!wxTheClipboard->Open ())
    {
        wxLogError (_("Can't open clipboard."));

        return;
    }

    if (!wxTheClipboard->SetData (new wxTextDataObject (text)))
    {
        wxLogError (_("Can't copy text to the clipboard"));
    }

    wxTheClipboard->Close ();
}

void PtermFrame::OnExec (wxCommandEvent &event)
{
    wxString url = GetRegionText (true);

    wxLaunchDefaultBrowser (url, wxBROWSER_NEW_WINDOW | wxBROWSER_NOBUSYCURSOR);
}

void PtermFrame::OnMailTo (wxCommandEvent &event)
{
    wxString l_Email;
    wxString l_FixText;
    wxString newchr;
    wxString pnt;
    int cnt;

    for (pnt = GetRegionText (), cnt = 0; pnt[cnt]; cnt++)
    {
        if (pnt[cnt] == '/')
            newchr = wxChar ('@');
        //fix ' at ' , '(at)', and '[at]'
        else if (pnt[cnt] == ' ' && pnt[cnt+1] == 'a' &&
                 pnt[cnt+2] == 't' && pnt[cnt+3] == ' ')
            cnt += 3, newchr = wxChar ('@');
        else if (pnt[cnt] == '(' && pnt[cnt+1] == 'a' &&
                 pnt[cnt+2] == 't' && pnt[cnt+3] == ')')
            cnt += 3, newchr = wxChar ('@');
        else if (pnt[cnt] == '[' && pnt[cnt+1] == 'a' &&
                 pnt[cnt+2] == 't' && pnt[cnt+3] == ']')
            cnt += 3, newchr = wxChar ('@');
        //fix ' dot ' , '(dot)', and '[dot]'
        else if (pnt[cnt] == ' ' && pnt[cnt+1] == 'd' && 
                 pnt[cnt+2] == 'o' && pnt[cnt+3] == 't' &&
                 pnt[cnt+4] == ' ')
            cnt += 4, newchr = wxChar ('.');
        else if (pnt[cnt] == '(' && pnt[cnt+1] == 'd' &&
                 pnt[cnt+2] == 'o' && pnt[cnt+3] == 't' &&
                 pnt[cnt+4] == ')')
            cnt += 4, newchr = wxChar ('.');
        else if (pnt[cnt] == '[' && pnt[cnt+1] == 'd' &&
                 pnt[cnt+2] == 'o' && pnt[cnt+3] == 't' &&
                 pnt[cnt+4] == ']')
            cnt += 4, newchr = wxChar ('.');
        //strip 'nospam'
        else if (pnt[cnt] == 'n' && pnt[cnt+1] == 'o' &&
                 pnt[cnt+2] == 's' && pnt[cnt+3] == 'p' &&
                 pnt[cnt+4] == 'a' && pnt[cnt+5] == 'm')
            cnt += 5, newchr = wxChar ('*');
        //fix comma
        else if (pnt[cnt] == ',')
            newchr = wxChar ('.');
        else
            newchr = pnt[cnt];
        if (newchr != '*')
            l_FixText += newchr;
    }
    l_Email.Printf (ptermApp->m_Email, l_FixText);
    wxExecute (l_Email);
}

void PtermFrame::OnSearchThis (wxCommandEvent &event)
{
    wxString text = GetRegionText (true);

    debug (ptermApp->m_SearchURL + text);
    wxLaunchDefaultBrowser (ptermApp->m_SearchURL + text,
                            wxBROWSER_NEW_WINDOW | wxBROWSER_NOBUSYCURSOR);
}

void PtermFrame::OnMacro0 (wxCommandEvent &event)
{
    static const int key[] = {0034, 0034, 0034, 0034, 0034, 0034, 0034,
                              0034, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro1 (wxCommandEvent &event)
{
    static const int key[] = {0024, 0000, 0103, 0137, 0132, 0103, 0136, 
                              0105, 0122, 0122, 0106, 0024, 0001, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro2 (wxCommandEvent &event)
{
    static const int key[] = {0024, 0000, 0103, 0137, 0132, 0103, 0136,
                              0111, 0116, 0106, 0117, 0024, 0001, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro3 (wxCommandEvent &event)
{
    static const int key[] = {0024, 0000, 0103, 0137, 0132, 0103, 0136,
                              0113, 0105, 0131, 0123, 0024, 0001, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro4 (wxCommandEvent &event)
{
    static const int key[] = {0024, 0000, 0103, 0137, 0132, 0103, 0136,
                              0124, 0105, 0130, 0124, 0024, 0001, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro5 (wxCommandEvent &event)
{
    static const int key[] = {0103, 0117, 0114, 0117, 0122, 0100, 0100,
                              0100, 0104, 0111, 0123, 0120, 0114, 0101,
                              0131, 0134, 0132, 0103, 0136, 0105, 0122,
                              0122, 0106, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro6 (wxCommandEvent &event)
{
    static const int key[] = {0103, 0117, 0114, 0117, 0122, 0100, 0100,
                              0100, 0104, 0111, 0123, 0120, 0114, 0101,
                              0131, 0134, 0132, 0103, 0136, 0111, 0116,
                              0106, 0117, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro7 (wxCommandEvent &event)
{
    static const int key[] = {0103, 0117, 0114, 0117, 0122, 0100, 0100,
                              0100, 0104, 0111, 0123, 0120, 0114, 0101,
                              0131, 0134, 0132, 0103, 0136, 0113, 0105,
                              0131, 0123, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro8 (wxCommandEvent &event)
{
    static const int key[] = {0103, 0117, 0114, 0117, 0122, 0100, 0100,
                              0100, 0104, 0111, 0123, 0120, 0114, 0101,
                              0131, 0134, 0132, 0103, 0136, 0124, 0105,
                              0130, 0124, -1};
    ptermSendKeys (key);
}
void PtermFrame::OnMacro9 (wxCommandEvent &event)
{
    // 8boxes, <c,zc.keys> 2 boxes <c,zc.text>
    static const int key0[] = {0034, 0034, 0034, 0034, 0034, 0034, 0034,
                               0034, -1};
    ptermSendKeys (key0);
    static const int key3[] = {0024, 0000, 0103, 0137, 0132, 0103, 0136,
                               0113, 0105, 0131, 0123, 0024, 0001, -1};
    ptermSendKeys (key3);
    static const int key[] = {0034, 0034, -1};
    ptermSendKeys (key);
    static const int key4[] = {0024, 0000, 0103, 0137, 0132, 0103, 0136,
                               0124, 0105, 0130, 0124, 0024, 0001, -1};
    ptermSendKeys (key4);
}

void PtermFrame::OnPaste (wxCommandEvent &event)
{
    if (!wxTheClipboard->Open ())
    {
        wxLogError (_("Can't open clipboard."));

        return;
    }

    if (!wxTheClipboard->IsSupported (wxDF_TEXT))
    {
        wxLogWarning (_("No text data on clipboard"));

        wxTheClipboard->Close ();
        return;
    }

    wxTextDataObject text;

    if (!wxTheClipboard->GetData (text))
    {
        wxLogError (_("Can't paste data from the clipboard"));
    }
    else
    {
        m_pasteText = text.GetText ();
        m_pasteLen = m_pasteText.Len ();
        m_pasteIndex = 0;
        if (ptermApp->m_splitWords && ptermApp->m_autoLF != 0)
        {
            m_pasteNextIndex = 0;
        }
        else
        {
            m_pasteNextIndex = -1;  // "don't do anything"
        }
        m_pasteTimer.Start (ptermApp->m_charDelay, true);
        m_pastePrint = (event.GetId () == Pterm_PastePrint);
        m_pasteLinePos = 0;
    }

    wxTheClipboard->Close ();
}

void PtermFrame::OnUpdateUIPaste (wxUpdateUIEvent& event)
{
#ifdef __WXDEBUG__
    // too many trace messages if we don't do it - this function is called
    // very often
    wxLogNull nolog;
#endif

    event.Enable (wxTheClipboard->IsSupported (wxDF_TEXT));
}

void PtermFrame::OnSaveScreen (wxCommandEvent &)
{
    wxMemoryDC screenDC;
    wxString filename, ext;
    wxBitmapType type;
    wxFileDialog fd (this, _("Save screen to"), ptermApp->m_defDir,
                     wxT (""),
                     wxT ("PNG files (*.png)|*.png|")
                     wxT ("TIF files (*.tif)|*.tif|")
                     wxT ("BMP files (*.bmp)|*.bmp|")
                     wxT ("PNM files (*.pnm)|*.pnm|")
                     wxT ("XPM files (*.xpm)|*.xpm"),
                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
    int idx;
    // This list must match order and content of the filter list above
    static const wxChar *exts[] = { wxT ("png"), wxT ("tif"),
                                    wxT ("bmp"), wxT ("pnm"),
                                    wxT ("xpm") };
    
    if (fd.ShowModal () != wxID_OK)
    {
        return;
    }
    filename = fd.GetPath ();
    idx = fd.GetFilterIndex ();

    wxImage screenImage = m_bitmap->ConvertToImage ();
    wxFileName fn (filename);
    wxString filt_ext (exts[idx]);
    
    ptermApp->m_defDir = fn.GetPath ();
    ext = fn.GetExt ();
    if (! (ext.CmpNoCase (filt_ext) == 0 ||
           (idx == 1 && ext.CmpNoCase (wxT ("tiff")) == 0)))
    {
        // Filename extension doesn't match the selected format, fix that.
        // Note that we accept "tiff" as alternate for "tif" (filter index 1).
        ext = filt_ext;
        fn.SetFullName (fn.GetFullName () + wxT (".") + ext);
        filename = fn.GetFullPath ();
    }
    if (ext.CmpNoCase (wxT ("bmp")) == 0)
    {
        type = wxBITMAP_TYPE_BMP;
    }
    else if (ext.CmpNoCase (wxT ("png")) == 0)
    {
        type = wxBITMAP_TYPE_PNG;
    }
    else if (ext.CmpNoCase (wxT ("pnm")) == 0)
    {
        type = wxBITMAP_TYPE_PNM;
    }
    else if (ext.CmpNoCase (wxT ("tif")) == 0 ||
             ext.CmpNoCase (wxT ("tiff")) == 0)
    {
        type = wxBITMAP_TYPE_TIF;
        // 32773 is PACKBITS -- not referenced symbolically because tiff.h
        // isn't necessarily anywhere, for some reason.
        screenImage.SetOption (wxIMAGE_OPTION_COMPRESSION, 32773);
    }
    else if (ext.CmpNoCase (wxT ("xpm")) == 0)
    {
        type = wxBITMAP_TYPE_XPM;
    }
    else
    {
        screenImage.SaveFile (filename);
        return;
    }
    
    screenImage.SaveFile (filename, type);
}

void PtermFrame::OnSaveAudio (wxCommandEvent &)
{
    wxString filename, ext;
    wxFileDialog fd (this, _("Save GSW audio to"), ptermApp->m_defDir,
                     wxT (""), 
                     wxT ("WAV files (*.wav)|*.wav|")
                     wxT ("AIFF files (*.aiff)|*.aiff|")
                     wxT ("AU files (*.au)|*.au"),
                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
    int idx, type;
    // This list must match order and content of the filter list above
    static const wxChar *exts[] = { wxT ("wav"), wxT ("aiff"),
                                    wxT ("au") };
    
    if (fd.ShowModal () != wxID_OK)
    {
        return;
    }
    filename = fd.GetPath ();
    idx = fd.GetFilterIndex ();

    wxFileName fn (filename);
    wxString filt_ext (exts[idx]);
    
    ptermApp->m_defDir = fn.GetPath ();
    ext = fn.GetExt ();
    if (ext.CmpNoCase (filt_ext) != 0)
    {
        // Filename extension doesn't match the selected format, fix that.
        ext = filt_ext;
        fn.SetFullName (fn.GetFullName () + wxT (".") + ext);
        filename = fn.GetFullPath ();
    }
    if (ext.CmpNoCase (wxT ("wav")) == 0)
    {
        type = SF_FORMAT_WAV;
    }
    else if (ext.CmpNoCase (wxT ("aiff")) == 0)
    {
        type = SF_FORMAT_AIFF;
    }
    else
    {
        type = SF_FORMAT_AU;
    }

    m_gswFFmt = type;
    m_gswFile = fn.GetFullPath ();
}

void PtermFrame::OnPrint (wxCommandEvent &)
{
    wxPrintDialogData printDialogData (*g_printData);

    printDialogData.EnableSelection (false);
    printDialogData.EnablePageNumbers (false);
    
    wxPrinter printer (& printDialogData);
    
    PtermPrintout printout (this);
    if (!printer.Print (this, &printout, true /*prompt*/))
    {
        if (wxPrinter::GetLastError () == wxPRINTER_ERROR)
            wxMessageBox (_("There was a problem printing.\nPerhaps your current printer is not set correctly?"), _("Printing"), wxOK);
    }
    else
    {
        (*g_printData) = printer.GetPrintDialogData ().GetPrintData ();
    }
}

void PtermFrame::OnPrintPreview (wxCommandEvent &)
{
    // Pass two printout objects: for preview, and possible printing.
    wxPrintDialogData printDialogData (*g_printData);
    wxPrintPreview *preview = new wxPrintPreview (new PtermPrintout (this),
                                                  new PtermPrintout (this),
                                                  &printDialogData);

    printDialogData.EnableSelection (false);
    printDialogData.EnablePageNumbers (false);
    
    if (!preview->Ok ())
    {
        delete preview;
        wxMessageBox (_("There was a problem previewing.\nPerhaps your current printer is not set correctly?"), _("Previewing"), wxOK);
        return;
    }

    wxPreviewFrame *frame = new wxPreviewFrame (preview, this,
                                                _("Pterm Print Preview"),
                                                wxPoint (100, 100),
                                                wxSize (600, 650));
    frame->Centre (wxBOTH);
    frame->Initialize ();
    frame->Show ();
}

void PtermFrame::OnPageSetup (wxCommandEvent &)
{
    (*g_pageSetupData) = *g_printData;

    wxPageSetupDialog pageSetupDialog (this, g_pageSetupData);
    pageSetupDialog.ShowModal ();

    (*g_printData) = pageSetupDialog.GetPageSetupData ().GetPrintData ();
    (*g_pageSetupData) = pageSetupDialog.GetPageSetupData ();

}

void PtermFrame::OnPref (wxCommandEvent&)
{
    //show dialog
    PtermPrefDialog dlg (NULL, wxID_ANY, _("Pterm Preferences"),
                         wxDefaultPosition, wxSize (451, 400));
    
    //process changes if OK clicked
    if (dlg.ShowModal () == wxID_OK)
    {
        SetColors (dlg.m_fgColor, dlg.m_bgColor);
        //get prefs
        ptermApp->m_lastTab = dlg.m_lastTab;
        //tab0
        ptermApp->m_curProfile = dlg.m_curProfile;
        //tab1
        ptermApp->m_ShellFirst = dlg.m_ShellFirst;
        ptermApp->m_connect = dlg.m_connect;
        ptermApp->m_hostName = dlg.m_host;
        ptermApp->m_port = dlg.m_port;
        //tab2
        ptermApp->m_showSignon = dlg.m_showSignon;
        ptermApp->m_showSysName = dlg.m_showSysName;
        ptermApp->m_showHost = dlg.m_showHost;
        ptermApp->m_showStation = dlg.m_showStation;
        //tab3
        ptermApp->m_classicSpeed = dlg.m_classicSpeed;
        ptermApp->m_gswEnable = dlg.m_gswEnable;
        ptermApp->m_numpadArrows = dlg.m_numpadArrows;
        ptermApp->m_ignoreCapLock = dlg.m_ignoreCapLock;
        ptermApp->m_platoKb = dlg.m_platoKb;
        ptermApp->m_useAccel = dlg.m_useAccel;
        ptermApp->m_beepEnable = dlg.m_beepEnable;
        ptermApp->m_DisableShiftSpace = dlg.m_DisableShiftSpace;
        ptermApp->m_DisableMouseDrag = dlg.m_DisableMouseDrag;
        //tab4
        ptermApp->m_noColor = dlg.m_noColor;
        ptermApp->m_fgColor = dlg.m_fgColor;
        ptermApp->m_bgColor = dlg.m_bgColor;
        //tab5
        ptermApp->m_charDelay = dlg.m_charDelay;
        ptermApp->m_lineDelay = dlg.m_lineDelay;
        ptermApp->m_autoLF = dlg.m_autoLF;
        ptermApp->m_splitWords = dlg.m_splitWords;
        ptermApp->m_smartPaste = dlg.m_smartPaste;
        ptermApp->m_convDot7 = dlg.m_convDot7;
        ptermApp->m_conv8Sp = dlg.m_conv8Sp;
        ptermApp->m_TutorColor = dlg.m_TutorColor;
        //tab6
        ptermApp->m_Email = dlg.m_Email;
        ptermApp->m_SearchURL = dlg.m_SearchURL;

        //update settings to config
        SavePreferences ();

        //update the display according to what we did
        UpdateDisplayState ();
    }
}

void PtermFrame::SavePreferences (void)
{
    wxString rgb;
    //write prefs
    ptermApp->m_config->Write (wxT (PREF_LASTTAB), ptermApp->m_lastTab);
    //tab0
    ptermApp->m_config->Write (wxT (PREF_CURPROFILE), ptermApp->m_curProfile);
    //tab1
    ptermApp->m_config->Write (wxT (PREF_SHELLFIRST), ptermApp->m_ShellFirst);
    ptermApp->m_config->Write (wxT (PREF_CONNECT), (ptermApp->m_connect) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_HOST), m_hostName);
    ptermApp->m_config->Write (wxT (PREF_PORT), ptermApp->m_port);
    //tab2
    ptermApp->m_config->Write (wxT (PREF_SHOWSIGNON), (ptermApp->m_showSignon) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_SHOWSYSNAME), (ptermApp->m_showSysName) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_SHOWHOST), (ptermApp->m_showHost) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_SHOWSTATION), (ptermApp->m_showStation) ? 1 : 0);
    //tab3
    ptermApp->m_config->Write (wxT (PREF_1200BAUD), (ptermApp->m_classicSpeed) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_GSW), (ptermApp->m_gswEnable) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_ARROWS), (ptermApp->m_numpadArrows) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_IGNORECAP), (ptermApp->m_ignoreCapLock) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_PLATOKB), (ptermApp->m_platoKb) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_ACCEL), (ptermApp->m_useAccel) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_BEEP), (ptermApp->m_beepEnable) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_SHIFTSPACE), (ptermApp->m_DisableShiftSpace) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_MOUSEDRAG), (ptermApp->m_DisableMouseDrag) ? 1 : 0);
    //tab4
    ptermApp->m_config->Write (wxT (PREF_SCALE), (ptermApp->m_scale2) ? 2 : 1);
    ptermApp->m_config->Write (wxT (PREF_STRETCH), (ptermApp->m_stretch) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_STATUSBAR), (ptermApp->m_showStatusBar) ? 1 : 0);
#if !defined (__WXMAC__)
    ptermApp->m_config->Write (wxT (PREF_MENUBAR), (ptermApp->m_showMenuBar) ? 1 : 0);
#endif
    ptermApp->m_config->Write (wxT (PREF_NOCOLOR), (ptermApp->m_noColor) ? 1 : 0);
    rgb.Printf (wxT ("%d %d %d"), ptermApp->m_fgColor.Red (), ptermApp->m_fgColor.Green (), ptermApp->m_fgColor.Blue ());
    ptermApp->m_config->Write (wxT (PREF_FOREGROUND), rgb);
    rgb.Printf (wxT ("%d %d %d"), ptermApp->m_bgColor.Red (), ptermApp->m_bgColor.Green (), ptermApp->m_bgColor.Blue ());
    ptermApp->m_config->Write (wxT (PREF_BACKGROUND), rgb);
    //tab5
    ptermApp->m_config->Write (wxT (PREF_CHARDELAY), ptermApp->m_charDelay);
    ptermApp->m_config->Write (wxT (PREF_LINEDELAY), ptermApp->m_lineDelay);
    ptermApp->m_config->Write (wxT (PREF_AUTOLF), ptermApp->m_autoLF);
    ptermApp->m_config->Write (wxT (PREF_SPLITWORDS), (ptermApp->m_splitWords) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_SMARTPASTE), (ptermApp->m_smartPaste) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_CONVDOT7), (ptermApp->m_convDot7) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_CONV8SP), (ptermApp->m_conv8Sp) ? 1 : 0);
    ptermApp->m_config->Write (wxT (PREF_TUTORCOLOR), (ptermApp->m_TutorColor) ? 1 : 0);
    //tab6
    ptermApp->m_config->Write (wxT (PREF_EMAIL), ptermApp->m_Email);
    ptermApp->m_config->Write (wxT (PREF_SEARCHURL), ptermApp->m_SearchURL);
    ptermApp->m_config->Flush ();
}

void PtermFrame::OnFullScreen (wxCommandEvent &)
{
    m_fullScreen = !m_fullScreen;
    
    // If we're switching to full screen mode, save the current
    // window position and size
    if (m_fullScreen)
    {
        m_rect = GetRect ();
    }
    
    // Tell wxWidgets the desired full screen mode.
	ShowFullScreen (m_fullScreen);

    // If we're exiting full screen mode, restore the window position
    // and size
    if (!m_fullScreen)
    {
        SetSize (m_rect);
    }
    
    UpdateDisplayState ();
}

void PtermFrame::OnResize (wxSizeEvent& event)
{
    UpdateDisplayState ();
}

// This method recalculates all the display control variables: full screen
// mode, scale values, virtual size, etc., based on the current display
// mode flags and the window or display size.
//
// It also enables or disables display of the status bar and menu bar,
// according to the current preferences.  Note that the objects for
// these two always exist, but they are or are not attached to the frame,
// depending on the preference currently in effect.
void PtermFrame::UpdateDisplayState (void)
{
    int h, w;
    
    if (m_canvas == NULL)
    {
        return;
    }

    if (m_fullScreen)
    {
        wxDisplay d (wxDisplay::GetFromWindow (this));
        wxRect r = d.GetGeometry ();
        
        w = r.width;
        h = r.height;
    }
    else
    {
        GetClientSize (&w, &h);
    }

    m_canvas->SetSize (w, h);
    
    // If stretching, the client and virtual sizes are the same, and
    // we scale the image to match.  Otherwise, the scaling is 2x or 1x
    // and the client size is 512 * the scale, plus margins.
    if (m_stretch)
    {
		m_xscale = (w - 2 * DisplayMargin) / 512.0;
		m_yscale = (h - 2 * DisplayMargin) / 512.0;
        m_xmargin = DisplayMargin;
        m_ymargin = DisplayMargin;
        m_canvas->SetVirtualSize (w, h);
    }
    else
    {
        if (m_scale2)
        {
            m_xscale = m_yscale = 2.;
        }
        else
        {
            m_xscale = m_yscale = 1.;
        }
        if (m_fullScreen)
        {
            m_xmargin = (w / m_xscale - 512) / 2;
            m_ymargin = (h / m_yscale - 512) / 2;
        }
        else
        {
            m_xmargin = m_ymargin = DisplayMargin;
        }
        m_canvas->SetVirtualSize (XSize, YSize);
    }

    // Turn off scrollbars in full screen mode.
    if (m_fullScreen)
    {
        m_canvas->SetScrollRate (0, 0);
    }
    else
    {
        m_canvas->SetScrollRate (1, 1);
    }

    // Set the scale for the display window
    wxClientDC dc (m_canvas);

    dc.SetUserScale (m_xscale, m_yscale);

    // Set the menu/popup check marks.  Note that there is no menubar
    // in full screen mode, so no full screen check on the menubar!
    menuPopup->Check (Pterm_Toggle2xMode, (m_scale2));
    menuView->Check (Pterm_Toggle2xModeView, (m_scale2));
    menuPopup->Check (Pterm_ToggleStretchMode, m_stretch);
    menuView->Check (Pterm_ToggleStretchModeView, m_stretch);
    menuPopup->Check (Pterm_FullScreen, m_fullScreen);
#if !defined (__WXMAC__)
    menuPopup->Check (Pterm_ToggleMenuBar, ptermApp->m_showMenuBar);
    menuView->Check (Pterm_ToggleMenuBarView, ptermApp->m_showMenuBar);
#endif
    if (m_port >= 0)
    {
        menuPopup->Check (Pterm_ToggleStatusBar, ptermApp->m_showStatusBar);
        menuView->Check (Pterm_ToggleStatusBarView, ptermApp->m_showStatusBar);
    }

    // Update the display and return focus to the current display window.
    m_canvas->Refresh ();
    m_canvas->SetFocus ();
}

#if defined (__WXMSW__)
void PtermFrame::OnIconize (wxIconizeEvent &event)
{
    // this helps control to a certain extent, the scrollbars that
    // sometimes appear when restoring from iconized state.
    if (!IsIconized ())
    {
        UpdateDisplayState ();
    }
}
#endif

void PtermFrame::ptermDrawChar (int x, int y, int snum, int cnum, bool autobs)
{
    u32 fpix, bpix;
    const u16 *charp;
    PixelData pixmap (*m_bitmap);
    PixelData selmap (*m_selmap);
    
    // Drawing a character is done simply by drawing the dots one by one.
    if (snum == 0)
    {
        charp = plato_m0;
    }
    else if (snum == 1)
    {
        charp = plato_m1;
    }
    else
    {
        charp = plato_m23 + (snum - 2) * (8 * 64);
    }
    charp += 8 * cnum;
    //debug ("char %d mem %d addr %p", cnum, snum, charp);

    if (modexor || (wemode & 1))
    {
        // mode rewrite or write
        fpix = m_fgpix;
        bpix = m_bgpix;
    }
    else
    {
        // mode inverse or erase
        fpix = m_bgpix;
        bpix = m_fgpix;
    }

    // We draw the character twice: once onto the screen bitmap,
    // and once onto the selection region bitmap.  The latter reflects
    // the text currently stored in the savemap, which is coarse grid
    // aligned.  Selection region bitmap entries are normally drawn
    // in mode rewrite, except for autobackspace where we want to
    // save the resulting combined character shape.
    ptermDrawCharInto (x, y, charp, fpix, bpix, mode, modexor, pixmap);
    ptermDrawCharInto (x & 0770, y & 0760, charp, m_selpixf, m_selpixb,
                       (autobs ? 3 : 1), false, selmap);
}

void PtermFrame::ptermDrawCharInto (int x, int y, const u16 *charp,
                                    u32 fpix, u32 bpix, int cmode,
                                    bool xor_p, PixelData &pixmap)
{
    int &cx = (vertical) ? y : x;
    int &cy = (vertical) ? x : y;
    int i, j, saveY, dx, dy, sdy;
    u16 charw;

    saveY = cy;
    dx = dy = (large) ? 2 : 1;
    sdy = 1;
    if (vertical)
    {
        sdy = -1;
        dy = -dy;
    }

    for (j = 0; j < 8; j++)
    {
        cy = saveY;
        charw = *charp++;
        for (i = 0; i < 16; i++)
        {
            if ((charw & 1) == 0)
            {
                // background, do we erase it?
                if ((cmode & 2) == 0)
                {
                    ptermUpdatePoint (x, y, bpix, false, pixmap);
                    if (large)
                    {
                        ptermUpdatePoint (x + 1, y, bpix, false, pixmap);
                        ptermUpdatePoint (x, y + sdy, bpix, false, pixmap);
                        ptermUpdatePoint (x + 1, y + sdy, bpix, false, pixmap);
                    }
                }
            }
            else
            {
                ptermUpdatePoint (x, y, fpix, xor_p, pixmap);
                if (large)
                {
                    ptermUpdatePoint (x + 1, y, fpix, xor_p, pixmap);
                    ptermUpdatePoint (x, y + sdy, fpix, xor_p, pixmap);
                    ptermUpdatePoint (x + 1, y + sdy, fpix, xor_p, pixmap);
                }
            }
            charw >>= 1;
            cy += dy;
        }
        cx += dx;
    }
}

void PtermFrame::ptermDrawPoint (int x, int y)
{
    PixelData pixmap (*m_bitmap);

    if (modexor || (wemode & 1))
    {
        // mode rewrite or write
        ptermUpdatePoint (x, y, m_fgpix, modexor, pixmap);
    }
    else
    {
        // mode inverse or erase
        ptermUpdatePoint (x, y, m_bgpix, false, pixmap);
    }
}

void PtermFrame::ptermUpdatePoint (int x, int y, u32 pixval, bool xor_p,
                                   PixelData &pixmap)
{
    PixelData::Iterator p (pixmap);
    u32 *pmap;
    
    x = XMADJUST (x & 0777);
    y = YMADJUST (y & 0777);
    
    p.MoveTo (pixmap, x, y);
    pmap = (u32 *)(p.m_ptr);

    if (xor_p)
    {
        *pmap ^= pixval;
    }
    else
    {
        *pmap = pixval;
    }
}

void PtermFrame::ptermDrawLine (int x1, int y1, int x2, int y2)
{
    int dx, dy;
    int stepx, stepy;

    dx = x2 - x1;
    dy = y2 - y1;
    if (dx < 0) { dx = -dx;  stepx = -1; } else { stepx = 1; }
    if (dy < 0) { dy = -dy;  stepy = -1; } else { stepy = 1; }
    dx <<= 1;
    dy <<= 1;
    
    // draw first point
    ptermDrawPoint (x1, y1);
    
    //check for shallow line
    if (dx > dy) 
    {
        int fraction = dy - (dx >> 1);
        while (x1 != x2) 
        {
            if (fraction >= 0) 
            {
                y1 += stepy;
                fraction -= dx;
            }
            x1 += stepx;
            fraction += dy;
            ptermDrawPoint (x1, y1);
        }
    } 
    //otherwise steep line
    else 
    {
        int fraction = dx - (dy >> 1);
        while (y1 != y2) 
        {
            if (fraction >= 0) 
            {
                x1 += stepx;
                fraction -= dy;
            }
            y1 += stepy;
            fraction += dx;
            ptermDrawPoint (x1, y1);
        }
    }
}

void PtermFrame::ptermFullErase (void)
{
    const bool savexor = modexor;
    const int savemode = mode;
    wxClientDC dc (m_canvas);

    m_usefont = false;

    // We'll simply handle this as a mode-erase block erase operation
    // for the whole screen (0..512 in x and y).
    modexor = false;
    mode = 2;   // erase
    ptermBlockErase (0, 0, 511, 511);
    modexor = savexor;
    mode = savemode;

    ClearRegion ();
}

void PtermFrame::ptermBlockErase (int x1, int y1, int x2, int y2)
{
    int t;
    int x, y;
    u32 pix;
    PixelData pixmap (*m_bitmap);
    PixelData selmap (*m_selmap);
    
    if (x1 > x2)
        t = x1, x1 = x2, x2 = t;
    if (y1 > y2)
        t = y1, y1 = y2, y2 = t;

    if (modexor || (wemode & 1))
    {
        // mode rewrite or write
        pix = m_fgpix;
    }
    else
    {
        // mode inverse or erase
        pix = m_bgpix;
    }

    for (x = x1; x <= x2; x++)
    {
        for (y = y1; y <= y2; y++)
        {
            ptermUpdatePoint (x, y, pix, modexor, pixmap);
        }
    }
    
    // Wipe text map
    int scol = int (x1 / 8);
    int cols = int (ceil (double ((x2 - x1) / 8))) + 1;
    int srow = int (y1 / 16);
    int rows = int (ceil (double ((y2 - y1) / 16))) + 1;
    for (int row = srow; row < srow + rows; row++)
    {
        for (int col = scol; col < scol + cols; col++)
        {
            textmap[row * 64 + col][0] = '\0';
            textmap[row * 64 + col][1] = '\0';
            textmap[row * 64 + col][2] = '\0';
            textmap[row * 64 + col][3] = '\0';
        }
    }
    
    // Wipe the corresponding region of the selection image
    for (x = scol * 8; x < (scol + cols) * 8; x++)
    {
        for (y = srow * 16; y < (srow + rows) * 16; y++)
        {
            ptermUpdatePoint (x, y, m_selpixb, false, selmap);
        }
    }

}

void PtermFrame::ptermPaint (int pat)
{
    int xm, ym;
    PixelData pixmap (*m_bitmap);
    
    xm = XMADJUST (currentX);
    ym = YMADJUST (currentY);

    // According to the spec, "pat" is either a memory/char number, or if 0
    // means "fill with solid color".  What we're doing here is only the 
    // latter case, just as in previous versions of pterm.
    //
    // The algorithm used is simply a walk of the bitmap starting at
    // the current x/y.  More precisely, two walks: in the first we replace
    // all non-background pixels by a magic value.  In the second, we replace
    // that magic value by the foreground.
    // The reason for doing it that way is that the recursion won't terminate
    // correctly if we paint the pixels in one pass; an already-painted
    // pixel will be visited repeatedly.  And we can't skip "already painted"
    // pixels entirely because they might just be an isolated set of pixels
    // that are already foreground, and we should go right across them.
    ptermPaintWalker (xm, ym, pixmap, 0);
    ptermPaintWalker (xm, ym, pixmap, 1);
}

// Pass 0 means: stop if you hit background pixels; replace all others by 
// special value 0 (for use by pass 1).
// Pass 1 means: stop if you hit a pixel that's not 0; replace 0 by foreground.
#define wdone(pmap, pass) ((pass && *pmap != 0) || \
                           (!pass && (*pmap == m_bgpix || *pmap == 0)))

// This stack is used to implement what amounts to the trivial recursive
// fill algorithm without actual recursion.  All we need is one byte per
// recursion level, which is an acceptable hit.  Worst case (filling a 
// blank screen) depth is the number of pixels on the screen, plus one
// because we use one to visit a pixel that is already filled or is not
// going to be filled.  So allocate that plus one more; the one extra
// entry allows a stack overflow sanity check.
static u8 walkstack[512 * 512 + 2];

void PtermFrame::ptermPaintWalker (int x, int y, PixelData & pixmap, int pass)
{
    PixelData::Iterator p (pixmap);
    u32 *pmap;
    int sp;
#define PUSH walkstack[++sp] = 0
    const int newpixel = pass ? m_fgpix : 0;
    int maxsp = 0;
    int pixels = 0;
    int w;
    
    sp = -1;
    PUSH;
    while (sp >= 0)
    {
        if (sp > maxsp)
        {
            assert (sp < sizeof (walkstack) - 1);
            maxsp = sp;
        }
        p.MoveTo (pixmap, x, y);
        pmap = (u32 *)(p.m_ptr);

        // Each time through the loop we increment the top of stack
        // value, to reflect progress in the walk
        w = walkstack[sp];
        walkstack[sp]++;
        
        switch (w)
        {
        case 0:
            // If the pixel is already filled, leave this level.
            // Otherwise, fill the pixel and explore to the left.
            if (wdone (pmap, pass))
            {
                walkstack[sp] = 4;
                break;
            }
            *pmap = newpixel;
            pixels++;
            if (x > 0)
            {
                x--;
                PUSH;
            }
            break;
        case 1:
            // Explore to the right
            if (x < 511)
            {
                x++;
                PUSH;
            }
            break;
        case 2:
            // Explore up
            if (y < 511)
            {
                y++;
                PUSH;
            }
            break;
        case 3:
            // Explore down
            if (y > 0)
            {
                y--;
                PUSH;
            }
            break;
        case 4:
            // Done exploring at this pixel.  Pop the stack.  We have
            // to adjust the coordinate, which is done based on the
            // next to top stack entry -- that one reflects where the
            // previous level is in its exploration, i.e., which coordinate
            // adjustment it made before pushing this level.  Note that
            // the previous level entry reflects the increment of the
            // stack value, so the case labels are one higher than
            // the corresponding ones above.
            --sp;
            if (sp < 0)
            {
                break;
            }
            switch (walkstack[sp])
            {
            case 1:
                x++;
                break;
            case 2:
                x--;
                break;
            case 3:
                y--;
                break;
            case 4:
                y++;
                break;
            }
        }
    }
    
    trace ("paintwalker: %d pixels, %d max stack", pixels, maxsp);
}

void PtermFrame::ptermSetName (wxString &winName)
{
    wxString str;
    
    if (winName.IsEmpty ())
    {
        str = wxT ("Pterm");
    }
    else
    {
        str.Printf (wxT ("Pterm: %s"), winName);
    }
    SetTitle (str);
}

void PtermFrame::ptermSetStatus (wxString &str)
{
    if (m_statusBar != NULL)
    {
        m_statusBar->SetStatusText (str, STATUS_CONN);
    }
}

void PtermFrame::SetColors (wxColour &newfg, wxColour &newbg)
{
    union {
        u32 pix;
        u8 p[4];
    } cvtpix;
    
    trace ("fg: %d %d %d; bg: %d %d %d", newfg.Red (), newfg.Green (),
           newfg.Blue (), newbg.Red (), newbg.Green (), newbg.Blue ());

    m_defFg = newfg;
    m_defBg = newbg;

    // Calculate the new pixel values
    cvtpix.pix = m_maxalpha;
    cvtpix.p[m_red] = newfg.Red ();
    cvtpix.p[m_green] = newfg.Green ();
    cvtpix.p[m_blue] = newfg.Blue ();
    m_fgpix = cvtpix.pix;
    cvtpix.pix = m_maxalpha;
    cvtpix.p[m_red] = newbg.Red ();
    cvtpix.p[m_green] = newbg.Green ();
    cvtpix.p[m_blue] = newbg.Blue ();
    m_bgpix = cvtpix.pix;
    
}

void PtermFrame::drawFontChar (int x, int y, int c)
{
    wxString chr;

    chr.Printf (wxT ("%c"), c);

    m_memDC->SelectObject (*m_bitmap);
    switch (wemode)
    {
    case 0:         // inverse
        m_memDC->SetBackgroundMode (wxSOLID);
        m_memDC->SetTextBackground (m_currentFg);
        m_memDC->SetTextForeground (m_currentBg);
        break;
    case 1:         // rewrite
        m_memDC->SetBackgroundMode (wxSOLID);
        m_memDC->SetTextBackground (m_currentBg);
        m_memDC->SetTextForeground (m_currentFg);
        break;
    case 2:         // erase
        m_memDC->SetBackgroundMode (wxTRANSPARENT);
        m_memDC->SetTextForeground (m_currentBg);
        break;
    case 3:         // write
        m_memDC->SetBackgroundMode (wxTRANSPARENT);
        m_memDC->SetTextForeground (m_currentFg);
        break;
    }
    
    m_memDC->GetTextExtent (chr, &m_fontwidth, &m_fontheight);

    x = XMADJUST (x);
    y = YMADJUST (BOUND (y + m_fontheight - 1));

    currentX += m_fontwidth;
    
    if (modexor)
    {
        m_memDC->SetLogicalFunction (wxXOR);
    }
    else
    {
        m_memDC->SetLogicalFunction (wxCOPY);
    }
    m_memDC->DrawText (chr, x, y);
    m_memDC->SelectObject (wxNullBitmap);
}

/*--------------------------------------------------------------------------
**  Purpose:        Process word of PLATO output data
**
**  Parameters:     Name        Description.
**                  d           19-bit word
**                  ascii       true if using ASCII protocol
**
**  Returns:        true if the bitmap has changed, false otherwise.
**
**------------------------------------------------------------------------*/
bool PtermFrame::procPlatoWord (u32 d, bool ascii)
{
    mptr mp;
    const char *msg = "";
    int i, n = 0;
    AscState    ascState;
    bool changed = false;
    bool autobs;
    
    // used in load coordinate
    int &coord = (d & 01000) ? currentY : currentX;
    int &cx = (vertical) ? currentY : currentX;
    int &cy = (vertical) ? currentX : currentY;
    
    int deltax, deltay, supdelta;

    bool settitleflag = false;
    
    if (m_usefont && currentCharset <= 1)
    {
        supdelta = (m_fontheight / 3);
        // Not going to support reverse/vertical in font mode until I get documentation
        // on what worked with -font- in the past.  JWS 5/27/2007
        deltax = 8;
        deltay = m_fontheight;
    }
    else
    {
        deltax = (reverse) ? -8 : 8;
        deltay = (vertical) ? -16 : 16;
        if (large)
        {
            deltax *= 2;
            deltay *= 2;
        }
        supdelta = (deltay / 16) * 5;
    }
    
    seq++;
    m_currentWord = d;
    if (ascii)
    {
        if (m_dumbTty)
        {
            if (d == (033 << 8) + 002)   // ESC STX
            {
                trace ("Entering PLATO terminal mode");
                m_dumbTty = false;
                mode = (3 << 2) + 1;    // set character mode, rewrite
                ptermSetStation (-1, true, ptermApp->m_showStatusBar);   // Show connected in ASCII mode
            }
            else if ((d >> 8) == 0)
            {
                changed = true;
                if (d >= 32 && d < 127)
                {
                    d = asciiM0[d];
                    if (d != 0xff)
                    {
                        // Force mode rewrite
                        mode = (3 << 2) + 1;
                        i = (d & 0x80) >> 7;
                        d &= 0x7f;
                        SaveChar (currentX, currentY, rom01char[d + i * 64],
                                  false);
                        ptermDrawChar (currentX, currentY, i, d);
                        currentX = (currentX + 8) & 0777;
                    }
                }
                else if (d == 015)
                {
                    currentX = 0;
                }
                else if (d == 012)
                {
                    if (currentY != 0)
                    {
                        currentY -= 16;
                    }
                    else
                    {
                        // On the bottom line... scroll both the
                        // display and the selection image bitmaps.
                        // And scroll the saved text map.  And cancel
                        // any selected region because the image
                        // scrolled out from under the region.  No,
                        // we're not going to adjust the region
                        // positions...
                        PixelData pixmap (*m_bitmap);
                        PixelData selmap (*m_selmap);

                        PixelData::Iterator from (pixmap);
                        PixelData::Iterator to (pixmap);
                        PixelData::Iterator sfrom (selmap);
                        PixelData::Iterator sto (selmap);
    
                        from.MoveTo (pixmap, 0, 16);
                        to.MoveTo (pixmap, 0, 0);
                        sfrom.MoveTo (selmap, 0, 16);
                        sto.MoveTo (selmap, 0, 0);

                        memmove (to.m_ptr, from.m_ptr, (512 - 16) * 512 * 4);
                        memmove (sto.m_ptr, sfrom.m_ptr, (512 - 16) * 512 * 4);
                        // Note that the textmap has y==0 for the bottom line
                        memmove (&textmap[64], &textmap[0],
                                 (sizeof (textmap) / 32) * 31);
                        memset (&textmap[0], 0, sizeof (textmap) / 32);

                        ClearRegion ();
                    }
                    // Erase the line we just moved to.
                    mode = (3 << 2) + 2;    // set character mode, erase
                    ptermBlockErase (0, currentY, 511, currentY + 15);
                    mode = (3 << 2) + 1;    // set character mode, rewrite
                }
            }
        }
        else if (m_ascState == pni_rs)
        {
            // We just want to ignore 3 command codes.  Note that escape
            // sequences count for one, not two.
            if (++m_ascBytes == 3)
            {
                m_ascBytes = 0;
                m_ascState = none;
            }
        }
        else if (m_ascState == pmd)
        {
            n = AssembleAsciiPlatoMetaData (d);
            if (n == 0)
            {
                if (m_fontPMD)
                {
                    trace ("plato meta data complete: font data accepted");
                    trace ("Font selected: %s, %d, %d",
                           m_fontface.mb_str ().data (), m_fontsize,
                           m_fontbold|m_fontitalic|m_fontstrike|m_fontunderln);
                    m_fontPMD = false;
                }
                else if (m_fontinfo)
                {
                    trace ("plato meta data complete: get font data accepted and sent");
                    m_fontinfo = false;
                }
                else if (m_osinfo)
                {
                    trace ("plato meta data complete: get operating system info accepted and sent");
                    m_osinfo = false;
                }
                else
                {
                    trace ("plato meta data complete: %s",
                           m_PMD.mb_str ().data ());
                    ProcessPlatoMetaData ();
                }
                m_PMD = wxT ("");
            }
        }
        else if ((d >> 8) == 033)
        {
            // Escape sequence, the second character is in the low byte
            d &= 0377;
            switch (d)
            {
            case 002:   // ESC STX
                trace ("Still in PLATO terminal mode");
                m_dumbTty = false;
                break;
            case 003:   // ESC ETX
                trace ("Leaving PLATO terminal mode");
                m_dumbTty = true;
                m_flowCtrl = false;
                m_sendFgt = false;
                currentX = 0;
                currentY = 496;
                break;
            case 014:   // ESC FF
                trace ("Full screen erase");
                ptermFullErase ();
                changed = true;
                break;
            case 026:
                // mode xor (also sets mode write for off-screen DC operations)
                trace ("load mode xor");
                modexor = true;
                mode = (mode & ~3) + 2;
                break;
            case 021:   // ESC DC1
            case 022:   // ESC DC2
            case 023:   // ESC DC3
            case 024:   // ESC DC4
                // modes inverse, write, erase, rewrite
                modexor = false;
                mode = (mode & ~3) + ascmode[d - 021];
                //mode = 017;
                trace ("load mode %d", mode);
                break;
            case '2':
                // Load coordinate
                trace ("Start load coordinate");
                m_ascState = ldc;
                m_ascBytes = 0;
                break;
            case '@':
                // superscript
                trace ("Superscript");
                cy = (cy + supdelta) & 0777;
                break;
            case 'A':
                // subscript
                trace ("Subscript");
                cy = (cy - supdelta) & 0777;
                break;
            case 'B':
            case 'C':
            case 'D':
            case 'E':
            case 'F':
            case 'G':
            case 'H':
            case 'I':
                trace ("select memory M%d", d - 'B');
                setCmem (d - 'B');
                break;
            case 'J':
                trace ("Horizontal writing mode");
                setVertical (false);
                break;
            case 'K':
                trace ("Vertical writing mode");
                setVertical (true);
                break;
            case 'L':
                trace ("Forward writing mode");
                setReverse (false);
                break;
            case 'M':
                trace ("Reverse writing mode");
                setReverse (true);
                break;
            case 'N':
                trace ("Normal size writing mode");
                setLarge (false);
                break;
            case 'O':
                trace ("Double size writing mode");
                setLarge (true);
                break;
            case 'P':
                modexor = false;
                mode = (mode & 3) + (2 << 2);
                trace ("load mode %d", mode);
                break;
            case 'Q':
                m_ascState = ssf;
                m_ascBytes = 0;
                trace ("Start SSF");
                break;
            case 'R':
                // external data
                m_ascState = ext;
                m_ascBytes = 0;
                trace ("Start ext");
                break;
            case 'S':
                modexor = false;
                mode = (mode & 3) + (2 << 2);
                trace ("load mode %d", mode);
                break;
            case 'T':
                modexor = false;
                mode = (mode & 3) + (5 << 2);
                trace ("load mode %d", mode);
                break;
            case 'U':
                modexor = false;
                mode = (mode & 3) + (6 << 2);
                trace ("load mode %d", mode);
                break;
            case 'V':
                modexor = false;
                mode = (mode & 3) + (7 << 2);
                trace ("load mode %d", mode);
                break;
            case 'W':
                // Load memory address
                trace ("Start LDA");
                m_ascState = lda;
                m_ascBytes = 0;
                break;
            case 'X':
                trace ("Start load plato meta data");
                m_ascState = pmd;
                m_ascBytes = 0;
                break;
            case 'Y':
                // load echo
                trace ("Start LDE");
                m_ascState = lde;
                m_ascBytes = 0;
                break;
            case 'Z':
                // set margin
                trace ("set margin %d", currentX);
                setMargin (cx);
                break;
            case 'a':
                // set foreground color
                trace ("Start foreground color");
                m_ascState = fg;
                m_ascBytes = 0;
                break;
            case 'b':
                // set background color
                trace ("Start background color");
                m_ascState = bg;
                m_ascBytes = 0;
                break;
            case 'c':
                // paint
                trace ("Start paint");
                m_ascState = paint;
                m_ascBytes = 0;
                break;
            case 'g':
                // set gray-scale foreground color
                trace ("Start grayscale foreground color");
                m_ascState = gsfg;
                m_ascBytes = 0;
                break;
            default:
                trace ("Other unknown ESCAPE sequence: %d", d);
                break;
            }
        }
        else
        {
            switch (d)
            {
            case 010:   // backspace
                cx = (cx - deltax) & 0777;
                trace ("backspace");
                break;
            case 011:   // tab
                trace ("tab");
                cx = (cx + deltax) & 0777;
                break;
            case 012:   // linefeed
                trace ("linefeed");
                cy = (cy - deltay) & 0777;
                break;
            case 013:   // vertical tab
                trace ("vertical tab");
                cy = (cy + deltay) & 0777;
                break;
            case 014:   // form feed
                trace ("form feed");
                if (vertical)
                {
                    cx = deltay - 1;
                    cy = reverse ? 512 - deltax : 0;
                }
                else
                {
                    cy = 512 - deltay;
                    cx = reverse ? 512 - deltax : 0;
                }
                break;
            case 015:   // carriage return
                cx = margin;
                cy = (cy - deltay) & 0777;
                trace ("CR to %d %d", currentX, currentY);
                break;
            case 031:   // EM
                mode = (mode & 3) + (4 << 2);
                modewords = 0;              // words since entering mode
                trace ("load mode %d", mode);
                break;
            case 034:   // FS
                mode = (mode & 3) + (0 << 2);
                trace ("load mode %d", mode);
                break;
            case 035:   // FS
                mode = (mode & 3) + (1 << 2);
                m_ascState = ldc; // to have first coordinate be "dark"
                trace ("load mode %d", mode);
                break;
            case 036:   // RS -- used by PNI in connect handshake
                m_ascState = pni_rs;
                trace ("pni start download, ignoring next 3 commands");
                break;
            case 037:   // FS
                mode = (mode & 3) + (3 << 2);
                trace ("load mode %d", mode);
                break;
            }
            if (d >= 040)
            {
                switch (m_ascState)
                {
                case ldc:
                    if (AssembleCoord (d))
                    {
                        currentX = lastX;
                        currentY = lastY;
                        trace ("load coordinate %d %d", currentX, currentY);
                    }
                    break;
                case paint:
                    n = AssemblePaint (d);
                    if (n != -1)
                    {
                        trace ("paint %03o", n);
                        changed = true;
                        ptermPaint (n);
                    }
                    break;
                case lde:
                    n = AssembleData (d);
                    if (n != -1)
                    {
                        n &= 0177;
                        switch (n)
                        {
                        case 0160:
                            // 160 is terminal type query
                            n = 0160 + ASCTYPE;
                            trace ("load echo termtype %d", n);
                            break;
                        case 0x71:
                            if ((n = ptermApp->m_termType) == 0)
                            {
                                n = SUBTYPE;
                            }
                            trace ("load echo subtype %d", n);
                            break;
                        case 0x72:
                            trace ("load echo loadfile (unused)");
                            n = 0;
                            break;
                        case 0x73:
                            // hex 73 is report terminal config
                            trace ("load echo termdata");
                            n = TERMCONFIG;
                            break;
                        case 0x7b:
                            // hex 7b is beep
                            if (ptermApp->m_beepEnable)
                            {
                                trace ("beep");
                                if (g_beep != NULL && !g_beep->Play ())
                                    printf ("beep play failed\n");
                                if (!IsActive ())
                                    RequestUserAttention (wxUSER_ATTENTION_INFO);
                            }
                            break;
                        case 0x7d:
                            // hex 7d is report MAR
                            trace ("report MAR %o", memaddr);
                            n = memaddr;
                            break;
                        case 0x52:
                            // hex 52 is enable flow control
                            trace ("enable flow control");
                            m_flowCtrl = true;
                            n = 0x53;
                            break;
                        case 0x60:
                            // hex 60 is inquire features
                            trace ("report features 0x%02x", ASCFEATURES);
                            n += ASCFEATURES;
                            m_sendFgt = true;
                            break;
                        default:
                            trace ("load echo %d (0x%02x)", n, n);
                        }
                        if (n == 0x7b)
                        {
                            // -beep- does NOT send an echo code in reply
                            break;
                        }
                        if (m_conn == NULL)
                        {
                            // If not connection, don't try to reply
                            break;
                        }
                        
                        n += 0200;
                        if (m_conn->RingCount () > RINGXOFF1)
                        {
                            debug ("pend echo %d, previous was %d",
                                    n, m_pendingEcho);
                            m_pendingEcho = n;
                        }
                        else
                        {
                            ptermSendKey1 (n);
                            m_pendingEcho = -1;
                        }
                    }
                    break;
                case lda:
                    n = AssembleData (d);
                    if (n != -1)
                    {
                        trace ("load memory address %04x", n);
                        memaddr = n & 077777;
                    }
                    break;
                case ext:
                    trace ("ext %04x", d);
                    n = AssembleData (d);
                    switch (n)
                    {
                    case -1:
                        break;
                    // check for special TERM area save/restore
                    case CWS_TERMSAVE:
                        trace ("ext completed %04x", n);
                        // data items are, in order: xleft, ytop, xright, ybot
                        cwswindow[0].data[0] = 0;
                        cwswindow[0].data[1] = 48;
                        cwswindow[0].data[2] = 511;
                        cwswindow[0].data[3] = 0;
                        ptermSaveWindow (0);
                        break;
                    case CWS_TERMRESTORE:
                        trace ("ext completed %04x", n);
                        ptermRestoreWindow (0);
                        changed = true;
                        break;
                    default:
                        trace ("ext completed %04x", n);
                        // check if in cws mode
                        switch (cwsmode)
                        {
                            // not in cws; check for font data
                            case 0:
                                cwscnt = 0;
                                switch (n & 07700)
                                {
                                case 05000:     // font face name and family
                                    SetFontFaceAndFamily (n & 077);
                                    break;
                                case 05100:     // font size
                                    SetFontSize (n & 077);
                                    break;
                                case 05200:     // font flags
                                    SetFontFlags (n & 077);
                                    SetFontActive ();
                                    break;
                                }
                                break;
                            // check for cws data mode
                            case 1:
                                cwscnt++;
                                if     (cwscnt == 1 && n == CWS_SAVE)          
                                {
                                    trace ("CWS: specify save function");
                                    cwsfun = CWS_SAVE;
                                }
                                else if (cwscnt == 1 && n == CWS_RESTORE)   
                                {
                                    trace ("CWS: specify restore function");
                                    cwsfun = CWS_RESTORE;
                                }
                                else if (cwscnt == 1 || cwscnt > 6)
                                {
                                    // unknown function; terminate cws mode
                                    trace ("CWS: invalid function; %d", n);
                                    cwsmode = 0;
                                    cwsfun = 0;
                                    cwscnt = 0;
                                }
                                else if (cwscnt == 2)
                                {
                                    if ((unsigned) n < sizeof (cwswindow) / sizeof (cwswindow[0]))
                                    {
                                        trace ("CWS: specify window; %d", n);
                                        cwswin = n;
                                    }
                                    else
                                    {
                                        trace ("CWS: invalid window; %d", n);
                                        cwsmode = 0;
                                        cwsfun = 0;
                                        cwscnt = 0;
                                    }
                                }
                                else if (cwscnt < 7)
                                {
                                    trace ("CWS: data; %d", n);
                                    cwswindow[cwswin].data[cwscnt - 3] = n;
                                }
                                break;
                            // check for cws execute mode
                            case 2:
                                cwscnt = 0;
                                if (n == CWS_EXEC)
                                {
                                    trace ("CWS: process exec");
                                    switch (cwsfun)
                                    {
                                    case CWS_SAVE:
                                        ptermSaveWindow (cwswin);
                                        break;
                                    case CWS_RESTORE:
                                        ptermRestoreWindow (cwswin);
                                        changed = true;
                                        break;
                                    }
                                }
                                else
                                {
                                    // unknown function; terminate cws mode
                                    trace ("CWS: invalid function; %d", n);
                                    cwsmode = 0;
                                    cwsfun = 0;
                                    cwscnt = 0;
                                }
                                break;
                            }
                    }
                    break;
                case ssf:
                    n = AssembleData (d);
                    if (n != -1)
                    {
                        trace ("ssf %04x", n);
                        m_canvas->ptermTouchPanel ((n & 0x20) != 0);
                    }
                    switch (n)
                    {
                    case 0x1f00:    // xin 7; means start CWS functions
                        trace ("ssf; start cws mode; %04x", n);
                        cwsmode = 1;
                        break;
                    case 0x1d00:    // xout 7; means stop CWS functions
                        trace ("ssf; stop cws mode; %04x", n);
                        cwsmode = 2;
                        break;
                    case -1:
                        break;
                    default:
                        trace ("ssf %04x", n);
                        m_canvas->ptermTouchPanel ((n & 0x20) != 0);
                        break;
                    }
                    break;
                case fg:
                case bg:
                    ascState = m_ascState;
                    n = AssembleColor (d);
                    if (n != -1 && !ptermApp->m_noColor)
                    {
                        wxColour c ((n >> 16) & 0xff, (n >> 8) & 0xff, n & 0xff);
                        if (ascState == fg)
                        {
                            trace ("set foreground color %06x", n);
                            m_currentFg = c;
                        }
                        else
                        {
                            trace ("set background color %06x", n);
                            m_currentBg = c;
                        }
                        SetColors (m_currentFg, m_currentBg);
                    }
                    break;
                case gsfg:
                    ascState = m_ascState;
                    n = AssembleGrayScale (d);
                    if (n != -1 && !ptermApp->m_noColor)
                    {
                        wxColour c (n & 0xff, n & 0xff, n & 0xff);
                        if (ascState == gsfg)
                        {
                            trace ("set gray-scale foreground color %06x", n);
                            m_currentFg = c;
                        }
                        SetColors (m_currentFg, m_currentBg);
                    }
                    break;
                case pmd:
                    break; // handled above
                case none:
                    switch (mode >> 2)
                    {
                    case 0:
                        if (AssembleCoord (d))
                        {
                            mode0 ((lastX << 9) + lastY);
                            changed = true;
                        }
                        break;
                    case 1:
                        if (AssembleCoord (d))
                        {
                            mode1 ((lastX << 9) + lastY);
                            changed = true;
                        }
                        break;
                    case 2:
                        n = AssembleData (d);
                        if (n != -1)
                        {
                            mode2 (n);
                        }
                        break;
                    case 3: // text mode
                        trace ("char %03o (%c)", d, d);
                        m_ascState = none;
                        m_ascBytes = 0;
                        changed = true;
                        i = currentCharset;
                        if (m_usefont && i == 0)
                        {
                            SaveChar (currentX, currentY, d, large);
                            drawFontChar (currentX, currentY, d);
                        }
                        else
                        {
                            if (i == 0)
                            {
                                d = asciiM0[d];
                                // The ROM vs. RAM choice is given by the
                                // current character set.  
                                // For the ROM characters, the even vs. odd
                                // (M0 vs. M1) choice is given by the top bit
                                // of the ASCII translation table.
                                i = (d & 0x80) >> 7;
                            }
                            else if (i == 1)
                            {
                                d = asciiM1[d];
                                i = (d & 0x80) >> 7;
                            }
                            else
                            {
                                // RAM characters are indexed by printable
                                // ASCII characters; the RAM character offset
                                // is simply the character code - 32.
                                // The set choice is simply what the host sent.
                                d = (d - 040) & 077;
                            }
                            if (d != 0xff)
                            {
                                d &= 0x7f;
                                autobs = SaveChar (currentX, currentY,
                                                   rom01char[d + i * 64],
                                                   large);
                                ptermDrawChar (currentX, currentY, 
                                               i, d, autobs);
                                cx = (cx + deltax) & 0777;
                            }
                        }
                        break;
                    case 4:
                        if (AssembleCoord (d))
                        {
                            modewords++;
                            mode4 ((lastX << 9) + lastY);
                            changed = true;
                        }
                        break;
                    case 5:
                        n = AssembleData (d);
                        if (n != -1)
                        {
                            changed = true;     // assume changed for
                            mode5 (n);
                        }
                        break;
                    case 6:
                        n = AssembleData (d);
                        if (n != -1)
                        {
                            changed = true;     // assume changed for
                            mode6 (n);
                        }
                        break;
                    case 7:
                        n = AssembleData (d);
                        if (n != -1)
                        {
                            changed = true;     // assume changed for
                            mode7 (n);
                        }
                        break;
                    }
                    break;
                case pni_rs:
                    break;
                }
            }
        }
    }
    else
    {
        if ((d & NOP_MASK) == 0)
        {
            // NOP command...
            if (d & 1)
            {
                wc = (wc + 1) & 0177;
            }
        }
        else
        {
            wc = (wc + 1) & 0177;
        }
    
        if (d & 01000000)
        {
            const int dmode = mode >> 2;
            
            modewords++;
            mp = modePtr[dmode];
            (this->*mp) (d);
            // Modes 0, 1, 3, 4 change the screen.  Mode 5-7 we can't tell so
            // we assume it does.  So only mode 2 leaves "changed" untouched.
            changed |= dmode != 2;
        }
        else
        {
            switch ((d >> 15) & 7)
            {
            case 0:     // nop
                settitleflag = false;
                // special code to tell pterm the station number
                if ((d & NOP_MASKDATA) == NOP_SETSTAT)
                {
                    d &= 0777;
                    ptermSetStation (d, true, ptermApp->m_showStatusBar);
                }
                // special code to get font type / size / flags
                else if ((d & NOP_MASKDATA) == NOP_FONTTYPE)
                    SetFontFaceAndFamily (d & 077);
                else if ((d & NOP_MASKDATA) == NOP_FONTSIZE)
                    SetFontSize (d & 077);
                else if ((d & NOP_MASKDATA) == NOP_FONTFLAG)
                {
                    SetFontFlags (d & 077);
                    SetFontActive ();
                }
                else if ((d & NOP_MASKDATA) == NOP_FONTINFO)
                {
                    const int chardelay = ptermApp->m_charDelay;
                    ptermSendExt ((int) m_fontwidth);
                    wxMilliSleep (chardelay);
                    ptermSendExt ((int) m_fontheight);
                    wxMilliSleep (chardelay);
                }
                else if ((d & NOP_MASKDATA) == NOP_OSINFO)
                {
                    const int chardelay = ptermApp->m_charDelay;
                    // sends 3 external keys, OS, major version, minor version
                    int os, major, minor;
                    os = wxGetOsVersion (&major, &minor);
                    ptermSendExt (os);
                    wxMilliSleep (chardelay);
                    if (os == wxMAC || os == wxMAC_DARWIN)
                        ptermSendExt (10 * (major >> 4) + (major & 0x0f));
                    else
                        ptermSendExt (major);
                    wxMilliSleep (chardelay);
                    if (os == wxMAC || os == wxMAC_DARWIN)
                        ptermSendExt (10* (minor >> 4) + (minor & 0x0f));
                    else
                        ptermSendExt (minor);
                    wxMilliSleep (chardelay);
                }
                // otherwise check for plato meta data codes
                else
                {
                    if ((d & NOP_MASKDATA) == NOP_PMDSTART && !m_loadingPMD)
                    {
                        m_loadingPMD = true;
                        m_PMD = wxT ("");
                        settitleflag = AssembleClassicPlatoMetaData (d & 077);
                    }
                    else if ((d & NOP_MASKDATA) == NOP_PMDSTREAM && m_loadingPMD) 
                    {
                        settitleflag = AssembleClassicPlatoMetaData (d & 077);
                    }
                    else if ((d & NOP_MASKDATA) == NOP_PMDSTOP && m_loadingPMD)
                    {
                        m_loadingPMD = false;
                        AssembleClassicPlatoMetaData (d & 077);
                        settitleflag = true;
                    }
                    if (settitleflag)
                        ProcessPlatoMetaData ();
                }
                trace ("nop");
                break;

            case 1:     // load mode
                modewords = 0;              // words since entering mode
                if ((d & 020000) != 0)
                {
                    // load wc bit is set
                    wc = (d >> 6) & 0177;
                }
                modexor = false;
                mode = (d >> 1) & 037;
                if (d & 1)
                {
                    // full screen erase
                    ptermFullErase ();
                    changed = true;
                }
                trace ("load mode %d screen %d", mode, (d & 1));
                break;
            
            case 2:     // load coordinate
            
                if (d & 04000)
                {
                    // Add or subtract from current coordinate
                    if (d & 02000)
                    {
                        coord -= d & 0777;
                    }
                    else
                    {
                        coord += d & 0777;
                    }
                }
                else
                {
                    coord = d & 0777;
                }
        
                if (d & 010000)
                {
                    setMargin (coord);
                    msg = "margin";
                }
                trace ("load coord %c %d %s", (d & 01000) ? 'Y' : 'X',
                       d & 0777, msg);
                break;
            case 3:     // echo
                d &= 0177;
                switch (d)
                {
                case 0160:
                    // 160 is terminal type query
                    if ((d = ptermApp->m_termType) == 0)
                    {
                        d = 0160 + TERMTYPE;
                    }
                    trace ("load echo termtype %d", d);
                    break;
                case 0x7b:
                    // hex 7b is beep
                    if (ptermApp->m_beepEnable)
                    {
                        trace ("beep");
                        wxBell ();
                        if (!IsActive ())
                            RequestUserAttention (wxUSER_ATTENTION_INFO);
                    }
                    break;
                case 0x7d:
                    // hex 7d is report MAR
                    trace ("report MAR %o", memaddr);
                    d = memaddr;
                    break;
                default:
                    trace ("load echo %d", d);
                }
                if (d == 0x7b)
                {
                    break;          // -beep- does NOT send an echo code in reply
                }
                if (m_conn == NULL)
                {
                    // If not connection, don't try to reply
                    break;
                }
                d += 0200;
                if (m_conn->RingCount () > RINGXOFF1)
                {
                    m_pendingEcho = d;
                }
                else
                {
                    ptermSendKey1 (d);
                    m_pendingEcho = -1;
                }
                break;
            
            case 4:     // load address
                memaddr = d & 077777;
                trace ("load address %o (0x%x)", memaddr, memaddr);
                break;
            
            case 5:     // SSF on PPT
                switch ((d >> 10) & 037)
                {
                case 1: // Touch panel control ?
                    trace ("ssf touch %o", d);
                    m_canvas->ptermTouchPanel ((d & 040) != 0);
                    break;
                default:
                    trace ("ssf %o", d);
                    break;  // ignore
                }
                break;

            case 6:
            case 7:
                d &= 0177777;
                trace ("Ext %07o", d);
                // Take no other action here -- it's been done already
                // when the word was fetched
                break;

            default:    // ignore
                trace ("ignored command word %07o", d);
                break;
            }
        }
    }
    m_currentWord = 0;
    
    return changed;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble a 9 bit data word for the ASCII protocol
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        -1 if word not complete yet, otherwise the word
**
**------------------------------------------------------------------------*/
int PtermFrame::AssemblePaint (int d)
{
    if (m_ascBytes == 0)
    {
        m_assembler = 0;
    }    
    m_assembler |= ((d & 077) << (m_ascBytes * 6));
    if (++m_ascBytes == 2)
    {
        m_ascBytes = 0;
        m_ascState = none;
        trace ("paint %03o (0x%04x)", m_assembler, m_assembler);
        return m_assembler;
    }
    else
    {
        trace ("paint byte %d: %d", m_ascBytes, d & 077);
    }
    return -1;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble an 18 bit data word for the ASCII protocol
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        -1 if word not complete yet, otherwise the word
**
**------------------------------------------------------------------------*/
int PtermFrame::AssembleData (int d)
{
    if (m_ascBytes == 0)
    {
        m_assembler = 0;
    }    
    m_assembler |= ((d & 077) << (m_ascBytes * 6));
    if (++m_ascBytes == 3)
    {
        m_ascBytes = 0;
        m_ascState = none;
        trace ("data %07o (0x%04x)", m_assembler, m_assembler);
        return m_assembler;
    }
    else
    {
        trace ("data byte %d: %d", m_ascBytes, d & 077);
    }
    return -1;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble a 24 bit color word for the ASCII protocol
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        -1 if word not complete yet, otherwise the color word
**
**------------------------------------------------------------------------*/
int PtermFrame::AssembleColor (int d)
{
    if (m_ascBytes == 0)
    {
        m_assembler = 0;
    }    
    m_assembler |= ((d & 077) << (m_ascBytes * 6));
    if (++m_ascBytes == 4)
    {
        m_ascBytes = 0;
        m_ascState = none;
        //trace ("color 0x%06x)", m_assembler);
        return m_assembler;
    }
    else
    {
        trace ("color byte %d: %d", m_ascBytes, d & 077);
    }
    return -1;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble an 7 bit grayscale word for the ASCII protocol
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        -1 if word not complete yet, otherwise the color word
**
**------------------------------------------------------------------------*/
int PtermFrame::AssembleGrayScale (int d)
{
    if (m_ascBytes == 0)
    {
        m_assembler = 0;
    }    
    m_assembler = (d & 077) << 2;
    if (++m_ascBytes == 1)
    {
        m_ascBytes = 0;
        m_ascState = none;
        trace ("gray-scale color 0x%06x", m_assembler);
        return m_assembler;
    }
    else
    {
        trace ("gray-scale color byte %d: %d", m_ascBytes, d & 0177);
    }
    return -1;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble a coordinate pair for the ASCII protocol
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        true if word is complete, false if not
**                  lastX and lastY are the x/y coordinate received
**
**------------------------------------------------------------------------*/
bool PtermFrame::AssembleCoord (int d)
{
    int c = d & 037;
    
    switch (d >> 5)
    {
    case 1: // High X or high Y
        if (m_ascBytes == 0)
        {
            // High Y
            trace ("high Y %d", c);
            lastY = (lastY & 037) | (c << 5);
            m_ascBytes = 2;
        }
        else
        {
            trace ("high X %d", c);
            lastX = (lastX & 037) | (c << 5);
        }
        break;
    case 2:
        lastX = (lastX & 0740) | c;
        m_assembler = (lastX << 16) + lastY;
        m_ascBytes = 0;
        m_ascState = none;
        trace ("low X %d, coordinates %d %d", c, lastX, lastY);
        return true;
    case 3:
        trace ("low Y %d", c);
        lastY = (lastY & 0740) | c;
        m_ascBytes = 2;
        break;
    }
    return false;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble a string for use as extended data from Plato
**                  sent in ASCII connection mode.  Could be meta data
**                  for setting window caption or could be font data, or
**                  other data someday.
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        -1 if word not complete yet, otherwise 0 and get the
**                     data from m_PlatoMetaData
**
**------------------------------------------------------------------------*/
int PtermFrame::AssembleAsciiPlatoMetaData (int d)
{
    int od = d;
    const int chardelay = ptermApp->m_charDelay;

    trace ("plato meta data: %d (counter=%d)", d, m_ascBytes+1);
    d &= 077;
    if (m_ascBytes==0)
        m_PMD = wxT ("");
    m_ascBytes++;
    // check for start font mode
    if (od=='F' && m_ascBytes==1)
        m_fontPMD = true;
    // check for request font character info
    else if (od=='f' && m_ascBytes==1)
    {
        m_fontinfo = true;
        m_ascBytes = 0;
        m_ascState = none;
        ptermSendExt ((int) m_fontwidth);
        wxMilliSleep (chardelay);
        ptermSendExt ((int) m_fontheight);
        wxMilliSleep (chardelay);
        return 0;
    }
    // check for request operating system info
    else if (od=='o' && m_ascBytes==1)
    {
        // sends 3 external keys, OS, major version, minor version
        int os, major, minor;
        os = wxGetOsVersion (&major, &minor);
        ptermSendExt (os);
        wxMilliSleep (chardelay);
        if (os==wxMAC || os==wxMAC_DARWIN)
            ptermSendExt (10* (major>>4) + (major &0x0f));
        else
            ptermSendExt (major);
        wxMilliSleep (chardelay);
        if (os==wxMAC || os==wxMAC_DARWIN)
            ptermSendExt (10* (minor>>4) + (minor &0x0f));
        else
            ptermSendExt (minor);
        wxMilliSleep (chardelay);
        m_osinfo = true;
        m_ascBytes = 0;
        m_ascState = none;
        return 0;
    }
    // check if in font mode
    else if (m_fontPMD && m_ascBytes==2)
    {
        SetFontFaceAndFamily (d);
        if (d==0)
        {
            m_ascBytes = 0;
            m_ascState = none;
            return 0;
        }
    }
    else if (m_fontPMD && m_ascBytes==3)
        SetFontSize (d);
    else if (m_fontPMD && m_ascBytes==4)
    {
        SetFontFlags (d);
        SetFontActive ();
        m_ascBytes = 0;
        m_ascState = none;
        return 0;
    }
    // check if done / full
    else if (d==0 || m_ascBytes==1001)
    {
        if (m_ascBytes==1001)
        {
            trace ("plato meta data limit reached: %d bytes", 1000);
        }
        m_ascBytes = 0;
        m_ascState = none;
        return 0;
    }
    // otherwise keep assembling
    else
    {
        if (d >= 1 && d <= 26)
        {
            m_PMD.Append ((char) ('a'+d-1), 1);
        }
        else if (d >= 27 && d <= 36)
        {
            m_PMD.Append ((char) ('0'+d-27), 1);
        }
        else if (d == 38)
        {
            m_PMD.Append ('-', 1);
        }
        else if (d == 40)
        {
            m_PMD.Append ('/', 1);
        }
        else if (d == 44)
        {
            m_PMD.Append ('=', 1);
        }
        else if (d == 45)
        {
            m_PMD.Append (' ', 1);
        }
        else if (d == 63)
        {
            m_PMD.Append (';', 1);
        }
    }
    return -1;
}

/*--------------------------------------------------------------------------
**  Purpose:        Assemble a string for use as frame caption from data
**                  sent in Classic connection mode.
**
**  Parameters:     Name        Description.
**                  d           current byte of input
**
**  Returns:        -1 
**
**------------------------------------------------------------------------*/
bool PtermFrame::AssembleClassicPlatoMetaData (int d)
{
    trace ("plato meta data: %d", d);
    if (m_PMD.Len ()==1000)
    {
        trace ("plato meta data limit reached: 1000 bytes");
        m_loadingPMD = false;
        return true;
    }
    else
    {
        d &= 077;
        if (d >= 1 && d <= 26)
        {
            m_PMD.Append ((char) ('a'+d-1), 1);
        }
        else if (d >= 27 && d <= 36)
        {
            m_PMD.Append ((char) ('0'+d-27), 1);
        }
        else if (d == 38)
        {
            m_PMD.Append ('-', 1);
        }
        else if (d == 40)
        {
            m_PMD.Append ('/', 1);
        }
        else if (d == 44)
        {
            m_PMD.Append ('=', 1);
        }
        else if (d == 45)
        {
            m_PMD.Append (' ', 1);
        }
        else if (d == 63)
        {
            m_PMD.Append (';', 1);
        }
    }
    return false;
}

/*--------------------------------------------------------------------------
**  Purpose:        Change frame title to string specified in the assembled
**                  meta data.
**
**  Parameters:     none
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::ProcessPlatoMetaData ()
{
    int fnd;
    int len;
    wxString l_str;
    wxString l_name;
    wxString l_group;
    wxString l_system;
    wxString l_station;

    //initialize
    l_name = wxT ("");
    l_group = wxT ("");
    l_system = wxT ("");
    l_station = wxT ("");
    len = m_PMD.Len ();

    //collect meta data items
    if ((fnd = m_PMD.Find (wxT ("name="))) != -1)
    {
        l_name = m_PMD.Mid (fnd+5, len-fnd-5);
        l_name = l_name.BeforeFirst (wxT (';'));
    }

    if ((fnd = m_PMD.Find (wxT ("group="))) != -1)
    {
        l_group = m_PMD.Mid (fnd+6, len-fnd-6);
        l_group = l_group.BeforeFirst (wxT (';'));
    }

    if ((fnd = m_PMD.Find (wxT ("system="))) != -1)
    {
        l_system = m_PMD.Mid (fnd+7, len-fnd-7);
        l_system = l_system.BeforeFirst (wxT (';'));
    }

    if ((fnd = m_PMD.Find (wxT ("station="))) != -1)
    {
        l_station = m_PMD.Mid (fnd+8, len-fnd-8);
        l_station = l_station.BeforeFirst (wxT (';'));
    }

    //make title string based on flags
    l_str = wxT ("");
    if (ptermApp->m_showSignon)
    {
        l_str = l_name;
        l_str.Append (wxT ("/"));
        l_str += l_group;
    }
    if (ptermApp->m_showSysName)
    {
        l_str.Append (wxT ("/"));
        l_str += l_system;
    }
    if (ptermApp->m_showHost)
    {
        l_str.Append (wxT (" "));
        l_str += m_hostName;
    }
    if (ptermApp->m_showStation)
    {
        l_str.Append (wxT (" ("));
        l_str += l_station;
        l_str.Append (wxT (")"));
    }
    l_str.Trim (true);
    l_str.Trim (false);
    if (l_str.IsEmpty ())
    {
        l_str = wxT ("Pterm");
    }
    SetTitle (l_str);
}


/*--------------------------------------------------------------------------
**  Purpose:        Set font face name and family flag.
**
**  Parameters:     font type code
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::SetFontFaceAndFamily (int n)
{
    switch (n)
    {
    case 2:
        m_fontface = wxT ("Terminal");
        m_fontfamily = wxFONTFAMILY_TELETYPE;
        break;
    case 3:
        m_fontface = wxT ("UOL8X14");
        m_fontfamily = wxFONTFAMILY_TELETYPE;
        break;
    case 4:
        m_fontface = wxT ("UOL8X16");
        m_fontfamily = wxFONTFAMILY_TELETYPE;
        break;
    case 5:
        m_fontface = wxT ("Courier");
        m_fontfamily = wxFONTFAMILY_TELETYPE;
        break;
    case 6:
        m_fontface = wxT ("Courier New");
        m_fontfamily = wxFONTFAMILY_MODERN;
        break;
    case 16:
        m_fontface = wxT (SSFONT);
        m_fontfamily = wxFONTFAMILY_DECORATIVE;
        break;
    case 17:
        m_fontface = wxT ("Times New Roman");
        m_fontfamily = wxFONTFAMILY_ROMAN;
        break;
    case 18:
        m_fontface = wxT ("Script");
        m_fontfamily = wxFONTFAMILY_SCRIPT;
        break;
    case 19:
        m_fontface = wxT ("MS Sans Serif");
        m_fontfamily = wxFONTFAMILY_SWISS;
        break;
    default:
        m_fontface = wxT ("default");
        m_fontfamily = wxFONTFAMILY_TELETYPE;
    }
}

/*--------------------------------------------------------------------------
**  Purpose:        Set font size.
**
**  Parameters:     font size
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::SetFontSize (int n)
{
    m_fontsize = (n < 1 ? 1 : (n > 63 ? 63 : n));
}

/*--------------------------------------------------------------------------
**  Purpose:        Set font flags.
**
**  Parameters:     font flag
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::SetFontFlags (int n)
{
    m_fontitalic = ((n & 0x01) != 0);
    m_fontbold = ((n & 0x02) != 0);
    m_fontstrike = ((n & 0x04) != 0);
    m_fontunderln = ((n & 0x08) != 0);
    trace ("Font selected: %s, %d, %d", m_fontface.mb_str ().data (), m_fontsize, n & 0x0f);
}

/*--------------------------------------------------------------------------
**  Purpose:        Set font.
**
**  Parameters:     none
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::SetFontActive ()
{
    m_usefont = (m_fontface.Cmp (wxT (""))!=0 && m_fontface.Cmp (wxT ("default"))!=0);
    if (m_usefont)
    {
        // Set size in pixels (not points), height is supplied, width defaulted
        wxFontInfo fi (wxSize (0, m_fontsize));
        
        fi.FaceName (m_fontface);
        fi.Family (m_fontfamily);
        fi.AntiAliased (true);
        fi.Bold (m_fontbold);
        fi.Italic (m_fontitalic);
        fi.Underlined (m_fontunderln);
        fi.Strikethrough (m_fontstrike);
        
        m_font = new wxFont (fi);

        // We need to select a bitmap into the memDC for GetTextExtent
        // to be accepted.
        m_memDC->SelectObject (*m_bitmap);
        m_memDC->SetFont (*m_font);
		m_memDC->GetTextExtent (wxT (" "), &m_fontwidth, &m_fontheight);
        m_memDC->SelectObject (wxNullBitmap);
    }
}

/*--------------------------------------------------------------------------
**  Purpose:        Save a screen "window"
**
**  Parameters:     Name        Description.
**                  d           window number (range checked by caller)
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::ptermSaveWindow (int d)
{
    wxBitmap *bm = new wxBitmap (512, 512, 32);
    wxMemoryDC dc (*bm);

    // We'll just copy the whole screen worth of bitmap, at restore time
    // we'll do a selective restore.  Note that just constructing a wxBitmap
    // with the copy constructor doesn't work, that gives us a reference to
    // the other bitmap.  We actually have to copy the data, which Blit does
    // easily.
    trace ("CWS: process save; window %d", d);
    cwswindow[d].ok = true;
    m_memDC->SelectObject (*m_bitmap);
    dc.Blit (0, 0, 512, 512, m_memDC, 0, 0);
    m_memDC->SelectObject (wxNullBitmap);
    cwswindow[d].bm = bm;
}

/*--------------------------------------------------------------------------
**  Purpose:        Restore a screen "window"
**
**  Parameters:     Name        Description.
**                  d           window number (range checked by caller)
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::ptermRestoreWindow (int d)
{
    int x, y, w, h;

    x = XMADJUST (BOUND (cwswindow[d].data[0]));
    y = YMADJUST (BOUND (cwswindow[d].data[1]));
    w = BOUND (cwswindow[d].data[2] - cwswindow[d].data[0]);
    h = BOUND (cwswindow[d].data[1] - cwswindow[d].data[3]);
    if (cwswindow[d].ok)
    {
        wxMemoryDC srcdc (*cwswindow[d].bm);

        trace ("CWS: process restore; window %d, region %d %d %d %d",
               d, x, y, w, h);
        m_memDC->SelectObject (*m_bitmap);
        m_memDC->Blit (x, y, w, h, &srcdc, x, y);
        m_memDC->SelectObject (wxNullBitmap);

        cwswindow[d].ok = false;
        delete cwswindow[d].bm;
    }
}

/*--------------------------------------------------------------------------
**  Purpose:        Write a trace message.
**
**  Parameters:     message
**
**  Returns:        nothing
**
**------------------------------------------------------------------------*/
void PtermFrame::WriteTraceMessage (wxString msg)
{
    trace ("%s", msg.mb_str ().data ());
    return;
}

/*--------------------------------------------------------------------------
**  Purpose:        Display visual indication of trace status
**
**  Parameters:     Name        Description.
**                  trace       new trace state (true for on)
**
**  Returns:        nothing
**
**  This function opens or flushes the trace file depending on trace
**  setting.  It then redraws (on or off) the trace indication.
**
**------------------------------------------------------------------------*/
void PtermFrame::ptermSetTrace (bool trace)
{
    if (!trace)
    {
        // Turning trace off
        tracex ("Trace off");
        traceF.Close ();
        tracePterm = trace;
    }
    else
    {
        // Turning trace on
        tracePterm = trace;
        traceF.Open (ptermApp->traceFn);
        if (m_station < 0)
        {
            tracex ("Trace on, ASCII terminal");
        }
        else
        {
            tracex ("Trace on, %d-%d", m_station >> 5, m_station & 31);
        }
    }

    ptermShowTrace ();
}

/*
**--------------------------------------------------------------------------
**
**  Private Functions
**
**--------------------------------------------------------------------------
*/

/*--------------------------------------------------------------------------
**  Purpose:        Draw one character
**
**  Parameters:     Name        Description.
**                  c           Character code
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::plotChar (int c)
{
    int &cx = (vertical) ? currentY : currentX;
    int &cy = (vertical) ? currentX : currentY;
    
    int deltax, deltay, supdelta;
    bool autobs;
    
    if (m_usefont && currentCharset <= 1)
    {
        supdelta = (m_fontheight / 3);
        // Not going to support reverse/vertical in font mode until I get documentation
        // on what worked with -font- in the past.  JWS 5/27/2007
        deltax = 8;
        deltay = m_fontheight;
    }
    else
    {
        deltax = (reverse) ? -8 : 8;
        deltay = (vertical) ? -16 : 16;
        if (large)
        {
            deltax *= 2;
            deltay *= 2;
        }
        supdelta = (deltay / 16) * 5;
    }
    
    // check for uncover code and fast exit
    c &= 077;
    if (c == 077)
    {
        setUncover (true);
        return;
    }

    if (uncover)
    {
        setUncover (false);
        switch (c)
        {
        case 010:   // backspace
            cx = (cx - deltax) & 0777;
            break;
        case 011:   // tab
            cx = (cx + deltax) & 0777;
            break;
        case 012:   // linefeed
            cy = (cy - deltay) & 0777;
            break;
        case 013:   // vertical tab
            cy = (cy + deltay) & 0777;
            break;
        case 014:   // form feed
            if (vertical)
            {
                cx = deltay - 1;
                if (reverse)
                {
                    cy = 512 - deltax;
                }
                else
                {
                    cy = 0;
                }
            }
            else
            {
                cy = 512 - deltay;
                if (reverse)
                {
                    cx = 512 - deltax;
                }
                else
                {
                    cx = 0;
                }
            }
            break;
        case 015:   // carriage return
            cx = margin;
            cy = (cy - deltay) & 0777;
            break;
        case 016:   // superscript
            cy = (cy + supdelta) & 0777;
            break;
        case 017:   // subscript
            cy = (cy - supdelta) & 0777;
            break;
        case 020:   // select M0
        case 021:   // select M1
        case 022:   // select M2
        case 023:   // select M3
        case 024:   // select M4
        case 025:   // select M5
        case 026:   // select M6
        case 027:   // select M7
            setCmem (c - 020);
            break;
        case 030:   // horizontal writing
            setVertical (false);
            break;
        case 031:   // vertical writing
            setVertical (true);
            break;
        case 032:   // forward writing
            setReverse (false);
            break;
        case 033:   // reverse writing
            setReverse (true);
            break;
        case 034:   // normal size writing
            setLarge (false);
            break;
        case 035:   // double size writing
            setLarge (true);
            break;
        default:
            break;
        }
    }
    else
    {
        const wxChar ch = rom01char[c + 64 * currentCharset];
        
        if (m_usefont && currentCharset <= 1)
        {
            SaveChar (currentX, currentY, ch, large);
            drawFontChar (currentX, currentY, ch);
        }
        else
        {
            autobs = SaveChar (currentX, currentY, ch, large);
            ptermDrawChar (currentX, currentY, currentCharset, c, autobs);
            cx = (cx + deltax) & 0777;
        }
    }
}


/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 0 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode0 (u32 d)
{
    int x, y;
    
    x = (d >> 9) & 0777;
    y = d & 0777;
    trace ("dot %d %d", x, y);
    ptermDrawPoint (x, y);
    currentX = x;
    currentY = y;
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 1 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode1 (u32 d)
{
    int x, y;
    
    x = (d >> 9) & 0777;
    y = d & 0777;
    trace ("lineto %d %d", x, y);
    ptermDrawLine (currentX, currentY, x, y);
    currentX = x;
    currentY = y;
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 2 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode2 (u32 d)
{
    int chaddr;

    // Store the word in PPT RAM
    WriteRAM (memaddr, d);
    WriteRAM (memaddr + 1, d >> 8);

    // memaddr is a PPT RAM address; convert it to a character memory address
    chaddr = memaddr - ReadRAMW (C2ORIGIN);
    if (chaddr < 0 || chaddr > 127 * 16)
    {
        trace ("memdata %04x to %04x", d & 0xffff, memaddr);
    }
    else
    {
        chaddr /= 2;
        if (((d >> 16) & 3) == 0)
        {
            // load data
            trace ("character memdata %06o to char word %04o", d & 0xffff, chaddr);
            plato_m23[chaddr] = d & 0xffff;
            ++chaddr;
        }
    }
    memaddr += 2;
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 3 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode3 (u32 d)
{
    if (traceF.Active ())
    {
        wxString t;
    
        t.Printf ("char %02o %02o %02o (", (d >> 12) & 077, 
                  (d >> 6) & 077, d & 077);
        t.Append (rom01char[(d >> 12) & 077]);
        t.Append (rom01char[(d >> 6) & 077]);
        t.Append (rom01char[d & 077]);
        t.Append (")");
        trace (t);
    }
    
    plotChar (d >> 12);
    plotChar (d >> 6);
    plotChar (d);
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 4 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode4 (u32 d)
{
    int x1, y1, x2, y2;
    
    if (modewords & 1)
    {
        trace ("block erase first word");
        mode4start = d;
        return;
    }
    x1 = (mode4start >> 9) & 0777;
    y1 = mode4start & 0777;
    x2 = (d >> 9) & 0777;
    y2 = d & 0777;
    
    trace ("block erase %d %d to %d %d", x1, y1, x2, y2);

    ptermBlockErase (x1, y1, x2, y2);
    currentX = x1;
    currentY = y1 - 15;

}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 5 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode5 (u32 d)
{
    trace ("mode5 %06o", d);
    progmode (d, M5ORIGIN);
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 6 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode6 (u32 d)
{
    trace ("mode6 %06o", d);
    progmode (d, M6ORIGIN);
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 7 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::mode7 (u32 d)
{
    trace ("mode7 %06o", d);
    progmode (d, M7ORIGIN);
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Mode 5/6/7 data word
**
**  Parameters:     Name        Description.
**                  d           Data word
**                  origin      Address of mode handler pointer
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::progmode (u32 d, int origin)
{
    // Load C/D/E with data word
    BC.reg.C = d >> 16;
    DE.reg.D = d >> 8;
    DE.reg.E = d;

    // Initialize the stack
    SP = INITSP;

    // Push the fake return address for "return to main loop" onto the
    // stack, as if we just did a CALL instruction
    WriteRAM (--SP, R_MAIN >> 8);
    WriteRAM (--SP, R_MAIN);

    // Set the start PC for the requested mode
    PC = ReadRAMW (origin);
    main8080a ();
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Plato mode keyboard input
**
**  Parameters:     Name        Description.
**                  keys        Plato key codes for station (see KEY macro)
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::ptermSendKey (u32 keys)
{
    int i, key;
    
    if (m_conn == NULL)
    {
        return;
    }

    for (i = 0; i < 32; i += 8)
    {
        key = (keys >> i) & 0xff;
        if (key != (None & 0xff))
        {
            ptermSendKey1 (key);
        }
    }
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Plato mode keyboard input
**
**  Parameters:     Name        Description.
**                  key         Plato key code for station
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::ptermSendKey1 (int key)
{
    char data[5];
    int len;
    
    if (m_conn == NULL)
    {
        return;
    }

    tracex ("key to plato %03o", key);
    debug ("key to plato %03o", key);

    if (m_conn->Ascii ())
    {
        // Assume one byte key code
        len = 1;
        if (key < 0200)
        {
            // Regular keyboard key
            key = asciiKeycodes[key];
            if (key == 0xff)
            {
                return;
            }
            if (m_flowCtrl)
            {
                // Do the keycode translation for the
                // "flow control enabled" coding rules.
                switch (key)
                {
                case 0x00:              // access
                    key = 0x1d;
                    len = 2;
                    break;
                case 0x05:              // shift-sub
                    key = 0x04;
                    len = 2;
                    break;
                case 0x0a:              // tab
                    key = 0x09;
                    break;
                case 0x09:              // shift-help
                    key = 0x0a;
                    break;
                case 0x11:              // shift-stop
                    key = 0x05;
                    break;
                case 0x17:              // shift-super
                    len = 2;
                    // fall through
                case 0x13:              // super
                    key = 0x17;
                    break;
                case 0x7c:              // apostrophe
                    key = 0x27;
                    break;
                case 0x27:              // #
                    key = 0x7c;
                    break;
                }
                data[0] = 033;          // store esc for 2 byte codes
            }                        
            data[len - 1] = Parity (key);
            if (tracePterm)
            {
                if (len == 1)
                {
                    tracex ("ascii mode key to plato 0x%02x", data[0] & 0xff);
                }
                else
                {
                    tracex ("ascii mode key to plato 0x%02x 0x%02x", data[0], data[1] & 0xff);
                }
            }
            m_conn->SendData (data, len);
            if (m_dumbTty)
            {
                // do local echoing
                m_conn->StoreWord (key);
            }
        }
        else if (!m_dumbTty)
        {
            if (key == xofkey)
            {
                if (!m_flowCtrl)
                {
                    return;
                }
                data[0] = Parity (ascxof);
                tracex ("ascii mode key to plato XOFF");
            }
            else if (key == xonkey)
            {
                if (!m_flowCtrl)
                {
                    return;
                }
                data[0] = Parity (ascxon);
                tracex ("ascii mode key to plato XON");
            }
            else
            {
                len = 3;
                data[0] = 033;
                data[1] = Parity (0100 + (key & 077));
                data[2] = Parity (0140 + (key >> 6));
                tracex ("ascii mode key to plato 0x%02x 0x%02x 0x%02x", 
                        data[0] & 0xff, data[1] & 0xff, data[2] & 0xff);
            }
            m_conn->SendData (data, len);
        }
    }
    else
    {
        data[0] = key >> 7;
        data[1] = 0200 | key;
        m_conn->SendData (data, 2);
    }
}

/*--------------------------------------------------------------------------
**  Purpose:        Process Plato mode keyboard input (multiple keys)
**
**  Parameters:     Name        Description.
**                  key         Array of integers of Plato key codes
**
**  Returns:        Nothing.
**
**------------------------------------------------------------------------*/
void PtermFrame::ptermSendKeys (const int key[])
{
    for (int i = 0; key[i] != -1; i++)
    {
        ptermSendKey1 (key[i]);
        m_pasteTimer.Start (ptermApp->m_charDelay, true);
    }
}
            
void PtermFrame::ptermSendTouch (int x, int y)
{
    char data[6];

    if (m_conn == NULL)
    {
        return;
    }

    if (m_sendFgt)
    {
        // Send fine grid touch code first
        data[0] = 033;
        data[1] = 0x1f;
        data[2] = 0x40 + (x & 0x1f);
        data[3] = 0x40 + ((x >> 5) & 0x0f);
        data[4] = 0x40 + (y & 0x1f);
        data[5] = 0x40 + ((y >> 5) & 0x0f);
        m_conn->SendData (data, 6);
    }
    
    x /= 32;
    y /= 32;

    ptermSendKey1 (0x100 | (x << 4) | y);
}

void PtermFrame::ptermSendExt (int key)
{
    char data[3];

    if (m_conn == NULL)
    {
        return;
    }

    if (m_conn->Ascii ())
    {
        // Send external key
        data[0] = 033;
        data[1] = 0x40 | (key & 0x3f);
        data[2] = 0x68 | ((key >> 6) & 0x03);
        m_conn->SendData (data, 3);
    }
    else
    {
        data[0] = 0x04 | ((key >> 7) & 0x01);
        data[1] = 0x80 | (key & 0x7f);
        m_conn->SendData (data, 2);
    }
    
}

#if 0
int PtermFrame::Parity (int key)
{
    int i;
    int p = 0;
    key &= 0177;
    for (i = 0; i < 8; i++)
    {
        if (key & (1 << i))
        {
            p ^= 0200;
        }
    }
    return key; // | p;
}
#endif

void PtermFrame::ptermSetStation (int station, bool showtitle, bool showstatus)
{

    //this routine is called only when connection is initially made
    //see ProcessPlatoMetaData for the dynamic code.
    wxString l_hostname;
    wxString l_station;
    wxString l_str;

    SetCursor (wxNullCursor);
    m_station = station;

    //build string for frame name
    if (m_hostName.IsEmpty ())
    {
        l_hostname = wxT ("");
    }
    else
    {
        l_hostname = m_hostName.c_str ();
    }
    if (station < 0)
    {
        l_station = wxT ("ASCII");
    }
    else
    {
        l_station.Printf (wxT ("%d-%d"), station >> 5, station & 31);
    }

    // only show what the user wants to see in title bar
    if (showtitle)
    {
        l_str = wxT ("");
        if (ptermApp->m_showHost)
        {
            l_str.Printf (wxT ("%s "), l_hostname);
        }
        if (ptermApp->m_showStation)
        {
            l_str.Append (l_station);
        }
        ptermSetName (l_str);
    }

    //always show host and station in status bar
    if (showstatus)
    {
        l_str.Printf (wxT ("%s "), l_hostname);
        l_str.Append (l_station);
        ptermSetStatus (l_str);
    }
}

void PtermFrame::ptermShowTrace ()
{
    if (m_statusBar != NULL)
    {
        if (tracePterm)
        {
            m_statusBar->SetStatusText (_(" Trace "), STATUS_TRC);
        }
        else if (m_conn != NULL && m_conn->GswActive ())
        {
            // Display a musical note.
            m_statusBar->SetStatusText (wxT ("\u266C"), STATUS_TRC);
        }
        else if (ptermApp->m_platoKb)
        {
            m_statusBar->SetStatusText (_(" PLATO keyboard "), STATUS_TRC);
        }
        else
        {
            m_statusBar->SetStatusText (wxT (""), STATUS_TRC);
        }
    }
}

// Save a character into the character map used for text copy.
// Returns True if this character is an auto-backspaced accent.
bool PtermFrame::SaveChar (int x, int y, wxChar c, bool large_p)
{
    static int prevx = -100, prevy = -100;
    const int cx = (vertical) ? y : x;
    const int cy = (vertical) ? x : y;
    const int px = (vertical) ? prevy : prevx;
    const int py = (vertical) ? prevx : prevy;
    const int size = (large) ? 2 : 1;
    const int savex = x, savey = y;
    int dx, dy, i;
    bool autobs = false;
    
#if 0
    wxString s;
    
    s.Printf ("saving at %d, %d: %x: ", x, y, (int) c);
    s.Append (c);
    trace (s);
#endif
    // Check for autobackspace BEFORE rounding the x/y position down
    // to the text grid (coarse grid) boundaries.  Note that 
    // auto-backspace is defined in terms of the current writing direction.
    // Also, it's only auto-backspace if the current writing mode is
    // write or erase, not if it is inverse or rewrite.  
    dx = cx - px;
    dy = cy - py;

    if ((wemode & 2) != 0 &&     // mode erase or write
        (dx == 0 || dx == 4 * size) &&
        dy >= -10 * size && 
        dy <= 11 * size)
    {
        // Mode and coordinates look like it might be an autobackspace.
        // So assume it is, and save this character accordingly as the
        // second or (occasionally) third character of a sequence.
        // The actual mapping of such sequences to the correct code is
        // done when we fetch the text from the selected region, e.g., 
        // for a "Copy text" operation.
        
        // dx == 4 applies only to the ae and AE ligatures.  Conversely,
        // there are no dx == 0 combinations that have e or E as the
        // second character.
        if (c == 'e' || c == 'E')
        {
            autobs = (dx == 4 * size);
        }
        else
        {
            autobs = (dx == 0);
        }
            
        // Convert an accent character to its "combining diacritical" form.
        i = accent.find (c);
        if (i != wxString::npos)
        {
            c = combining_accent[i];
        }
        if (c == L'\u0301')
        {
            // Acute accent.  If dy is negative, that's actually a cedilla
            // (on classic).
            if (dy < 0)
            {
                c = L'\u0327';
            }
        }
    }
            
    // Convert the current x/y to a coarse grid position, and save
    // the current character into the textmap array.
    // Note that the Y coordinate starts with 0 for the bottom
    // line, just as the fine grid Y coordinate does.
    // The X/Y are taken from the previous X/Y if we're doing autobackspace,
    // otherwise we might end up in the wrong cell if dy is negative.
    if (autobs)
    {
        x = prevx;
        y = prevy;
    }
    x /= 8;
    y /= 16;

    // It if seemed autobackspaced but there is no "primary" character
    // yet, then it isn't actually.
    if (textmap[y * 64 + x][0] == '\0')
    {
        autobs = false;
    }
    
    if (autobs)
    {
        // This is an autobackspaced character, so put it into the 
        // second or third slot of the textmap entry.  Special case:
        // if the previous one is grave and this is acute, replace the
        // existing accent by hacek, because that's how hacek is sent 
        // on classic terminals.
        if (textmap[y * 64 + x][1] == '\0')
        {
            textmap[y * 64 + x][1] = c;
        }
        else if (c == L'\u0301' && textmap[y * 64 + x][1] == L'\u0300')
        {
            textmap[y * 64 + x][1] = L'\u030c';
        }
        else
        {
            textmap[y * 64 + x][2] = c;
        }
    }
    else
    {
        textmap[y * 64 + x][0] = c;
        textmap[y * 64 + x][1] = '\0';
        textmap[y * 64 + x][2] = '\0';
        textmap[y * 64 + x][3] = '\0';

        // Also save the current x/y as previous
        prevx = savex;
        prevy = savey;
    }
    
    if (large_p)
    {
        x = (x + 1) & 077;
        textmap[y * 64 + x][0] = '\0';
        textmap[y * 64 + x][1] = '\0';
        textmap[y * 64 + x][2] = '\0';
        textmap[y * 64 + x][3] = '\0';
        y = (y + 1) & 037;
        textmap[y * 64 + x][0] = '\0';
        textmap[y * 64 + x][1] = '\0';
        textmap[y * 64 + x][2] = '\0';
        textmap[y * 64 + x][3] = '\0';
        x = (x - 1) & 077;
        textmap[y * 64 + x][0] = '\0';
        textmap[y * 64 + x][1] = '\0';
        textmap[y * 64 + x][2] = '\0';
        textmap[y * 64 + x][3] = '\0';
    }

    return autobs;
}

// Return the content of the currently selected text region, as a wxString.
// If "url" is true, leading and trailing spaces are trimmed off, and
// for a multi-line region the lines are concatenated without a newline.
// Also, the resulting string is run through a URL processor to escape
// any characters not valid as literal characters in a URL.
// If "url" is false (default), spaces are kept, and for a multi-line
// region the lines are separated by newline (CRLF in the Windows case).
wxString PtermFrame::GetRegionText (bool url) const
{
    int i, j, k;
    wxString text;
    wxString c;
    
    if (m_regionHeight == 0 || m_regionWidth == 0)
    {
        debug ("GetRegionText: empty region");
        return text;
    }
    
    // regionX and regionY are the lowest coordinate corner of
    // the copy region, i.e., the lower left corner, expressed in
    // 0-based coarse grid coordinates.
    for (i = m_regionY + m_regionHeight - 1; i >= m_regionY; i--)
    {
        wxString line;
        
        for (j = m_regionX; j < m_regionX + m_regionWidth; j++)
        {
            c = textmap[i * 64 + j];
            // Each textmap entry can be up to 3 characters, for example
            // for accent marks (stored as combining accent), or for
            // various overstruck special characters like universal delimiter.
            // So if the entry is longer than one character code, search
            // for match in the combinations we know.
            if (c.Len () > 1)
            {
                for (k = 0; 
                     k < sizeof (autobsmap) / sizeof (autobsmap[0]);
                     k++)
                {
                    if (c == autobsmap[k].s)
                    {
                        c = autobsmap[k].c;
                        break;
                    }
                }
                
                // If we didn't find a match, keep only the first
                // character unless the others are accent marks.
                if (c.Len () > 1)
                {
                    if (!(c[1] >= L'\u0300' && c[1] <= L'\u033f' &&
                          (c[2] == '\0' ||
                           (c[2] >= L'\u0300' && c[2] <= L'\u033f'))))
                    {
                        c = c[0];
                    }
                }
            }
            line.Append (c);
            // Copyright takes 3 positions: left paren, copyright body,
            // right paren.  Copyright body is recognized as an autobackspaced
            // form above (for classic -- ascii sends it as one character).
            // But we need to strip off the ( ).
            line.Replace (L"(\u00A9)", L"\u00A9");
        }
        if (url)
        {
            // Strip off leading/trailing spaces
            line.Trim (true);
            line.Trim (false);
        }
        else
        {
            if (m_regionHeight > 1)
            {
#if defined (__WXMSW__)
                // Windows really likes a CRLF and it must be done
                // on two separate function calls.  (really? gpk)
                line.Append (wxT ('\r'));
#endif
                line.Append (wxT ('\n'));
            }
        }
        text.Append (line);
    }
    if (url)
    {
        wxURI u (text);
        text = u.BuildURI ();
    }
#ifdef DEBUG
    debug ("GetRegionText (%d) returns:", (int) url);
    debug (text);
#endif
    return text;
}

void PtermFrame::ClearRegion (void)
{
    // Cancel any region selection
    if (m_regionHeight != 0 || m_regionWidth != 0)
    {
        m_regionHeight = 0;
        m_regionWidth = 0;
        menuBar->Enable (Pterm_Copy, false);
        menuBar->Enable (Pterm_Exec, false);
        menuBar->Enable (Pterm_MailTo, false);
        menuBar->Enable (Pterm_SearchThis, false);
        menuPopup->Enable (Pterm_Copy, false);
        menuPopup->Enable (Pterm_Exec, false);
        menuPopup->Enable (Pterm_MailTo, false);
        menuPopup->Enable (Pterm_SearchThis, false);
        if (m_statusBar != NULL)
        {
            m_statusBar->SetStatusText (wxT (""), STATUS_TIP);
        }
        Refresh (false);
    }
}

void PtermFrame::UpdateRegion (int x, int y, int mouseX, int mouseY)
{
    int x1, x2, y1, y2, mtoler;
    wxString msg;
    int scfx, scgx, ecfx, ecgx;

    mtoler = (ptermApp->m_DisableMouseDrag) ? 512 : MouseTolerance;
    
    if (abs (x - mouseX) > mtoler || abs (y - mouseY) > mtoler)
    {
        // It was a mouse drag (region selection)
        // rather than a click
        if (mouseX > x)
        {
            x1 = x;
            x2 = mouseX;
        }
        else
        {
            x1 = mouseX;
            x2 = x;
        }
        if (mouseY > y)
        {
            y1 = y;
            y2 = mouseY;
        }
        else
        {
            y1 = mouseY;
            y2 = y;
        }
        x1 = BOUND (x1);
        x2 = BOUND (x2);
        y1 = BOUND (y1);
        y2 = BOUND (y2);
        m_regionX = x1 / 8;
        m_regionY = y1 / 16;
        m_regionWidth = (x2 + 1 - (m_regionX * 8)) / 8;
        m_regionHeight = (y2 - (m_regionY * 16)) / 16 + 1;
        menuBar->Enable (Pterm_Copy, (m_regionWidth > 0));
        menuBar->Enable (Pterm_Exec, (m_regionWidth > 0)); 
        menuBar->Enable (Pterm_MailTo, (m_regionWidth > 0)); 
        menuBar->Enable (Pterm_SearchThis, (m_regionWidth > 0)); 
        menuPopup->Enable (Pterm_Copy, (m_regionWidth > 0));
        menuPopup->Enable (Pterm_Exec, (m_regionWidth > 0)); 
        menuPopup->Enable (Pterm_MailTo, (m_regionWidth > 0)); 
        menuPopup->Enable (Pterm_SearchThis, (m_regionWidth > 0)); 
        debug ("region %d %d size %d %d", m_regionX, m_regionY,
               m_regionWidth, m_regionHeight);
        Refresh (false);
        return;
    }
    if (m_regionWidth == 0 && m_regionHeight == 0)
        msg = wxT ("");
    else
    {
        scfx = 256 - 4 * m_regionWidth;
        ecfx = 256 + 4 * m_regionWidth;
        if ((scfx % 8) == 0)
        {
            scgx = (scfx / 8) + 1;
            ecgx = scgx + m_regionWidth - 1;
            msg.Printf (wxT ("%dx%d, f=%d-%d, g=%02d-%02d"),
                        m_regionWidth, m_regionHeight, 
                        scfx, ecfx, scgx, ecgx);
        }
        else
            msg.Printf (wxT ("%dx%d, f=%d-%d"),
                        m_regionWidth, m_regionHeight, scfx, ecfx);
    }
    if (m_statusBar != NULL)
    {
        m_statusBar->SetStatusText (msg, STATUS_TIP);
    }
}


// This emulates the "ROM resident".  Return values:
// 0: PC is not special (not in resident), proceed normally.
// 1: PC is ROM function entry point, it has been emulated,
//    do a RET now.
// 2: PC is either the 8080 emulation exit magic value, or R_INIT,
//    or an invalid resident value.  Exit 8080 emulation.
int PtermFrame::check_pc8080a (void)
{
    int x, y, cp, c, x2, y2;
    
    if (PC < WORKRAM)
    {
        tracex ("Resident call %04x DE=%04x HL=%04x", 
                PC, DE.pair, HL.pair);
    }
    
    switch (PC)
    {
    case R_MAIN:
        // "r.main" -- fake return address value used as the return
        // address for invocations of the mode 5/6/7 handler code
        return 2;

    case R_INIT:
        // r.init -- TBD
        return 2;
        
    case R_DOT:
        x = HL.pair;
        y = DE.pair;
        ptermDrawPoint (x, y);
        currentX = x;
        currentY = y;
        return 1;
        
    case R_LINE:
        x = HL.pair;
        y = DE.pair;
        ptermDrawLine (currentX, currentY, x, y);
        currentX = x;
        currentY = y;
        return 1;

    case R_CHARS:
        // draw chars ending at 07700
        cp = HL.pair;
        c = RAM[cp++];
        for (;;)
        {
            if (c == 077 && RAM[cp] == 0)
            {
                break;
            }
            plotChar (c);
            c = RAM[cp++];
        }
        return 1;
        
    case R_BLOCK:
        // block erase
        cp = HL.pair;
        x = ReadRAMW (cp);
        y = ReadRAMW (cp + 2);
        x2 = ReadRAMW (cp + 4);
        y2 = ReadRAMW (cp + 6);
        ptermBlockErase (x, y, x2, y2);
        return 1;
        
    case R_INPX:
        HL.pair = currentX;
        return 1;

    case R_INPY:
        HL.pair = currentY;
        return 1;
        
    case R_OUTX:
        currentX = HL.pair;
        return 1;
        
    case R_OUTY:
        currentY = HL.pair;
        return 1;
        
    case R_XMIT:
        // send key in HL
        return 1;
        
    case R_MODE:
        // set mode from L
        if (HL.reg.L & 1)
        {
            ptermFullErase ();
        }
        modexor = false;
        mode = (HL.reg.L >> 1) & 037;
        return 1;
        
    case R_STEPX:
        currentX = (currentX + ((RAM[M_DIR] & 2) ? -1 : 1)) & 0777;
        return 1;
        
    case R_STEPY:
        currentY = (currentY + ((RAM[M_DIR] & 1) ? -1 : 1)) & 0777;
        return 1;
        
    case R_WE:
        ptermDrawPoint (currentX, currentY);
        return 1;
        
    case R_DIR:
        RAM[M_DIR] = HL.reg.L & 3;
        return 1;
        
    case R_INPUT:
        // r.input
        return 1;
        
    case R_SSH:
        // r.ssh
        return 1;
        
    case R_CCR:
        RAM[M_CCR] = HL.reg.L;
        return 1;
        
    case R_EXTOUT:
        // r.extout
        return 1;
        
    case R_EXEC:
        // r.exec
        return 1;
        
    case R_GJOB:
        // r.gjob
        return 1;
        
    case R_XJOB:
        // r.xjob
        return 1;
        
    default:
        if (PC < WORKRAM)
        {
            // Wild jump into ROM resident, quit
            fprintf (stderr, "Wild jump to %04x\n", PC);
            return 2;
        }
        else
        {
            // Plain old RAM PC -- keep executing
            return 0;
        }
    }
}

/*******************************************************************************

input8080a:
----------

   This case switches on the content of data.  If the content of data is equal
to 1, ...
The program counter, named PC, is incremented by one.


Input:
-----

   data - This variable is the byte after the INp instruction, RAM[PC].


Output:
------

   This variable is the data that has been read or calculated from the input
device and returned to the main simulator core.


Note:
----

   This function is called directly by main8080a.

*******************************************************************************/

/*
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    NOTE:  this routine obviously needs some more work, as it has
    paths thru it (case 1, 2) that can pass thru the routine without
    setting the value of -retval-.  --bg 2010/06/18
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/
u8 PtermFrame::input8080a (u8 data)
{

    /***********************************************************************
    * Declaration:
    * -----------
    *
    *   retval - This variable contains data read from an input device.
    ***********************************************************************/
    u8 retval;


    switch (data)
    {

    /***********************************************************************
    *   If the value of the 8080a's memory, RAM[PC], is equal to 1 or 2,
    * then first or second player input is requested.
    ***********************************************************************/
        case 1:
        case 2:
            retval = 0;
            break;

    /***********************************************************************
    *   If the value of the 8080a's memory, RAM[PC], is not equal to any of
    * the above cases, then the program requested bad input data. Debug
    * information containing the value of the data is printed to STDOUT.
    ***********************************************************************/
        default:
            printf ("INp BAD -> Data = %d\n", data);

            retval = 0;
            break;
    }


    /***********************************************************************
    *   This keyword returns the read input data to the main simulator core.
    ***********************************************************************/
    return retval;
}



/*******************************************************************************

output8080a:
-----------

   This case switches on the contents of the data.  If the content of data is
equal to 2, ...
If the content of data is any other value, then the request is bad and 
debugging information is printed to STDOUT.


Input:
-----

   data - This variable is the byte after the OUTp instruction, RAM[PC].
   acc - This variable is the accumulator, which is the 8080a's A register.


Note:
----

   This function is called directly by main8080a.

*******************************************************************************/

void PtermFrame::output8080a (u8 data, u8 acc)
{
    switch (data)
    {

    /***********************************************************************
    *   If the value of the 8080a's memory, RAM[PC], is equal to 2, the
    * content of the A register is moved into the left shift amount.
    ***********************************************************************/
        case 2:
            break;


    /***********************************************************************
    *   If the value of the 8080a's memory, RAM[PC], is not equal to any of
    * the above cases, then the program requested bad output data.  Debug
    * information containing the value of the data and the accumulator are
    * printed to STDOUT.
    ***********************************************************************/
        default:
            printf ("OUTp BAD -> Data = %d   A = %d\n", data, acc);
            break;
    }
}


// ----------------------------------------------------------------------------
// PtermPrefDialog
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (PtermPrefDialog, wxDialog)
    EVT_CLOSE (PtermPrefDialog::OnClose)
    EVT_BUTTON (wxID_ANY, PtermPrefDialog::OnButton)
    EVT_CHECKBOX (wxID_ANY, PtermPrefDialog::OnCheckbox)
    EVT_LISTBOX (wxID_ANY, PtermPrefDialog::OnSelect)
    EVT_LISTBOX_DCLICK (wxID_ANY, PtermPrefDialog::OnDoubleClick)
    EVT_TEXT (wxID_ANY, PtermPrefDialog::OnChange)
    EVT_COMBOBOX (wxID_ANY, PtermPrefDialog::OnComboSelect)
    END_EVENT_TABLE ();

PtermPrefDialog::PtermPrefDialog (PtermFrame *parent, wxWindowID id, const wxString &title, wxPoint pos, wxSize size)
    : wxDialog (parent, id, title, pos, size)
{

    // static ui objects, note dynamic controls, e.g. those that hold values or require event processing are declared above
//  wxNotebook* tabPrefsDialog;
    //tab0
    wxScrolledWindow* tab0;
    wxStaticText* lblExplain0;
//  wxListBox* lstProfiles;
    wxStaticText* lblProfileActionExplain;
//  wxButton* btnSave;
//  wxButton* btnLoad;
//  wxButton* btnDelete;
//  wxStaticText* lblProfileStatusMessage;
    wxStaticText* lblNewProfileExplain;
    wxStaticText* lblNewProfile;
//  wxTextCtrl* txtProfile;
//  wxButton* btnAdd;
    //tab1
    wxScrolledWindow* tab3;
    wxStaticText* lblExplain3;
//  wxCheckBox* chkConnectAtStartup;
    wxStaticText* lblShellFirst;
//  wxTextCtrl* txtShellFirst;
    wxStaticText* lblDefaultHost;
//  wxTextCtrl* txtDefaultHost;
    wxStaticText* lblDefaultPort;
//  wxComboBox* cboDefaultPort;
    wxStaticText* lblExplainPort;
    //tab2
    wxScrolledWindow* tab1;
    wxStaticText* lblExplain1;
//  wxCheckBox* chkShowSignon;
//  wxCheckBox* chkShowSysName;
//  wxCheckBox* chkShowHost;
//  wxCheckBox* chkShowStation;
    //tab3
    wxScrolledWindow* tab2;
    wxStaticText* lblExplain2;
//  wxCheckBox* chkSimulate1200Baud;
//  wxCheckBox* chkEnableGSW;
//  wxCheckBox* chkEnableNumericKeyPad;
//  wxCheckBox* chkIgnoreCapLock;
//  wxCheckBox* chkUsePLATOKeyboard;
//  wxCheckBox* chkUseAccelerators;
//  wxCheckBox* chkEnableBeep;
//  wxCheckBox* chkDisableShiftSpace;
//  wxCheckBox* chkDisableMouseDrag;
    //tab4
    wxScrolledWindow* tab4;
    wxStaticText* lblExplain4;
//  wxCheckBox* chkZoom200;
//  wxCheckBox* chkStatusBar;
//  wxCheckBox* chkMenuBar;
//  wxCheckBox* chkDisableColor;
//  wxButton* btnFGColor; // windows
//  wxBitmapButton* btnFGColor; // other
    wxStaticText* lblFGColor;
//  wxButton* btnBGColor; // windows
//  wxBitmapButton* btnBGColor; // other
    wxStaticText* lblBGColor;
    wxStaticText* lblExplainColor;
    //tab5
    wxScrolledWindow* tab5;
    wxStaticText* lblExplain5;
    wxStaticText* lblCharDelay;
//  wxTextCtrl* txtCharDelay;
    wxStaticText* lblCharDelay2;
    wxStaticText* lblLineDelay;
//  wxTextCtrl* txtLineDelay;
    wxStaticText* lblLineDelay2;
    wxStaticText* lblAutoNewLine;
//  wxComboBox* cboAutoLF;
    wxStaticText* lblAutoNewLine2;
//  wxCheckBox* chkSplitWords;
//  wxCheckBox* chkSmartPaste;
//  wxCheckBox* chkConvertDot7;
//  wxCheckBox* chkConvert8Spaces;
//  wxCheckBox* chkTutorColor;
    //tab6
    wxScrolledWindow* tab6;
    wxStaticText* lblEmail;
    wxStaticText* lblSearchURL;
//  wxTextCtrl* txtEmail;
//  wxTextCtrl* txtSearchURL;
    //button  bar
//  wxButton* btnOK;
//  wxButton* btnCancel;
//  wxButton* btnDefaults;
    

    
    // ui object creation / placement, note initialization of values is below
    wxBoxSizer* bSizer1;
    bSizer1 = new wxBoxSizer (wxVERTICAL);

    //notebook
    tabPrefsDialog = new wxNotebook (this, wxID_ANY, wxDefaultPosition,
                                     wxDefaultSize, wxNB_TOP);
    tabPrefsDialog->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));

    //tab0
    tab0 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY,
                                 wxDefaultPosition, wxDefaultSize,
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab0->SetScrollRate (5, 5);
    tab0->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));

    wxBoxSizer* page0;
    page0 = new wxBoxSizer (wxVERTICAL);
    lblExplain0 = new wxStaticText (tab0, wxID_ANY,
                                    _("Use this page to manage your connection preference profiles."),
                                    wxDefaultPosition, wxDefaultSize, 0);
    page0->Add (lblExplain0, 0, wxALL, 5);
    lstProfiles = new wxListBox (tab0, wxID_ANY, wxDefaultPosition,
                                 wxSize (-1, -1), 0, NULL, wxLB_SORT); 
    page0->Add (lstProfiles, 1, wxALL | wxEXPAND | wxALIGN_TOP, 5);

    wxFlexGridSizer* fgs01;
    fgs01 = new wxFlexGridSizer (4, 1, 0, 0);
    fgs01->SetFlexibleDirection (wxVERTICAL);
    lblProfileActionExplain = new wxStaticText (tab0, wxID_ANY,
                                                _("Select a profile above, then click a button below."),
                                                wxDefaultPosition,
                                                wxDefaultSize, 0);
    fgs01->Add (lblProfileActionExplain, 0, wxALL, 5);

    wxFlexGridSizer* fgs011;
    fgs011 = new wxFlexGridSizer (2, 4, 0, 0);
    btnSave = new wxButton (tab0, wxID_ANY, _("Save"), wxDefaultPosition,
                            wxDefaultSize, 0);
    btnSave->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs011->Add (btnSave, 0, wxBOTTOM | wxRIGHT | wxLEFT, 5);
    btnLoad = new wxButton (tab0, wxID_ANY, _("Load"), wxDefaultPosition,
                            wxDefaultSize, 0);
    btnLoad->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs011->Add (btnLoad, 0, wxBOTTOM | wxRIGHT | wxLEFT, 5);
    btnDelete = new wxButton (tab0, wxID_ANY, _("Delete"), wxDefaultPosition,
                              wxDefaultSize, 0);
    btnDelete->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs011->Add (btnDelete, 0, wxRIGHT | wxLEFT, 5);
    lblProfileStatusMessage = new wxStaticText (tab0, wxID_ANY,
                                                _("Profile saved."),
                                                wxDefaultPosition,
                                                wxDefaultSize, 0);
    lblProfileStatusMessage->SetFont (wxFont (10, 74, 90, 90, false,
                                              wxT (SSFONT)));
    fgs011->Add (lblProfileStatusMessage, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    fgs01->Add (fgs011, 1, 0, 5);
    lblNewProfileExplain = new wxStaticText (tab0, wxID_ANY,
                                             _("Or, enter the name of a new profile below and click Add."),
                                             wxDefaultPosition, wxDefaultSize,
                                             0);
    fgs01->Add (lblNewProfileExplain, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);

    wxFlexGridSizer* fgs012;
    fgs012 = new wxFlexGridSizer (1, 3, 0, 0);
    fgs012->AddGrowableCol (1);
    fgs012->SetFlexibleDirection (wxHORIZONTAL);
    lblNewProfile = new wxStaticText (tab0, wxID_ANY, _("Profile"),
                                      wxDefaultPosition, wxDefaultSize, 0);
    fgs012->Add (lblNewProfile, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    txtProfile = new wxTextCtrl (tab0, wxID_ANY, wxT (""), wxDefaultPosition,
                                 wxDefaultSize, 0);
    txtProfile->SetMinSize (wxSize (310, -1));
    fgs012->Add (txtProfile, 1, wxALL | wxEXPAND, 5);
    btnAdd = new wxButton (tab0, wxID_ANY, _("Add"), wxDefaultPosition,
                           wxDefaultSize, wxBU_EXACTFIT);
    fgs012->Add (btnAdd, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    fgs01->Add (fgs012, 1, wxEXPAND | wxALIGN_TOP, 5);
    page0->Add (fgs01, 0, wxEXPAND, 5);
    tab0->SetSizer (page0);
    tab0->Layout ();
    page0->Fit (tab0);
    tabPrefsDialog->AddPage (tab0, _("Profiles"), true);

    //tab1
    tab1 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY, wxDefaultPosition,
                                 wxDefaultSize,
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab1->SetScrollRate (5, 5);

    wxFlexGridSizer* page1;
    page1 = new wxFlexGridSizer (3, 1, 0, 0);
    page1->AddGrowableCol (0);
    page1->AddGrowableRow (1);
    page1->SetFlexibleDirection (wxVERTICAL);
    lblExplain1 = new wxStaticText (tab1, wxID_ANY,
                                    _("Settings on this page specify where PLATO is on the internet."),
                                    wxDefaultPosition, wxDefaultSize, 0);
    page1->Add (lblExplain1, 1, wxALL, 5);

    wxBoxSizer* bs11;
    bs11 = new wxBoxSizer (wxVERTICAL);
    chkConnectAtStartup = new wxCheckBox (tab1, wxID_ANY,
                                          _("Connect at startup"),
                                          wxDefaultPosition, wxDefaultSize, 0);
    chkConnectAtStartup->SetValue (true);
    bs11->Add (chkConnectAtStartup, 0, wxTOP | wxLEFT, 5);

    wxFlexGridSizer* fgs111;
    fgs111 = new wxFlexGridSizer (3, 2, 0, 0);
    fgs111->AddGrowableCol (1);
    fgs111->SetNonFlexibleGrowMode (wxFLEX_GROWMODE_SPECIFIED);
    lblShellFirst = new wxStaticText (tab1, wxID_ANY, _("Run this first"),
                                      wxDefaultPosition, wxDefaultSize, 0);
    lblShellFirst->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs111->Add (lblShellFirst, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    txtShellFirst = new wxTextCtrl (tab1, wxID_ANY, wxT (""),
                                    wxDefaultPosition, wxDefaultSize, 0);
    txtShellFirst->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    txtShellFirst->SetMaxLength (255); 
    fgs111->Add (txtShellFirst, 0,
                 wxALIGN_CENTER_VERTICAL | wxALL, 5);
    lblDefaultHost = new wxStaticText (tab1, wxID_ANY, _("Default Host"),
                                       wxDefaultPosition, wxDefaultSize, 0);
    fgs111->Add (lblDefaultHost, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    txtDefaultHost = new wxTextCtrl (tab1, wxID_ANY, wxT ("cyberserv.org"),
                                     wxDefaultPosition, wxSize (-1, -1), 0);
    txtDefaultHost->SetMaxLength (100); 
    txtDefaultHost->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs111->Add (txtDefaultHost, 1, 
                 wxALL | wxALIGN_CENTER_VERTICAL, 5);
    lblDefaultPort = new wxStaticText (tab1, wxID_ANY, _("Default Port*"),
                                       wxDefaultPosition, wxDefaultSize, 0);
    fgs111->Add (lblDefaultPort, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);

    wxIntegerValidator<long> portval;
    portval.SetRange (1, 65535);
    
    cboDefaultPort = new wxComboBox (tab1, wxID_ANY, wxT ("5004"),
                                     wxDefaultPosition, wxSize (75, -1),
                                     0, NULL, 0, portval);
    cboDefaultPort->Append (wxT ("5004"));
    cboDefaultPort->Append (wxT ("8005"));
    fgs111->Add (cboDefaultPort, 0, wxALL, 5);
    bs11->Add (fgs111, 1, wxEXPAND | wxALIGN_TOP, 5);
    page1->Add (bs11, 1, wxEXPAND | wxALIGN_TOP, 0);
    lblExplainPort = new wxStaticText (tab1, wxID_ANY,
                                       _("* NOTE: 5004=Classic, 8005=Color Terminal"),
                                       wxDefaultPosition, wxDefaultSize, 0);
    page1->Add (lblExplainPort, 0, wxALL, 5);
    tab1->SetSizer (page1);
    tab1->Layout ();
    page1->Fit (tab1);
    tabPrefsDialog->AddPage (tab1, _("Connection"), true);

    //tab2
    tab2 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY, wxDefaultPosition,
                                 wxDefaultSize,
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab2->SetScrollRate (5, 5);
    tab2->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));

    wxBoxSizer* page2;
    page2 = new wxBoxSizer (wxVERTICAL);
    lblExplain2 = new wxStaticText (tab2, wxID_ANY,
                                    _("Settings on this page configure the text shown in the window title."),
                                    wxDefaultPosition, wxDefaultSize, 0);
    page2->Add (lblExplain2, 0, wxALL, 5);
    chkShowSignon = new wxCheckBox (tab2, wxID_ANY,
                                    _("Show name/group in frame title"),
                                    wxDefaultPosition, wxDefaultSize, 0);
    chkShowSignon->SetValue (true);
    page2->Add (chkShowSignon, 0, wxALL, 5);
    chkShowSysName = new wxCheckBox (tab2, wxID_ANY,
                                     _("Show system name in frame title"),
                                     wxDefaultPosition, wxDefaultSize, 0);
    chkShowSysName->SetValue (true);
    page2->Add (chkShowSysName, 0, wxALL, 5);
    chkShowHost = new wxCheckBox (tab2, wxID_ANY,
                                  _("Show host name in frame title"),
                                  wxDefaultPosition, wxDefaultSize, 0);
    chkShowHost->SetValue (true);
    page2->Add (chkShowHost, 0, wxALL, 5);
    chkShowStation = new wxCheckBox (tab2, wxID_ANY,
                                     _("Show site-station numbers in frame title"),
                                     wxDefaultPosition, wxDefaultSize, 0);
    chkShowStation->SetValue (true);
    page2->Add (chkShowStation, 0, wxALL, 5);
    tab2->SetSizer (page2);
    tab2->Layout ();
    page2->Fit (tab2);
    tabPrefsDialog->AddPage (tab2, _("Title"), false);

    //tab3
    tab3 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY, wxDefaultPosition,
                                 wxDefaultSize, 
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab3->SetScrollRate (5, 5);

    wxBoxSizer* page3;
    page3 = new wxBoxSizer (wxVERTICAL);
    lblExplain3 = new wxStaticText (tab3, wxID_ANY,
                                    _("Settings on this page let you fine-tune your PLATO experience."),
                                    wxDefaultPosition, wxDefaultSize, 0);
    page3->Add (lblExplain3, 0, wxALL, 5);
    chkSimulate1200Baud = new wxCheckBox (tab3, wxID_ANY,
                                          _("Simulate 1260 baud"),
                                          wxDefaultPosition, wxDefaultSize, 0);
    page3->Add (chkSimulate1200Baud, 0, wxALL, 5);
    chkEnableGSW = new wxCheckBox (tab3, wxID_ANY,
                                   _("Enable GSW (not in ASCII)"),
                                   wxDefaultPosition, wxDefaultSize, 0);
    chkEnableGSW->SetValue (true);
    page3->Add (chkEnableGSW, 0, wxALL, 5);
    chkEnableNumericKeyPad = new wxCheckBox (tab3, wxID_ANY,
                                             _("Enable numeric keypad for arrow operation"),
                                             wxDefaultPosition,
                                             wxDefaultSize, 0);
    chkEnableNumericKeyPad->SetValue (true);
    page3->Add (chkEnableNumericKeyPad, 0, wxALL, 5);
    chkIgnoreCapLock = new wxCheckBox (tab3, wxID_ANY, _("Ignore CAPS LOCK"),
                                       wxDefaultPosition, wxDefaultSize, 0);
    chkIgnoreCapLock->SetValue (true);
    page3->Add (chkIgnoreCapLock, 0, wxALL, 5);
    chkUsePLATOKeyboard = new wxCheckBox (tab3, wxID_ANY,
                                          _("Use real PLATO keyboard"),
                                          wxDefaultPosition, wxDefaultSize, 0);
    page3->Add (chkUsePLATOKeyboard, 0, wxALL, 5);
    chkUseAccelerators = new wxCheckBox (tab3, wxID_ANY,
                                         _("Enable control-key menu accelerators"),
                                         wxDefaultPosition, wxDefaultSize, 0);
    chkUseAccelerators->SetValue (true);
    chkUseAccelerators->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
#if defined (__WXMAC__)
    chkUseAccelerators->Disable ();
#endif
    page3->Add (chkUseAccelerators, 0, wxALL, 5);
    chkEnableBeep = new wxCheckBox (tab3, wxID_ANY, _("Enable -beep-"),
                                    wxDefaultPosition, wxDefaultSize, 0);
    chkEnableBeep->SetValue (true);
    page3->Add (chkEnableBeep, 0, wxALL, 5);
    chkDisableShiftSpace = new wxCheckBox (tab3, wxID_ANY,
                                           _("Disable Shift-Space (backspace via Ctrl-Space)"),
                                           wxDefaultPosition, wxDefaultSize, 0);
    chkDisableShiftSpace->SetValue (false);
    page3->Add (chkDisableShiftSpace, 0, wxALL, 5);
    chkDisableMouseDrag = new wxCheckBox (tab3, wxID_ANY,
                                          _("Disable mouse drag (select)"),
                                          wxDefaultPosition, wxDefaultSize, 0);
    chkDisableMouseDrag->SetValue (false);
    page3->Add (chkDisableMouseDrag, 0, wxALL, 5);
    tab3->SetSizer (page3);
    tab3->Layout ();
    page3->Fit (tab3);
    tabPrefsDialog->AddPage (tab3, _("Emulation"), false);

    //tab4
    tab4 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY,
                                 wxDefaultPosition, wxDefaultSize,
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab4->SetScrollRate (5, 5);

    wxFlexGridSizer* page4;
    page4 = new wxFlexGridSizer (3, 1, 0, 0);
    page4->AddGrowableCol (0);
    page4->AddGrowableRow (1);
    page4->SetFlexibleDirection (wxVERTICAL);
    lblExplain4 = new wxStaticText (tab4, wxID_ANY,
                                    _("Settings on this page allow you to change the display appearance."),
                                    wxDefaultPosition, wxDefaultSize, 0);
    page4->Add (lblExplain4, 0, wxALL, 5);

    wxBoxSizer* bs41;
    bs41 = new wxBoxSizer (wxVERTICAL);
    chkDisableColor = new wxCheckBox (tab4, wxID_ANY,
                                      _("Disable -color- (ASCII mode)"),
                                      wxDefaultPosition, wxDefaultSize, 0);
    bs41->Add (chkDisableColor, 0, wxALL, 5);

    wxFlexGridSizer* fgs411;
    fgs411 = new wxFlexGridSizer (2, 2, 0, 0);
#if defined (_WIN32)
    btnFGColor = new wxButton (tab4, wxID_ANY, wxT (""),
                               wxDefaultPosition, wxSize (25, -1), 0);
    btnFGColor->SetBackgroundColour (wxColour (255, 128, 0));
    fgs411->Add (btnFGColor, 0, wxALL, 5);
    lblFGColor = new wxStaticText (tab4, wxID_ANY, _("Foreground color*"),
                                   wxDefaultPosition, wxDefaultSize, 0);
    fgs411->Add (lblFGColor, 1, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    btnBGColor = new wxButton (tab4, wxID_ANY, wxT (""), wxDefaultPosition,
                               wxSize (25, -1), 0);
    btnBGColor->SetBackgroundColour (wxColour (0, 0, 0));
    fgs411->Add (btnBGColor, 0, wxALL, 5);
    lblBGColor = new wxStaticText (tab4, wxID_ANY, _("Background color*"),
                                   wxDefaultPosition, wxDefaultSize, 0);
    fgs411->Add (lblBGColor, 0,
                 wxALL | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL,
                 5);
#else
    btnFGColor = new wxBitmapButton (tab4, wxID_ANY, wxNullBitmap,
                                     wxDefaultPosition, wxDefaultSize,
                                     wxBU_AUTODRAW);
    fgs411->Add (btnFGColor, 0, wxALL, 5);
    lblFGColor = new wxStaticText (tab4, wxID_ANY, _("Foreground color*"),
                                   wxDefaultPosition, wxDefaultSize, 0);
    fgs411->Add (lblFGColor, 1, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    btnBGColor = new wxBitmapButton (tab4, wxID_ANY, wxNullBitmap,
                                     wxDefaultPosition, wxDefaultSize,
                                     wxBU_AUTODRAW);
    fgs411->Add (btnBGColor, 0, wxALL, 5);
    lblBGColor = new wxStaticText (tab4, wxID_ANY, _("Background color*"),
                                   wxDefaultPosition, wxDefaultSize, 0);
    fgs411->Add (lblBGColor, 0,
                 wxALL | wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL,
                 5);
#endif
    bs41->Add (fgs411, 1, 0, 5);
    page4->Add (bs41, 1, wxEXPAND | wxALIGN_TOP, 5);
    lblExplainColor = new wxStaticText (tab4, wxID_ANY, 
                                        _("* NOTE: Applied in Classic mode or if -color- is disabled in ASCII mode"),
                                        wxDefaultPosition, wxDefaultSize, 0);
    page4->Add (lblExplainColor, 0, wxALL | wxALIGN_BOTTOM, 5);
    tab4->SetSizer (page4);
    tab4->Layout ();
    page4->Fit (tab4);
    tabPrefsDialog->AddPage (tab4, _("Display"), false);

    //tab5
    tab5 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY,
                                 wxDefaultPosition, wxDefaultSize,
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab5->SetScrollRate (5, 5);

    wxFlexGridSizer* page5;
    page5 = new wxFlexGridSizer (2, 1, 0, 0);
    page5->AddGrowableCol (0);
    page5->AddGrowableRow (0);
    page5->SetFlexibleDirection (wxVERTICAL);

    wxBoxSizer* bs51;
    bs51 = new wxBoxSizer (wxVERTICAL);
    lblExplain5 = new wxStaticText (tab5, wxID_ANY,
                                    _("Settings on this page allow you to specify Paste options."),
                                    wxDefaultPosition, wxDefaultSize, 0);
    bs51->Add (lblExplain5, 0, wxALL, 5);

    wxFlexGridSizer* fgs511;
    wxIntegerValidator<long> cdval, ldval, autoval;

    fgs511 = new wxFlexGridSizer (2, 3, 0, 0);
    // We don't really want the delays to be as low as 1 ms, but if the
    // lower bound is more than that, you can't enter a value starting from
    // a blank field because that would be less than 10...
    cdval.SetRange (1, 1000);
    ldval.SetRange (1, 1000);
    autoval.SetRange (0, 120);
    lblCharDelay = new wxStaticText (tab5, wxID_ANY, _("Delay between chars"),
                                     wxDefaultPosition, wxDefaultSize, 0);
    fgs511->Add (lblCharDelay, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    txtCharDelay = new wxTextCtrl (tab5, wxID_ANY, wxT ("50"),
                                   wxDefaultPosition, wxSize (48, -1), 0,
                                   cdval);
    txtCharDelay->SetMaxLength (3); 
    fgs511->Add (txtCharDelay, 0, wxALL, 5);
    lblCharDelay2 = new wxStaticText (tab5, wxID_ANY, _("milliseconds"),
                                      wxDefaultPosition, wxDefaultSize, 0);
    fgs511->Add (lblCharDelay2, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
    lblLineDelay = new wxStaticText (tab5, wxID_ANY,
                                     _("Delay after end of line"),
                                     wxDefaultPosition, wxDefaultSize, 0);
    fgs511->Add (lblLineDelay, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    txtLineDelay = new wxTextCtrl (tab5, wxID_ANY, wxT ("100"),
                                   wxDefaultPosition, wxSize (48, -1), 0,
                                   ldval);
    txtLineDelay->SetMaxLength (3); 
    fgs511->Add (txtLineDelay, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    lblLineDelay2 = new wxStaticText (tab5, wxID_ANY, _("milliseconds"),
                                      wxDefaultPosition, wxDefaultSize, 0);
    fgs511->Add (lblLineDelay2, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    bs51->Add (fgs511, 1, wxEXPAND, 5);

    wxFlexGridSizer* fgs512;
    fgs512 = new wxFlexGridSizer (2, 3, 0, 0);
    lblAutoNewLine = new wxStaticText (tab5, wxID_ANY,
                                       _("Automatic new line every"),
                                       wxDefaultPosition, wxDefaultSize, 0);
    fgs512->Add (lblAutoNewLine, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);

    cboAutoLF = new wxComboBox (tab5, wxID_ANY, wxT ("60"), wxDefaultPosition,
                                wxSize (-1, -1), 0, NULL, 0 | wxTAB_TRAVERSAL,
                                autoval);
    cboAutoLF->Append (wxT ("0"));
    cboAutoLF->Append (wxT ("60"));
    cboAutoLF->Append (wxT ("120"));
    cboAutoLF->SetMinSize (wxSize (65, -1));
    fgs512->Add (cboAutoLF, 1, wxALL, 5);
    lblAutoNewLine2 = new wxStaticText (tab5, wxID_ANY, _("characters"),
                                        wxDefaultPosition, wxDefaultSize, 0);
    fgs512->Add (lblAutoNewLine2, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    bs51->Add (fgs512, 0, wxEXPAND, 5);
    chkSplitWords = new wxCheckBox (tab5, wxID_ANY,
                                    _("Allow words to be split across lines"),
                                    wxDefaultPosition, wxDefaultSize, 0);
    chkSplitWords->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    bs51->Add (chkSplitWords, 0, wxALL, 5);
    chkSmartPaste = new wxCheckBox (tab5, wxID_ANY,
                                    _("Use TUTOR pasting (certain sequences are treated specially)"),
                                    wxDefaultPosition, wxDefaultSize, 0);
    chkSmartPaste->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    bs51->Add (chkSmartPaste, 0, wxALL, 5);
    chkConvertDot7 = new wxCheckBox (tab5, wxID_ANY,
                                     _("Convert periods followed by 7 spaces into period/tab"),
                                     wxDefaultPosition, wxDefaultSize, 0);
    chkConvertDot7->SetValue (true);
    //chkConvertDot7->Hide ();
    bs51->Add (chkConvertDot7, 0, wxALL, 5);
    chkConvert8Spaces = new wxCheckBox (tab5, wxID_ANY,
                                        _("Convert 8 consecutive spaces into a tab"),
                                        wxDefaultPosition, wxDefaultSize, 0);
    chkConvert8Spaces->SetValue (true);
    //chkConvert8Spaces->Hide ();
    bs51->Add (chkConvert8Spaces, 0, wxALL, 5);
    chkTutorColor = new wxCheckBox (tab5, wxID_ANY,
                                    _("Display TUTOR colorization options on Edit/Context menus"),
                                    wxDefaultPosition, wxDefaultSize, 0);
    chkTutorColor->SetValue (true);
    //chkTutorColor->Hide ();
    bs51->Add (chkTutorColor, 0, wxALL, 5);
    page5->Add (bs51, 1, wxEXPAND, 5);
    tab5->SetSizer (page5);
    tab5->Layout ();
    page5->Fit (tab5);
    tabPrefsDialog->AddPage (tab5, _("Pasting"), false);

    //tab6
    tab6 = new wxScrolledWindow (tabPrefsDialog, wxID_ANY,
                                 wxDefaultPosition, wxDefaultSize,
                                 wxHSCROLL | wxTAB_TRAVERSAL | wxVSCROLL);
    tab6->SetScrollRate (5, 5);
    tab6->Hide ();

    wxFlexGridSizer* page6;
    page6 = new wxFlexGridSizer (0, 1, 0, 0);
    page6->AddGrowableCol (0);
    page6->SetFlexibleDirection (wxBOTH);
    lblEmail = new wxStaticText (tab6, wxID_ANY,
                                 _("Command line for menu option 'Mail to...' (%s=address)"),
                                 wxDefaultPosition, wxDefaultSize, 0);
    page6->Add (lblEmail, 0, wxALL | wxALIGN_BOTTOM, 5);
    txtEmail = new wxTextCtrl (tab6, wxID_ANY, wxT (""), wxDefaultPosition,
                               wxDefaultSize, 0 | wxTAB_TRAVERSAL);
    txtEmail->SetMaxLength (255); 
    txtEmail->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    page6->Add (txtEmail, 0, wxALL | wxEXPAND, 5);
    lblSearchURL = new wxStaticText (tab6, wxID_ANY,
                                     _("Specify URL for menu option 'Search this...'"),
                                     wxDefaultPosition, wxDefaultSize, 0);
    page6->Add (lblSearchURL, 0, wxALL, 5);
    txtSearchURL = new wxTextCtrl (tab6, wxID_ANY, wxT (""), wxDefaultPosition,
                                   wxDefaultSize, 0 | wxTAB_TRAVERSAL);
    txtSearchURL->SetMaxLength (255); 
    page6->Add (txtSearchURL, 0, wxALL | wxEXPAND, 5);
    tab6->SetSizer (page6);
    tab6->Layout ();
    page6->Fit (tab6);
    tabPrefsDialog->AddPage (tab6, _("Local"), false);

    //notebook
    bSizer1->Add (tabPrefsDialog, 1, wxEXPAND | wxTOP | wxRIGHT | wxLEFT, 6);

    //button bar
    wxFlexGridSizer* fgsButtons;
    fgsButtons = new wxFlexGridSizer (1, 5, 0, 0);
    fgsButtons->AddGrowableCol (0);
    fgsButtons->AddGrowableRow (0);
    fgsButtons->SetFlexibleDirection (wxHORIZONTAL);
    fgsButtons->Add (0, 0, 1, wxALL | wxEXPAND, 5);
    btnOK = new wxButton (this, wxID_ANY, _("OK"), wxDefaultPosition,
                          wxDefaultSize, 0);
    btnOK->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgsButtons->Add (btnOK, 0, wxALL, 5);
    btnCancel = new wxButton (this, wxID_CANCEL, _("Cancel"),
                              wxDefaultPosition, wxDefaultSize, 0);
    btnCancel->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgsButtons->Add (btnCancel, 0, wxALL, 5);
    btnDefaults = new wxButton (this, wxID_ANY, _("Defaults"),
                                wxDefaultPosition, wxDefaultSize, 0);
    btnDefaults->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgsButtons->Add (btnDefaults, 0, wxALL, 5);
    bSizer1->Add (fgsButtons, 0, wxEXPAND, 5);
    this->SetSizer (bSizer1);
    this->Layout ();

    // set object value properties
    SetControlState ();

    //set active tab
    m_lastTab = ptermApp->m_config->Read (wxT (PREF_LASTTAB), 0L);
    tabPrefsDialog->SetSelection (m_lastTab);

    //button bar
    btnOK->SetDefault ();
}

void PtermPrefDialog::OnClose (wxCloseEvent& event)
{
    EndModal (wxID_CANCEL);
}

bool PtermPrefDialog::ValidProfile (wxString profile)
{
    wxString validchr = wxT ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_. ()");
    unsigned int cnt;
    for (cnt=0; cnt<profile.Len (); cnt++)
        if (!validchr.Contains (profile.Mid (cnt, 1)))
            return false;
    return true;
}

bool PtermPrefDialog::SaveProfile (wxString profile)
{
    wxString filename;
    wxString buffer;
    bool openok;

    //open file
    filename = ptermApp->ProfileFileName (profile);
    wxTextFile file (filename);
    if (file.Exists ())
        openok = file.Open ();
    else
        openok = file.Create ();
    if (!openok)
        return false;
    file.Clear ();

    //write prefs
    //tab0
    buffer.Printf (wxT (PREF_CURPROFILE) wxT ("=%s"), profile);
    file.AddLine (buffer);
    //tab1
    buffer.Printf (wxT (PREF_CONNECT) wxT ("=%d"), (m_connect) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SHELLFIRST) wxT ("=%s"), m_ShellFirst);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_HOST) wxT ("=%s"), m_host);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_PORT) wxT ("=%s"), m_port);
    file.AddLine (buffer);
    //tab2
    buffer.Printf (wxT (PREF_SHOWSIGNON) wxT ("=%d"), (m_showSignon) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SHOWSYSNAME) wxT ("=%d"), (m_showSysName) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SHOWHOST) wxT ("=%d"), (m_showHost) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SHOWSTATION) wxT ("=%d"), (m_showStation) ? 1 : 0);
    file.AddLine (buffer);
    //tab3
    buffer.Printf (wxT (PREF_1200BAUD) wxT ("=%d"), (m_classicSpeed) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_GSW) wxT ("=%d"), (m_gswEnable) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_ARROWS) wxT ("=%d"), (m_numpadArrows) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_IGNORECAP) wxT ("=%d"), (m_ignoreCapLock) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_PLATOKB) wxT ("=%d"), (m_platoKb) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_ACCEL) wxT ("=%d"), (m_useAccel) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_BEEP) wxT ("=%d"), (m_beepEnable) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SHIFTSPACE) wxT ("=%d"), (m_DisableShiftSpace) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_MOUSEDRAG) wxT ("=%d"), (m_DisableMouseDrag) ? 1 : 0);
    file.AddLine (buffer);
    //tab4
    buffer.Printf (wxT (PREF_SCALE) wxT ("=%d"), (m_scale2) ? 2 : 1);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_STRETCH) wxT ("=%d"), (m_stretch) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_STATUSBAR) wxT ("=%d"), (m_showStatusBar) ? 1 : 0);
    file.AddLine (buffer);
#if !defined (__WXMAC__)
    buffer.Printf (wxT (PREF_MENUBAR) wxT ("=%d"), (m_showMenuBar) ? 1 : 0);
    file.AddLine (buffer);
#endif
    buffer.Printf (wxT (PREF_NOCOLOR) wxT ("=%d"), (m_noColor) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_FOREGROUND) wxT ("=%d %d %d"), m_fgColor.Red (), m_fgColor.Green (), m_fgColor.Blue ());
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_BACKGROUND) wxT ("=%d %d %d"), m_bgColor.Red (), m_bgColor.Green (), m_bgColor.Blue ());
    file.AddLine (buffer);
    //tab5
    buffer.Printf (wxT (PREF_CHARDELAY) wxT ("=%s"), m_charDelay);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_LINEDELAY) wxT ("=%s"), m_lineDelay);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_AUTOLF) wxT ("=%s"), m_autoLF);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SPLITWORDS) wxT ("=%d"), (m_splitWords) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SMARTPASTE) wxT ("=%d"), (m_smartPaste) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_CONVDOT7) wxT ("=%d"), (m_convDot7) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_CONV8SP) wxT ("=%d"), (m_conv8Sp) ? 1 : 0);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_TUTORCOLOR) wxT ("=%d"), (m_TutorColor) ? 1 : 0);
    file.AddLine (buffer);
    //tab6
    buffer.Printf (wxT (PREF_EMAIL) wxT ("=%s"), m_Email);
    file.AddLine (buffer);
    buffer.Printf (wxT (PREF_SEARCHURL) wxT ("=%s"), m_SearchURL);
    file.AddLine (buffer);

    //write to disk
    file.Write ();
    file.Close ();

    return true;

}

bool PtermPrefDialog::DeleteProfile (wxString profile)
{
    wxString filename;

    //delete file
    filename = ptermApp->ProfileFileName (profile);
    if (wxFileExists (filename))
    {
        wxRemoveFile (filename);
        return true;
    }
    return false;
}

void PtermPrefDialog::SetControlState (void)
{
    wxString ws;
#if !defined (_WIN32)
    wxBitmap fgBitmap (15, 15);
    wxBitmap bgBitmap (15, 15);
#endif

    //tab0
    m_curProfile = ptermApp->m_curProfile;
    //tab1
    m_ShellFirst = ptermApp->m_ShellFirst;
    m_connect = ptermApp->m_connect;
    m_host = ptermApp->m_hostName;
    m_port = ptermApp->m_port;
    //tab2
    m_showSignon = ptermApp->m_showSignon;
    m_showSysName = ptermApp->m_showSysName;
    m_showHost = ptermApp->m_showHost;
    m_showStation = ptermApp->m_showStation;
    //tab3
    m_classicSpeed = ptermApp->m_classicSpeed;
    m_gswEnable = ptermApp->m_gswEnable;
    m_numpadArrows = ptermApp->m_numpadArrows;
    m_ignoreCapLock = ptermApp->m_ignoreCapLock;
    m_platoKb = ptermApp->m_platoKb;
    m_useAccel = ptermApp->m_useAccel;
    m_beepEnable = ptermApp->m_beepEnable;
    m_DisableShiftSpace = ptermApp->m_DisableShiftSpace;
    m_DisableMouseDrag = ptermApp->m_DisableMouseDrag;
    //tab4
    m_scale2 = ptermApp->m_scale2;
    m_stretch = ptermApp->m_stretch;
    m_showStatusBar = ptermApp->m_showStatusBar;
#if !defined (__WXMAC__)
    m_showMenuBar = ptermApp->m_showMenuBar;
#endif
    m_noColor = ptermApp->m_noColor;
    //m_fgColor = ptermApp->m_fgColor;
    //m_bgColor = ptermApp->m_bgColor;
    int r, g, b;
    wxString rgb;
    ptermApp->m_config->Read (wxT (PREF_FOREGROUND), &rgb, wxT ("255 144 0")); // 255 144 0 is RGB for Plato Orange
    sscanf (rgb.mb_str (), "%d %d %d", &r, &g, &b);
    m_fgColor = wxColour (r, g, b);
    ptermApp->m_config->Read (wxT (PREF_BACKGROUND), &rgb, wxT ("0 0 0"));
    sscanf (rgb.mb_str (), "%d %d %d", &r, &g, &b);
    m_bgColor = wxColour (r, g, b);
    //tab5
    m_charDelay = ptermApp->m_charDelay;
    m_lineDelay = ptermApp->m_lineDelay;
    m_autoLF = ptermApp->m_autoLF;
    m_splitWords = ptermApp->m_splitWords;
    m_smartPaste = ptermApp->m_smartPaste;
    m_convDot7 = ptermApp->m_convDot7;
    m_conv8Sp = ptermApp->m_conv8Sp;
    m_TutorColor = ptermApp->m_TutorColor;
    //tab6
    m_Email = ptermApp->m_Email;
    m_SearchURL = ptermApp->m_SearchURL;
    
    //tab0
    wxDir ldir (wxGetCwd ());
    if (ldir.IsOpened ())
    {
        // populate listbox
        wxString filename;
        bool cont = ldir.GetFirst (&filename, wxT ("*.ppf"), wxDIR_DEFAULT);
        lstProfiles->Clear ();
        lstProfiles->Append (CURRENT_PROFILE);
        int i, cur=0;
        wxString str;
        for (i = 0; cont; i++)
        {
            filename = filename.Left (filename.Len () - 4);
            lstProfiles->Append (filename);
            cont = ldir.GetNext (&filename);
        }
        for (i = 0; i < (int) lstProfiles->GetCount (); i++)
        {
            filename = lstProfiles->GetString (i);
            if (filename.CmpNoCase (ptermApp->m_curProfile) == 0)
                cur = i;
        }
        lstProfiles->Select (cur);
    }
    //tab1
    chkConnectAtStartup->SetValue (m_connect);
    txtShellFirst->SetValue (m_ShellFirst);
    txtDefaultHost->SetValue (m_host);
    ws.Printf ("%ld", m_port);
    cboDefaultPort->SetValue (ws);
    //tab2
    chkShowSignon->SetValue (m_showSignon);
    chkShowSysName->SetValue (m_showSysName);
    chkShowHost->SetValue (m_showHost);
    chkShowStation->SetValue (m_showStation);
    //tab3
    chkSimulate1200Baud->SetValue (m_classicSpeed);
    chkEnableGSW->SetValue (m_gswEnable);
    chkEnableNumericKeyPad->SetValue (m_numpadArrows);
    chkIgnoreCapLock->SetValue (m_ignoreCapLock);
    chkUsePLATOKeyboard->SetValue (m_platoKb);
    chkUseAccelerators->SetValue (m_useAccel);
    chkEnableBeep->SetValue (m_beepEnable);
    chkDisableShiftSpace->SetValue (m_DisableShiftSpace);
    chkDisableMouseDrag->SetValue (m_DisableMouseDrag);
    //tab4
    //chkZoom200->SetValue (m_scale2);
    //chkStretch->SetValue (m_stretch);
    //chkStatusBar->SetValue (m_showStatusBar);
#if !defined (__WXMAC__)
    //chkMenuBar->SetValue (m_showMenuBar);
#endif
    chkDisableColor->SetValue (m_noColor);
#if defined (_WIN32)
    btnFGColor->SetBackgroundColour (m_fgColor);
    btnBGColor->SetBackgroundColour (m_bgColor);
#else
    paintBitmap (fgBitmap, m_fgColor);
    btnFGColor->SetBitmapLabel (fgBitmap);
    paintBitmap (bgBitmap, m_bgColor);
    btnBGColor->SetBitmapLabel (bgBitmap);
#endif
    //tab5
    ws.Printf ("%ld", m_charDelay);
    txtCharDelay->SetValue (ws);
    ws.Printf ("%ld", m_lineDelay);
    txtLineDelay->SetValue (ws);
    ws.Printf ("%ld", m_autoLF);
    cboAutoLF->SetValue (ws);
    chkSplitWords->SetValue (m_splitWords);
    chkSmartPaste->SetValue (m_smartPaste);
    chkConvertDot7->SetValue (m_convDot7);
    chkConvert8Spaces->SetValue (m_conv8Sp);
    chkTutorColor->SetValue (m_TutorColor);
    //tab6
    txtEmail->SetValue (m_Email);
    txtSearchURL->SetValue (m_SearchURL);

    //set button state
    wxString profile;
    bool enable;
    profile = lstProfiles->GetStringSelection ();
    enable = (profile.Cmp (CURRENT_PROFILE)!=0);
    btnSave->Enable (enable);
    btnLoad->Enable (enable);
    btnDelete->Enable (enable);
    profile = txtProfile->GetLineText (0);
    btnAdd->Enable (!profile.IsEmpty ());
}

void PtermPrefDialog::OnButton (wxCommandEvent& event)
{
    wxBitmap fgBitmap (15, 15);
    wxBitmap bgBitmap (15, 15);
    wxString profile;
    wxString filename;
    wxString str;
    
    void OnButton (wxCommandEvent& event);
    lblProfileStatusMessage->SetLabel (wxT (""));
    if (event.GetEventObject () == btnSave)
    {
        profile = lstProfiles->GetStringSelection ();
        ptermApp->m_curProfile = profile;
        m_curProfile = profile;
        if (SaveProfile (profile))
        {
            ptermApp->LoadProfile (profile, wxT (""));
            SetControlState ();
            lblProfileStatusMessage->SetLabel (_("Profile saved."));
        }
        else
        {
            filename = ptermApp->ProfileFileName (profile);
            str.Printf (wxT ("Unable to save profile: %s"), filename);
            wxMessageBox (str, _("Error"), wxOK | wxICON_HAND);
        }
    }
    else if (event.GetEventObject () == btnLoad)
    {
        profile = lstProfiles->GetStringSelection ();
        ptermApp->m_curProfile = profile;
        m_curProfile = profile;
        if (ptermApp->LoadProfile (profile, wxT ("")))
        {
            SetControlState ();
            lblProfileStatusMessage->SetLabel (_("Profile loaded."));
        }
        else
        {
            filename = ptermApp->ProfileFileName (profile);
            str.Printf (wxT ("Unable to load profile: %s"), filename);
            wxMessageBox (str, _("Error"), wxOK | wxICON_HAND);
        }
    }
    else if (event.GetEventObject () == btnDelete)
    {
        profile = lstProfiles->GetStringSelection ();
        if (DeleteProfile (profile))
        {
            SetControlState ();
            lblProfileStatusMessage->SetLabel (_("Profile deleted."));
        }
        else
        {
            filename = ptermApp->ProfileFileName (profile);
            str.Printf (wxT ("Unable to delete profile: %s"), 
                       filename);
            wxMessageBox (str, _("Error"), wxOK | wxICON_HAND);
        }
    }
    else if (event.GetEventObject () == btnAdd)
    {
        profile = txtProfile->GetLineText (0);
        if (ValidProfile (profile))
        {
            if (SaveProfile (profile))
            {
                ptermApp->m_curProfile = profile;
                m_curProfile = profile;
                ptermApp->LoadProfile (profile, wxT (""));
                SetControlState ();
                lblProfileStatusMessage->SetLabel (_("Profile added."));
            }
            else
            {
                filename = ptermApp->ProfileFileName (profile);
                str.Printf (wxT ("Unable to add/save profile: %s"), 
                           filename);
                wxMessageBox (str, _("Error"), wxOK | wxICON_HAND);
            }
        }
        else
        {
            wxMessageBox (_("The profile name you entered contains illegal characters.\n\n"
                         "Valid characters are:\n\n"
                         "Standard: a-z, A-Z, 0-9\n"
                           "Special:  parentheses, dash, underscore, period, and space"), _("Problem"), wxOK | wxICON_EXCLAMATION);
        }
    }
    else if (event.GetEventObject () == btnFGColor)
    {
        m_fgColor = PtermApp::SelectColor (*this, _("Foreground"), m_fgColor);
#if defined (_WIN32)
        btnFGColor->SetBackgroundColour (m_fgColor);
#else
        paintBitmap (fgBitmap, m_fgColor);
        btnFGColor->SetBitmapLabel (fgBitmap);
#endif
    }
    else if (event.GetEventObject () == btnBGColor)
    {
        m_bgColor = PtermApp::SelectColor (*this, _("Background"), m_bgColor);
#if defined (_WIN32)
        btnBGColor->SetBackgroundColour (m_bgColor);
#else
        paintBitmap (bgBitmap, m_bgColor);
        btnBGColor->SetBitmapLabel (bgBitmap);
#endif
    }
    else if (event.GetEventObject () == btnOK)
    {
        //buttonbar
        m_lastTab = tabPrefsDialog->GetSelection ();
        EndModal (wxID_OK);
    }
    else if (event.GetEventObject () == btnCancel)
    {
        m_lastTab = tabPrefsDialog->GetSelection ();
        EndModal (wxID_CANCEL);
    }
    else if (event.GetEventObject () == btnDefaults)
    {
        wxString str;
        
        //reset variable values
        //tab0
        //tab1
        m_ShellFirst = wxT ("");
        m_connect = true;
        m_host = DEFAULTHOST;
        m_port = DefNiuPort;
        //tab2
        m_showSignon = false;
        m_showSysName = false;
        m_showHost = true;
        m_showStation = true;
        //tab3
        m_classicSpeed = false;
        m_gswEnable = true;
        m_numpadArrows = true;
        m_ignoreCapLock = false;
        m_platoKb = false;
#if defined (__WXMAC__)
        m_useAccel = true;
#else
        m_useAccel = false;
#endif
        m_beepEnable = true;
        m_DisableShiftSpace = false;
        m_DisableMouseDrag = false;
        //tab4
        m_scale2 = false;
        m_stretch = false;
        m_showStatusBar = true;
#if !defined (__WXMAC__)
        m_showMenuBar = true;
#endif
        m_noColor = false;
        m_fgColor = wxColour (255, 144, 0);
        m_bgColor = *wxBLACK;
        //tab5
        m_charDelay = PASTE_CHARDELAY;
        m_lineDelay = PASTE_LINEDELAY;
        m_autoLF = 0;
        m_splitWords = false;
        m_smartPaste = false;
        m_convDot7 = false;
        m_conv8Sp = false;
        m_TutorColor = false;
        //tab6
        m_Email = wxT ("");
        m_SearchURL = DEFAULTSEARCH;

        //reset object values
        //tab0
        //tab1
        txtShellFirst->SetValue (m_ShellFirst);
        chkConnectAtStartup->SetValue (m_connect);
        txtDefaultHost->SetValue (m_host);
        str.Printf ("%ld", m_port);
        cboDefaultPort->SetValue (str);
        //tab2
        chkShowSignon->SetValue (m_showSignon);
        chkShowSysName->SetValue (m_showSysName);
        chkShowHost->SetValue (m_showHost);
        chkShowStation->SetValue (m_showStation);
        //tab3
        chkSimulate1200Baud->SetValue (m_classicSpeed);
        chkEnableGSW->SetValue (m_gswEnable);
        chkEnableNumericKeyPad->SetValue (m_numpadArrows);
        chkIgnoreCapLock->SetValue (m_ignoreCapLock);
        chkUsePLATOKeyboard->SetValue (m_platoKb);
        chkUseAccelerators->SetValue (m_useAccel);
        chkEnableBeep->SetValue (m_beepEnable);
        chkDisableShiftSpace->SetValue (m_DisableShiftSpace);
        chkDisableMouseDrag->SetValue (m_DisableMouseDrag);
        //tab4
        //chkZoom200->SetValue (m_scale2);
        //chkStretch->SetValue (m_stretch);
        //chkStatusBar->SetValue (m_showStatusBar);
#if !defined (__WXMAC__)
        //chkMenuBar->SetValue (m_showMenuBar);
#endif
        chkDisableColor->SetValue (m_noColor);
        btnFGColor->SetBackgroundColour (m_fgColor);
        btnBGColor->SetBackgroundColour (m_bgColor);
        //tab5
        str.Printf ("%ld", m_charDelay);
        txtCharDelay->SetValue (str);
        str.Printf ("%ld", m_lineDelay);
        txtLineDelay->SetValue (str);
        str.Printf ("%ld", m_autoLF);
        cboAutoLF->SetValue (str);
        chkSplitWords->SetValue (m_splitWords);
        chkSmartPaste->SetValue (m_smartPaste);
        chkConvertDot7->SetValue (m_convDot7);
        chkConvert8Spaces->SetValue (m_conv8Sp);
        chkTutorColor->SetValue (m_TutorColor);
        //tab6
        txtEmail->SetValue (m_Email);
        txtSearchURL->SetValue (m_SearchURL);
    }
    Refresh ();
}

void PtermPrefDialog::OnCheckbox (wxCommandEvent& event)
{
    void OnCheckbox (wxCommandEvent& event);
    lblProfileStatusMessage->SetLabel (wxT (" "));
    //tab0
    //tab1
    if (event.GetEventObject () == chkConnectAtStartup)
        m_connect = event.IsChecked ();
    //tab2
    else if (event.GetEventObject () == chkShowSignon)
        m_showSignon = event.IsChecked ();
    else if (event.GetEventObject () == chkShowSysName)
        m_showSysName = event.IsChecked ();
    else if (event.GetEventObject () == chkShowHost)
        m_showHost = event.IsChecked ();
    else if (event.GetEventObject () == chkShowStation)
        m_showStation = event.IsChecked ();
    //tab3
    else if (event.GetEventObject () == chkSimulate1200Baud)
        m_classicSpeed = event.IsChecked ();
    else if (event.GetEventObject () == chkEnableGSW)
        m_gswEnable = event.IsChecked ();
    else if (event.GetEventObject () == chkEnableNumericKeyPad)
        m_numpadArrows = event.IsChecked ();
    else if (event.GetEventObject () == chkIgnoreCapLock)
        m_ignoreCapLock = event.IsChecked ();
    else if (event.GetEventObject () == chkUsePLATOKeyboard)
        m_platoKb = event.IsChecked ();
    else if (event.GetEventObject () == chkUseAccelerators)
        m_useAccel = event.IsChecked ();
    else if (event.GetEventObject () == chkEnableBeep)
        m_beepEnable = event.IsChecked ();
    else if (event.GetEventObject () == chkDisableShiftSpace)
        m_DisableShiftSpace = event.IsChecked ();
    else if (event.GetEventObject () == chkDisableMouseDrag)
        m_DisableMouseDrag = event.IsChecked ();
    //tab4
    //else if (event.GetEventObject () == chkZoom200)
    //    m_scale2 = event.IsChecked ();
    //else if (event.GetEventObject () == chkStretch)
    //    m_stretch = event.IsChecked ();
    //else if (event.GetEventObject () == chkStatusBar)
    //    m_showStatusBar = event.IsChecked ();
#if !defined (__WXMAC__)
    //else if (event.GetEventObject () == chkMenuBar)
    //    m_showMenuBar = event.IsChecked ();
#endif
    else if (event.GetEventObject () == chkDisableColor)
        m_noColor = event.IsChecked ();
    //tab5
    else if (event.GetEventObject () == chkSplitWords)
        m_splitWords = event.IsChecked ();
    else if (event.GetEventObject () == chkSmartPaste)
        m_smartPaste = event.IsChecked ();
    else if (event.GetEventObject () == chkConvertDot7)
        m_convDot7 = event.IsChecked ();
    else if (event.GetEventObject () == chkConvert8Spaces)
        m_conv8Sp = event.IsChecked ();
    else if (event.GetEventObject () == chkTutorColor)
        m_TutorColor = event.IsChecked ();
    //tab6
}

void PtermPrefDialog::OnSelect (wxCommandEvent& event)
{
    void OnSelect (wxCommandEvent& event);
    wxString profile;
    bool enable;
    lblProfileStatusMessage->SetLabel (wxT (" "));
    if (event.GetEventObject () == lstProfiles)
    {
        profile = lstProfiles->GetStringSelection ();
        enable = (profile.Cmp (CURRENT_PROFILE)!=0);
        btnSave->Enable (enable);
        btnLoad->Enable (enable);
        btnDelete->Enable (enable);
        profile = txtProfile->GetLineText (0);
        btnAdd->Enable (!profile.IsEmpty ());
    }
}

void PtermPrefDialog::OnComboSelect (wxCommandEvent& event)
{
    void OnComboSelect (wxCommandEvent& event);
    lblProfileStatusMessage->SetLabel (wxT (" "));
    //tab1
    if (event.GetEventObject () == cboDefaultPort)
        cboDefaultPort->GetValue ().ToCLong (&m_port);
    //tab5
    else if (event.GetEventObject () == cboAutoLF)
        cboAutoLF->GetStringSelection ().ToCLong (&m_autoLF);
}

void PtermPrefDialog::OnDoubleClick (wxCommandEvent& event)
{
    void OnDoubleClick (wxCommandEvent& event);
    wxString profile;
    bool enable;
    wxString str;
    wxString filename;
    lblProfileStatusMessage->SetLabel (wxT (" "));
    if (event.GetEventObject () == lstProfiles)
    {
        profile = lstProfiles->GetStringSelection ();
        enable = (profile.Cmp (CURRENT_PROFILE)!=0);
        btnSave->Enable (enable);
        btnLoad->Enable (enable);
        btnDelete->Enable (enable);
        profile = txtProfile->GetLineText (0);
        btnAdd->Enable (!profile.IsEmpty ());
        //do the load code
        if (enable)
        {
            profile = lstProfiles->GetStringSelection ();
            if (ptermApp->LoadProfile (profile, wxT ("")))
            {
                SetControlState ();
                lblProfileStatusMessage->SetLabel (_("Profile loaded."));
            }
            else
            {
                filename = ptermApp->ProfileFileName (profile);
                str.Printf (wxT ("Unable to load profile: %s"), 
                            filename);
                wxMessageBox (str, _("Error"), wxOK | wxICON_HAND);
            }
        }
    }
}

void PtermPrefDialog::OnChange (wxCommandEvent& event)
{
    void OnChange (wxCommandEvent& event);
    wxString profile;
    lblProfileStatusMessage->SetLabel (wxT (" "));
    if (event.GetEventObject () == txtProfile)
    {
        profile = txtProfile->GetLineText (0);
        btnAdd->Enable (!profile.IsEmpty ());
    }
    //tab1
    else if (event.GetEventObject () == txtShellFirst)
        m_ShellFirst = txtShellFirst->GetLineText (0);
    else if (event.GetEventObject () == txtDefaultHost)
        m_host = txtDefaultHost->GetLineText (0);
    else if (event.GetEventObject () == cboDefaultPort)
        cboDefaultPort->GetValue ().ToCLong (&m_port);
    //tab5
    else if (event.GetEventObject () == cboAutoLF)
        cboAutoLF->GetValue ().ToCLong (&m_autoLF);
    //tab5
    else if (event.GetEventObject () == txtCharDelay)
        txtCharDelay->GetLineText (0).ToCLong (&m_charDelay);
    else if (event.GetEventObject () == txtLineDelay)
        txtLineDelay->GetLineText (0).ToCLong (&m_lineDelay);
    //tab6
    else if (event.GetEventObject () == txtEmail)
        m_Email = txtEmail->GetLineText (0);
    else if (event.GetEventObject () == txtSearchURL)
        m_SearchURL = txtSearchURL->GetLineText (0);
}

void PtermPrefDialog::paintBitmap (wxBitmap &bm, wxColour &color)
{
    wxBrush bitmapBrush (color, wxSOLID);
    wxMemoryDC memDC;

    memDC.SelectObject (bm);
    memDC.SetBackground (bitmapBrush);
    memDC.Clear ();
    memDC.SetBackground (wxNullBrush);
    memDC.SelectObject (wxNullBitmap);
}


// ----------------------------------------------------------------------------
// PtermConnDialog
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (PtermConnDialog, wxDialog)
    EVT_CLOSE (PtermConnDialog::OnClose)
    EVT_LISTBOX (wxID_ANY, PtermConnDialog::OnSelect)
    EVT_LISTBOX_DCLICK (wxID_ANY, PtermConnDialog::OnDoubleClick)
    EVT_TEXT (wxID_ANY, PtermConnDialog::OnChange)
    EVT_BUTTON (wxID_ANY, PtermConnDialog::OnButton)
    END_EVENT_TABLE ();

PtermConnDialog::PtermConnDialog (wxWindowID id, const wxString &title, wxPoint pos, wxSize loc)
    : wxDialog (NULL, id, title, pos, loc)
{

    // static ui objects, note dynamic controls, e.g. those that hold values or require event processing are declared above
    wxStaticText* lblExplainProfiles;
//  wxListBox* lstProfiles;
    wxStaticText* lblShellFirst;
//  wxTextCtrl* txtShellFirst;
    wxStaticText* lblExplainManual;
    wxStaticText* lblHost;
//  wxTextCtrl* txtHost;
    wxStaticText* lblPort;
//  wxComboBox* cboPort;
    wxStaticText* lblExplainPort;
//  wxButton* btnCancel;
//  wxButton* btnConnect;

    // ui object creation / placement, note initialization of values is below
    wxBoxSizer* bs1;
    bs1 = new wxBoxSizer (wxVERTICAL);
    lblExplainProfiles = new wxStaticText (this, wxID_ANY,
                                           _("Select a profile and click Connect."),
                                           wxDefaultPosition, wxDefaultSize, 0);
    lblExplainProfiles->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    bs1->Add (lblExplainProfiles, 0,
              wxTOP|wxRIGHT|wxLEFT|wxALIGN_CENTER_VERTICAL, 5);
    lstProfiles = new wxListBox (this, wxID_ANY, wxDefaultPosition,
                                 wxSize (-1, -1), 0, NULL, wxLB_SORT); 
    lstProfiles->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    lstProfiles->SetMinSize (wxSize (-1, 122));
    bs1->Add (lstProfiles, 0, wxALL | wxEXPAND, 5);
    lblExplainManual = new wxStaticText (this, wxID_ANY,
                                         _("Or, enter a hostname and port number, then click Connect."),
                                         wxDefaultPosition, wxDefaultSize, 0);
    lblExplainManual->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    bs1->Add (lblExplainManual, 0, wxALL, 5);

    wxFlexGridSizer* fgSizer11;
    fgSizer11 = new wxFlexGridSizer (0, 1, 0, 0);
    fgSizer11->AddGrowableCol (0);
    fgSizer11->AddGrowableRow (0);
    fgSizer11->SetFlexibleDirection (wxBOTH);

    wxFlexGridSizer* fgs111;
    fgs111 = new wxFlexGridSizer (3, 2, 0, 0);
    lblShellFirst = new wxStaticText (this, wxID_ANY, _("Run this first"),
                                      wxDefaultPosition, wxDefaultSize, 0);
    lblShellFirst->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs111->Add (lblShellFirst, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    txtShellFirst = new wxTextCtrl (this, wxID_ANY, wxT (""),
                                    wxDefaultPosition, wxDefaultSize, 0);
    txtShellFirst->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    txtShellFirst->SetMaxLength (255); 
    fgs111->Add (txtShellFirst, 0, wxALL | wxEXPAND, 5);
    lblHost = new wxStaticText (this, wxID_ANY, _("Host name"),
                                wxDefaultPosition, wxDefaultSize, 0);
    lblHost->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs111->Add (lblHost, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    txtHost = new wxTextCtrl (this, wxID_ANY, wxT ("cyberserv.org"),
                              wxDefaultPosition, wxSize (-1, -1),
                              wxTAB_TRAVERSAL);
    txtHost->SetMaxLength (100); 
    txtHost->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    txtHost->SetMinSize (wxSize (320, -1));
    fgs111->Add (txtHost, 0, wxALL, 5);
    lblPort = new wxStaticText (this, wxID_ANY, _("Port*"),
                                wxDefaultPosition, wxDefaultSize, 0);
    lblPort->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs111->Add (lblPort, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);

    wxIntegerValidator<long> portval;
    portval.SetRange (1, 65535);
    
    cboPort = new wxComboBox (this, wxID_ANY, wxT ("5004"),
                              wxDefaultPosition, wxDefaultSize, 0,
                              NULL, wxTAB_TRAVERSAL, portval);
    cboPort->Append (wxT ("5004"));
    cboPort->Append (wxT ("8005"));
    cboPort->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    cboPort->SetMinSize (wxSize (75, -1));
    fgs111->Add (cboPort, 0, wxALL, 5);
    fgSizer11->Add (fgs111, 0, 0, 5);
    lblExplainPort = new wxStaticText (this, wxID_ANY,
                                       _("* NOTE: 5004=Classic, 8005=Color Terminal"),
                                       wxDefaultPosition, wxDefaultSize, 0);
    lblExplainPort->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgSizer11->Add (lblExplainPort, 0, wxTOP | wxRIGHT | wxLEFT, 5);

    wxFlexGridSizer* fgs112;
    fgs112 = new wxFlexGridSizer (2, 3, 0, 0);
    fgs112->AddGrowableCol (0);
    fgs112->SetFlexibleDirection (wxBOTH);
    fgs112->Add (0, 0, 1, wxALL, 5);
    btnCancel = new wxButton (this, wxID_CANCEL, _("Cancel"), wxDefaultPosition,
                              wxDefaultSize, wxTAB_TRAVERSAL);
    btnCancel->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs112->Add (btnCancel, 0, wxALL, 5);
    btnConnect = new wxButton (this, wxID_ANY, _("Connect"), wxDefaultPosition,
                               wxDefaultSize, wxTAB_TRAVERSAL);
    btnConnect->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs112->Add (btnConnect, 0, wxALL, 5);
    fgSizer11->Add (fgs112, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL, 5);
    bs1->Add (fgSizer11, 0, wxEXPAND, 5);
    this->SetSizer (bs1);
    this->Layout ();
    bs1->Fit (this);
    btnConnect->SetDefault ();
    btnConnect->SetFocus ();

    // populate dropdown
    wxDir ldir (wxGetCwd ());
    if (ldir.IsOpened ())
    {
        wxString filename;
        bool cont = ldir.GetFirst (&filename, wxT ("*.ppf"), wxDIR_DEFAULT);
        lstProfiles->Append (CURRENT_PROFILE);
        int i, cur=0;
        for (i=0; cont; i++)
        {
            filename = filename.Left (filename.Len ()-4);
            lstProfiles->Append (filename);
            cont = ldir.GetNext (&filename);
        }
        for (i=0; i< (int)lstProfiles->GetCount (); i++)
        {
            filename = lstProfiles->GetString (i);
            if (filename.CmpNoCase (ptermApp->m_curProfile) == 0)
                cur = i;
        }
        lstProfiles->Select (cur);
    }

    // initialize values
    m_ShellFirst = ptermApp->m_ShellFirst;
    m_host = ptermApp->m_hostName;
    m_port.Printf (wxT ("%ld"), ptermApp->m_port);

    // set object value properties
    txtShellFirst->SetValue (m_ShellFirst);
    txtHost->SetValue (m_host);
    cboPort->SetValue (m_port);

}

void PtermConnDialog::OnButton (wxCommandEvent& event)
{
    void OnButton (wxCommandEvent& event);
    if (event.GetEventObject () == btnCancel)
        EndModal (wxID_CANCEL);
    if (event.GetEventObject () == btnConnect)
    {
        m_curProfile = lstProfiles->GetStringSelection ();
        m_ShellFirst = txtShellFirst->GetLineText (0);
        m_host = txtHost->GetLineText (0);
        m_port = cboPort->GetValue ();
        EndModal (wxID_OK);
    }
}

void PtermConnDialog::OnSelect (wxCommandEvent& event)
{
    void OnSelect (wxCommandEvent& event);
    wxString str;
    wxString profile;
    wxString filename;
    if (event.GetEventObject () == lstProfiles)
    {
        profile = lstProfiles->GetStringSelection ();
        if (profile.Cmp (CURRENT_PROFILE)==0)
            ;
        else if (ptermApp->LoadProfile (profile, wxT ("")))
        {
            m_curProfile = profile;
            m_ShellFirst = ptermApp->m_ShellFirst;
            m_host = ptermApp->m_hostName;
            m_port.Printf (wxT ("%ld"), ptermApp->m_port);
            txtShellFirst->SetValue (m_ShellFirst);
            txtHost->SetValue (m_host);
            cboPort->SetValue (m_port);
        }
        else
        {
            filename = ptermApp->ProfileFileName (profile);
            str.Printf (wxT ("Profile '%s' not found. Missing file:\n\n"),
                        profile);
            str.Append (filename);
            wxMessageBox (str, wxT ("Error"), wxOK | wxICON_HAND);
        }
    }
}

void PtermConnDialog::OnChange (wxCommandEvent& event)
{
    void OnChange (wxCommandEvent& event);
    if (event.GetEventObject () == txtShellFirst)
        m_ShellFirst = txtShellFirst->GetLineText (0);
}

void PtermConnDialog::OnDoubleClick (wxCommandEvent& event)
{
    void OnDoubleClick (wxCommandEvent& event);
    wxString str;
    wxString profile;
    wxString filename;
    if (event.GetEventObject () == lstProfiles)
    {
        profile = lstProfiles->GetStringSelection ();
        if (ptermApp->LoadProfile (profile, wxT ("")))
        {
            m_curProfile = profile;
            m_ShellFirst = ptermApp->m_ShellFirst;
            m_host = ptermApp->m_hostName;
            m_port.Printf (wxT ("%ld"), ptermApp->m_port);
            txtShellFirst->SetValue (m_ShellFirst);
            txtHost->SetValue (m_host);
            cboPort->SetValue (m_port);
            m_host = txtHost->GetLineText (0);
            m_port = cboPort->GetValue ();
            EndModal (wxID_OK);
        }
        else
        {
            filename = ptermApp->ProfileFileName (profile);
            str.Printf (wxT ("Profile '%s' not found. Missing file:\n\n"),
                        profile);
            str.Append (filename);
            wxMessageBox (str, wxT ("Error"), wxOK | wxICON_HAND);
        }
    }
}

// ----------------------------------------------------------------------------
// PtermConnFailDialog
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE (PtermConnFailDialog, wxDialog)
    EVT_CLOSE (PtermConnFailDialog::OnClose)
    EVT_BUTTON (wxID_ANY, PtermConnFailDialog::OnButton)
    END_EVENT_TABLE ();

PtermConnFailDialog::PtermConnFailDialog (wxWindowID id, const wxString &title,
                                          wxPoint pos, wxSize loc)
    : wxDialog (NULL, id, title, pos, loc)
{

    // static ui objects, note dynamic controls, e.g. those that hold values
    // or require event processing are declared above
    wxStaticText* lblPrompt;
    wxStaticText* lblHost;
//  wxButton* btnNew;
//  wxButton* btnRetry;
//  wxButton* btnCancel;
    wxString str;

    // ui object creation / placement, note initialization of values is below
    this->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    wxBoxSizer* bs1;
    bs1 = new wxBoxSizer (wxVERTICAL);
    lblPrompt = new wxStaticText (this, wxID_ANY, wxT (""), wxDefaultPosition,
                                  wxDefaultSize, 0);
    lblPrompt->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    bs1->Add (lblPrompt, 1,
              wxTOP|wxRIGHT|wxLEFT|wxALIGN_CENTER_VERTICAL|wxEXPAND, 5);
    lblHost = new wxStaticText (this, wxID_ANY, wxT (""), wxDefaultPosition,
                                wxDefaultSize, 0);
    lblHost->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    bs1->Add (lblHost, 0, wxALL, 5);
    wxFlexGridSizer* fgs11;
    fgs11 = new wxFlexGridSizer (0, 4, 0, 0);
    fgs11->AddGrowableCol (2);
    fgs11->SetFlexibleDirection (wxHORIZONTAL);
    btnNew = new wxButton (this, wxID_ANY, _("New Connection"),
                           wxDefaultPosition, wxDefaultSize, 0|wxTAB_TRAVERSAL);
    btnNew->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs11->Add (btnNew, 0, wxALL, 5);
    btnRetry = new wxButton (this, wxID_ANY, _("Retry"), wxDefaultPosition,
                             wxDefaultSize, 0|wxTAB_TRAVERSAL);
    btnRetry->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs11->Add (btnRetry, 0, wxALL, 5);
    fgs11->Add (0, 0, 1, wxALL, 5);
    btnCancel = new wxButton (this, wxID_CANCEL, _("Cancel"),
                              wxDefaultPosition, wxDefaultSize,
                              0|wxTAB_TRAVERSAL);
    btnCancel->SetFont (wxFont (10, 74, 90, 90, false, wxT (SSFONT)));
    fgs11->Add (btnCancel, 0, wxALL, 5);
    bs1->Add (fgs11, 0, wxEXPAND, 5);

    // set object value properties
    if (ptermApp->m_connError == C_DISCONNECT)
        str.Printf (_("Your connection to the host was lost.  You may open a\nnew connection, try this connection again, or exit."));
    else
        str.Printf (_("The host did not respond to the connection request.\nYou may open a new connection, try this connection\nagain, or exit."));
    lblPrompt->SetLabel (str);
    str.Printf (_("\nFailed: %s:%ld"), ptermApp->m_hostName,
                ptermApp->m_port);
    lblHost->SetLabel (str);

    //size the controls
    this->SetSizer (bs1);
    this->Layout ();
    bs1->Fit (this);
    btnRetry->SetDefault ();
    btnRetry->SetFocus ();
}

void PtermConnFailDialog::OnButton (wxCommandEvent& event)
{
    void OnButton (wxCommandEvent& event);
    if (event.GetEventObject () == btnNew)
        ptermApp->m_connAction = 1;
    else if (event.GetEventObject () == btnRetry)
        ptermApp->m_connAction = 2;
    else if (event.GetEventObject () == btnCancel)
        ptermApp->m_connAction = 0;
    EndModal (wxID_OK);
}

void PtermConnFailDialog::OnClose (wxCloseEvent& event)
{
    void OnClose (wxCloseEvent& event);
    ptermApp->m_connAction = 0;
    EndModal (wxID_OK);
}

// ----------------------------------------------------------------------------
// PtermConnection
// ----------------------------------------------------------------------------

PtermConnection::PtermConnection (PtermFrame *owner, wxString &host, int port)
    : m_displayIn (0),
      m_displayOut (0),
      m_gswIn (0),
      m_gswOut (0),
      m_owner (owner),
      m_port (port),
      m_gswActive (false),
      m_gswStarted (false),
      m_savedGswMode (0),
      m_gswWord2 (0),
      m_connMode (both),
      m_pending (0),
      m_connActive (false)
{
    struct hostent *hp;
    in_addr_t hostaddr;
    int i, addrcount, r, conntries;
    in_addr_t *addresses = NULL;

    m_hostName = host;

    m_portset.callBack = s_connCallback;
    m_portset.dataCallBack = s_dataCallback;
    m_portset.callArg = m_portset.dataCallArg = this;
    m_portset.portNum = 0;      // No listening
    m_portset.maxPorts = 1;
    dtInitPortset (&m_portset, BufSiz, 0);
    m_fet = m_portset.portVec;
    
    hp = gethostbyname (m_hostName.mb_str ());
    if (hp == NULL || hp->h_length == 0)
    {
        StoreWord (C_CONNFAIL);
        wxWakeUpIdle ();
        return;
    }

    for (addrcount = 0; hp->h_addr_list[addrcount] != NULL; addrcount++) ;
    addresses = new in_addr_t[addrcount];
    for (i = 0; i < addrcount; i++)
    {
        memcpy (&addresses[i], hp->h_addr_list[i], sizeof (int));
    }
    for (conntries = 0; conntries < addrcount; conntries++)
    {
        while (addresses[(r = (rand () >> 10) % addrcount)] == 0) ;
        hostaddr = addresses[r];
        m_hostAddr = ntohl (hostaddr);
        addresses[r] = 0;
        StoreWord (C_CONNECTING);
        wxWakeUpIdle ();
        if (dtConnect (m_fet, &m_portset, hostaddr, m_port) >= 0)
        {
            break;
        }
    }
    if (conntries == addrcount)
    {
        // We ran out of addresses
        StoreWord (C_CONNFAIL);
    }
}

PtermConnection::~PtermConnection ()
{
    if (m_gswActive)
    {
        ptermCloseGsw ();
        m_owner->m_gswFile = wxString ();
    }
    dtClose (m_fet, &m_portset, TRUE);
}

void PtermConnection::s_connCallback (NetFet *, int, void *arg)
{
    PtermConnection *self = (PtermConnection *) arg;
    
    self->connCallback ();
}

void PtermConnection::s_dataCallback (NetFet *, int, void *arg)
{
    PtermConnection *self = (PtermConnection *) arg;
    
    self->dataCallback ();
}

void PtermConnection::connCallback (void)
{
    if (m_fet->connFd == 0)
    {
        // lost connection
        m_savedGswMode = m_gswWord2 = 0;
        if (m_gswActive)
        {
            m_gswActive = m_gswStarted = false;
            ptermCloseGsw ();
            m_owner->m_gswFile = wxString ();
        }
        if (m_connActive)
        {
            StoreWord (C_DISCONNECT);
        }
    }
    wxWakeUpIdle ();
}

void PtermConnection::dataCallback (void)
{
    u32 platowd = 0;
    int i;

    // We received something so the connection is now active
    if (!m_connActive)
    {
        m_connActive = true;
        StoreWord (C_CONNECTED);
    }

    for (;;)
    {
        /*
        **  Assemble words from the network buffer, all the
        **  while looking for "abort output" codes (word == 2).
        */
        if (IsFull ())
        {
            printf ("ring is full\n");
            break;
        }

        switch (m_connMode)
        {
        case niu:
            platowd = AssembleNiuWord ();
            break;
        case ascii:
            platowd = AssembleAsciiWord ();
            break;
        case both:
            platowd = AssembleAutoWord ();
            break;
        }
            
        if (platowd == (u32)C_NODATA)   // makes me nervous -- bg
        {
            break;
        }
        else if (m_connMode == niu && platowd == 2)
        {
            m_savedGswMode = m_gswWord2 = 0;
            if (m_gswActive)
            {
                m_gswActive = m_gswStarted = false;
                ptermCloseGsw ();
                m_owner->m_gswFile = wxString ();
                m_owner->ptermShowTrace ();
            }
                
            // erase abort marker -- reset the ring to be empty
            wxCriticalSectionLocker lock (m_pointerLock);

            m_displayOut = m_displayIn;
        }

        StoreWord (platowd);
        i = RingCount ();
        debug ("Stored %07o, ring count is %d", platowd, i);


        if (m_gswActive && !m_gswStarted && i >= GSWRINGSIZE / 2)
        {
            ptermStartGsw ();
            m_gswStarted = true;
        }
            
        if (i == RINGXOFF1 || i == RINGXOFF2)
        {
            m_owner->ptermSendKey1 (xofkey);
        }
    }
    if (!IsEmpty ())
    {
        // Send a do-nothing event to the frame; that will wake up
        // the main thread and cause it to process the words we 
        // buffered.
        wxWakeUpIdle ();
    }
}

int PtermConnection::AssembleNiuWord (void)
{
    int i, j, k;
    
    for (;;)
    {
        if (dtFetData (m_fet) < 3)
        {
            return C_NODATA;
        }
        i = dtReado (m_fet);
        if (i & 0200)
        {
            printf ("Plato output out of sync byte 0: %03o\n", i);
            continue;
        }
newj:
        j = dtReado (m_fet);
        if ((j & 0300) != 0200)
        {
            printf ("Plato output out of sync byte 1: %03o\n", j);
            if ((j & 0200) == 0)
            {
                i = j;
                goto newj;
            }
            continue;
        }
        k = dtReado (m_fet);
        if ((k & 0300) != 0300)
        {
            printf ("Plato output out of sync byte 2: %03o\n", k);
            if ((k & 0200) == 0)
            {
                i = k;
                goto newj;
            }
            continue;
        }
        return (i << 12) | ((j & 077) << 6) | (k & 077);
    }
}

int PtermConnection::AssembleAutoWord (void)
{
    u8 buf[3];
    
    if (dtPeekw (m_fet, buf, 3) < 0)
    {
        return C_NODATA;
    }
    if ((buf[0] & 0200) == 0 &&
        (buf[1] & 0300) == 0200 &&
        (buf[2] & 0300) == 0300)
    {
        m_connMode = niu;
        m_owner->menuFile->Enable (Pterm_SaveAudio, true);
        return AssembleNiuWord ();
    }
    else
    {
        m_connMode = ascii;
        return AssembleAsciiWord ();
    }
}

int PtermConnection::AssembleAsciiWord (void)
{
    int i;
    
    for (;;)
    {
        i = dtReado (m_fet);
        if (i == -1)
        {
            return C_NODATA;
        }
        else if (m_pending == 0 && i == 0377)
        {
            // 0377 is used by Telnet to introduce commands (IAC).
            // We recognize only IAC IAC for now.
            // Note that the check has to be made before the sign
            // bit is stripped off.
            m_pending = 0377;
            continue;
        }

        i &= 0177;
        if (i == 033)
        {
            m_pending = 033;
            continue;
        }
        if (m_pending == 033)
        {
            m_pending = 0;
            return (033 << 8) + i;
        }
        else
        {
            m_pending = 0;
            if (i == 0)
            {
                // NUL is for -delay-
                i = 1 << 19;
            }
            return i;
        }
    }
}


int PtermConnection::NextRingWord (void)
{
    int word, next, i;

    {
        wxCriticalSectionLocker lock (m_pointerLock);
        
        if (m_displayIn == m_displayOut)
        {
            return C_NODATA;
        }
    
        i = RingCount ();
        word = m_displayRing[m_displayOut];
        next = m_displayOut + 1;
        if (next == RINGSIZE)
        {
            next = 0;
        }
        m_displayOut = next;
    }
    debug ("consumed word %07o, ring count now %d", word, i);
    if (i < RINGXOFF1 && m_owner->m_pendingEcho != -1)
    {
        m_owner->ptermSendKey1 (m_owner->m_pendingEcho);
        m_owner->m_pendingEcho = -1;
    }
    if (i == RINGXON1 || i == RINGXON2)
    {
        m_owner->ptermSendKey1 (xonkey);
    }

    return word;
}

int PtermConnection::NextWord (void)
{
    int next, word;
    int delay = 0;
    wxString msg;

    if (m_gswActive)
    {
        // Take data from the ring of words that have just been given
        // to the GSW emulation.  Note that the delay amount has already
        // been set in those entries.  Usually it is 1, but it may be 0
        // if the display is falling behind.
        if (m_gswIn == m_gswOut)
        {
            return C_NODATA;
        }
        
        word = m_gswRing[m_gswOut];
        next = m_gswOut + 1;
        if (next == GSWRINGSIZE)
        {
            next = 0;
        }
        m_gswOut = next;

        if (word == C_GSWEND)
        {
            m_gswActive = m_gswStarted = false;
            ptermCloseGsw ();
            m_owner->m_gswFile = wxString ();
            m_owner->ptermShowTrace ();
        }
        else
        {
            return word;
        }
    }

    // Take data from the main input ring
    word = NextRingWord ();

    if (!Ascii () && 
        (word >> 16) == 3 &&
        word != 0700001 &&
        !(m_owner->m_station == 1 && 
          (word == 0770000 || word == 0730000)) &&
        ptermApp->m_gswEnable)
    {
        // It's an -extout- word, which means we'll want to start up
        // GSW emulation, if enabled.
        // However, we'll see these also when PLATO is turning OFF
        // the GSW (common entry code in the various gsw lessons).
        // We don't want to grab the GSW subsystem in that case.
        // The "turn off" sequence consists of a mode word followed
        // by voice words that specify "rest" (operand == 1).
        // The sound is silenced when the GSW is not active, so we'll
        // ignore "rest" voice words in that case.  We'll save the last
        // voice word, because it sets number of voices and volumes.
        // We have to do that because when the music actually starts,
        // we'll first see a mode word and then some non-rest voice
        // words.  The non-rest voice words trigger the GSW startup,
        // so we'll need to send the preceding mode word for correct
        // initialization.
        // Also, if we're connected to station 0-1, ignore the
        // operator box command words.
        if ((word >> 15) == 6)
        {
            // mode word, just save it
            m_savedGswMode = word;
        }
        else if (ptermOpenGsw (this, m_owner->m_gswFile, 
                               m_owner->m_gswFFmt) == 0)
        {
            m_gswActive = true;
            m_gswWord2 = word;
            m_owner->ptermShowTrace ();
                
            if (!m_gswStarted && RingCount () >= GSWRINGSIZE / 2)
            {
                ptermStartGsw ();
                m_gswStarted = true;
            }
        }
    }
            
    // See if emulating 1260 baud, or the -delay- NOP code
    if (ptermApp->m_classicSpeed ||
        word == 1)
    {
        delay = 1;
    }
    
    // Pass the delay to the caller
    word |= (delay << 19);

    // The processing for the Connecting and Connected message
    // formatting is here because setting the statusbar out of
    // connection thread context doesn't work.  The usual story,
    // GUI actions belong in the GUI (main) thread.
    if (word == C_CONNECTING)
    {
        msg.Printf (_("Connecting to %d.%d.%d.%d"), 
                    (m_hostAddr >> 24) & 0xff,
                    (m_hostAddr >> 16) & 0xff,
                    (m_hostAddr >>  8) & 0xff,
                    m_hostAddr & 0xff);
        m_owner->ptermSetStatus (msg);
    }
    else if (word == C_CONNECTED)
    {
        msg.Printf (_("Connected to %d.%d.%d.%d"), 
                    (m_hostAddr >> 24) & 0xff,
                    (m_hostAddr >> 16) & 0xff,
                    (m_hostAddr >>  8) & 0xff,
                    m_hostAddr & 0xff);
        m_owner->ptermSetStatus (msg);
    }
    else if (word == C_CONNFAIL || word == C_DISCONNECT)
    {
        if (m_owner->m_statusBar != NULL)
        {
            m_owner->m_statusBar->SetStatusText (_(" Not connected"),
                                                 STATUS_CONN);
        }

        wxDateTime ldt;
        ldt.SetToCurrent ();
        if (word == C_CONNFAIL)
            msg.Printf (_("Connection failed @ %s on "),
                        ldt.FormatTime ());
        else
            msg.Printf (_("Dropped connection @ %s on "),
                        ldt.FormatTime ());
        msg.Append (ldt.FormatDate ());
        m_owner->WriteTraceMessage (msg);
        msg.Printf (_("%s on "), ldt.FormatTime ());
        msg.Append (ldt.FormatDate ());   // fits in dialog box title
        
        m_owner->Iconize (false);    // ensure window is visible when connection fails
        ptermApp->m_connError = word;
        PtermConnFailDialog dlg (wxID_ANY, msg, wxDefaultPosition,
                                 wxSize (320, 140));
        dlg.CenterOnScreen ();
        dlg.ShowModal ();

        switch (ptermApp->m_connAction)
        {
        case 1:         // prompt for a connection rather than just bailing out
            ptermApp->DoConnect (true);
            word = C_NODATA;
            break;
        case 2:         // stay in pterm in disconnected state
            if (word == C_DISCONNECT)
                word = C_NODATA;
            else if (m_owner->m_statusBar != NULL)
            {
                m_owner->m_statusBar->SetStatusText (_(" Retrying..."),
                                                     STATUS_CONN);
            }
            break;
        default:        // cancel exits
            m_owner->Close (true);
            word = C_NODATA;
        }
    }
        
    return word;
}

// Get a word from the main data ring for the GSW emulation.  The
// "idle" flag is true if the GSW has seen several consecutive periods
// (adding up to about a second of play) in which there was no data
// for it to process.  We treat that as end of song, something that is
// not explicitly encoded in the GSW data stream.
//
// When a word is given to the GSW, it is stored in the "gsw ring" which
// holds words to be processed by the terminal emulation display machinery
// whenever GSW emulation is active.  That's a small ring, because entries
// are added to it at 60 per second and they are processed promptly.
// This approach means the GSW is doing the display pacing, rather than
// some other timer.  That ties the display directly to the audio stream,
// which we want so things like "watch music display while it is playing"
// work right.

int PtermConnection::NextGswWord (bool idle)
{
    int next, word;

    if (m_savedGswMode != 0)
    {
        word = m_savedGswMode;
        m_savedGswMode = 0;
    }
    else if (m_gswWord2 != 0)
    {
        word = m_gswWord2;
        m_gswWord2 = 0;
    }
    else
    {
        next = m_gswOut;
        next = m_gswIn + 1;
        if (next == GSWRINGSIZE)
        {
            next = 0;
        }
        // If there is no room to save this data for the display,
        // tell the sound that there isn't any more data.
        if (next == m_gswOut)
        {
            return C_NODATA;
        }
        word = NextRingWord ();
        if (word == C_NODATA && idle)
        {
            word = C_GSWEND;
        }
        
        m_gswRing[m_gswIn] = word;
        m_gswIn = next;
        wxWakeUpIdle ();
    }
    
    return word;
}

int ptermNextGswWord (void *connection, int idle)
{
    return ((PtermConnection *) connection)->NextGswWord (idle != 0);
}

void PtermConnection::StoreWord (int word)
{
    int next;
    
    if (word < 0)
    {
        m_displayOut = m_displayIn;
    }
    next = m_displayIn + 1;
    if (next == RINGSIZE)
    {
        next = 0;
    }
    if (next == m_displayOut)
    {
        return;
    }
    m_displayRing[m_displayIn] = word;
    m_displayIn = next;
    
//    debug ("data from plato %07o", word);
}

void PtermConnection::SendData (const void *data, int len)
{
    // Windows has the wrong type for the buffer pointer argument...
    send (m_fet->connFd, (const char *) data, len, 0);
}

// ----------------------------------------------------------------------------
// PtermCanvas
// ----------------------------------------------------------------------------

// the event tables connect the wxWindows events with the functions (event
// handlers) which process them.
BEGIN_EVENT_TABLE (PtermCanvas, wxScrolledCanvas)
    EVT_CHAR (PtermCanvas::OnChar)
    EVT_CHAR_HOOK (PtermCanvas::OnCharHook)
    EVT_LEFT_DOWN (PtermCanvas::OnMouseDown)
    EVT_LEFT_UP (PtermCanvas::OnMouseUp)
    EVT_RIGHT_UP (PtermCanvas::OnMouseContextMenu)
    EVT_MOTION (PtermCanvas::OnMouseMotion)
    EVT_MOUSEWHEEL (PtermCanvas::OnMouseWheel)
    END_EVENT_TABLE ();

PtermCanvas::PtermCanvas (PtermFrame *parent)
    : wxScrolledCanvas (parent, -1, wxDefaultPosition, 
                        wxDefaultSize,
                        wxHSCROLL | wxVSCROLL | wxFULL_REPAINT_ON_RESIZE |
                        wxWANTS_CHARS
                        /* , "Default Name" */),
      m_owner (parent),
      m_touchEnabled (false)
{
    wxClientDC dc (this);

    DisableKeyboardScrolling ();
    SetBackgroundColour (ptermApp->m_bgColor);
}

// Convert device x/y (on the scrolled canvas) to PLATO x/y.  This
// takes into account the current scroll position as well as scaling.
void PtermCanvas::Unadjust (int x, int y, int *xx, int *yy) const
{
    int xu, yu;

    CalcUnscrolledPosition (x, y, &xu, &yu);
    debug ("scrolled %d, %d maps to %d %d", x, y, xu, yu);

    *xx = xu / m_owner->m_xscale - m_owner->m_xmargin;
    *yy = 511 - (yu / m_owner->m_yscale - m_owner->m_ymargin);
}

void PtermCanvas::OnDraw (wxDC &dc)
{
    const int rh = m_owner->m_regionHeight;
    const int rw = m_owner->m_regionWidth;

    dc.SetUserScale (m_owner->m_xscale, m_owner->m_yscale);
    dc.SetClippingRegion (m_owner->m_xmargin, m_owner->m_ymargin, 512, 512);
    dc.DrawBitmap (*m_owner->m_bitmap, m_owner->m_xmargin,
                   m_owner->m_ymargin, false);
    
    debug ("Drawing bitmap onto the window canvas");

    if (rh != 0 && rw != 0)
    {
        dc.SetClippingRegion (m_owner->m_xmargin + (8 * m_owner->m_regionX), 
                              m_owner->m_ymargin + 511 -
                              (16 * (m_owner->m_regionY + rh)), 
                              rw * 8, rh * 16);
        dc.DrawBitmap (*m_owner->m_selmap, m_owner->m_xmargin,
                       m_owner->m_ymargin, false);
        dc.DestroyClippingRegion ();
    
        debug ("Drawing selection region, top %d %d, size %d %d",
                m_owner->m_regionX, m_owner->m_regionY, rw, rh);
    }
}

void PtermCanvas::OnCharHook (wxKeyEvent &event)
{
    unsigned int key;
    int shift = 0;
    u32 pc = None;
    bool ctrl;

    // Most keyboard inputs are handled here, because if we defer them to
    // the EVT_CHAR stage, too many evil things happen in too many of the
    // platforms, all different...
    //
    // The one thing we do defer until EVT_CHAR is plain old characters, i.e.,
    // keystrokes that aren't function keys or other special keys, and
    // neither Ctrl nor Alt are active.

    debug ("keydown: ctrl %d shift %d alt %d keycode %d",
            event.RawControlDown (), event.ShiftDown (),
            event.AltDown (), event.GetKeyCode ());

    if (m_owner->m_bPasteActive)
    {
        // If pasting is active, this key is NOT passed on to the application
        // until the paste operation is properly canceled
        m_owner->m_bCancelPaste = true;
        return;
    }

    ctrl = event.RawControlDown ();
    if (event.ShiftDown ())
    {
        shift = 040;
    }
    key = event.GetKeyCode ();

    if (!m_owner->HasConnection () ||
        (m_owner->m_conn->Ascii () && m_owner->m_dumbTty) ||
        key == WXK_ALT ||
        key == WXK_SHIFT ||
        key == WXK_RAW_CONTROL)
    {
        // We don't take any action on the modifier key keydown events,
        // but we do want to make sure they are seen by the rest of
        // the system.
        // The same applies to keys sent to the help window (which has
        // no connection on which to send them).
        // And finally, it applies to ASCII when in dumb TTY mode, in
        // that case we just want to send ASCII keycodes straight.
        //
        // Note that wxWidgets V3 maps the Mac Command key to WXK_CONTROL,
        // "to improve compatibility with other systems".  Gah.  But
        // fortunately, WXK_RAW_CONTROL means the real control key.

        event.Skip ();
        return;
    }
    if (key < 0200 && isalpha (key))
    {
        key = tolower (key);
    }

#if 0
    tracex ("ctrl %d shift %d alt %d key %d\n",
            event.RawControlDown (), event.ShiftDown (),
            event.AltDown (), key);
#endif

    if (ctrl && key == ']')         // control-] : trace
    {
        m_owner->ptermSetTrace (!m_owner->tracePterm);
        return;
    }

    // Special case: ALT-left is assignment arrow
    if (event.AltDown () && key == WXK_LEFT)
    {
        m_owner->ptermSendKey1 (015 | shift);
        return;
    }
    // Special case:user has disabled Shift-Space, which means to treat
    // it as a space
    if (ptermApp->m_DisableShiftSpace && key == WXK_SPACE)
    {
        shift = 0;
    }

    if (key < sizeof (asciiToPlato) / sizeof (asciiToPlato[0]))
    {
        if (event.AltDown ())
        {
            if (altKeyToPlato[key] != -1)
            {
                m_owner->ptermSendKey1 (altKeyToPlato[key] | shift);
                return;
            }
            else
            {
                event.Skip ();
                return;
            }
        }
        else if (ctrl && key >= 040)
        {
            // Control key is active.  There are several possibilities:
            //
            // Control plus letter: look up the translate table entry
            //  for the matching control code.
            // Control plus non-letter: look up the translate table entry
            //  for that character, and set the "shift" bit.
            // Control plus control code or function key: don't pay attention
            //  to the control key being active, and don't do a table lookup
            //  on the keycode.  Instead, those are handled later, in a switch.

            if (isalpha (key))
            {
                // Control letter -- do a lookup for the matching control code.
                pc = asciiToPlato[key & 037];
            }
            else
            {
                // control but not a letter or ASCII control code -- 
                // translate to what a PLATO keyboard
                // would have on the shifted position for that key
                // but only if it's a simple key (not a sequence of
                // access and other stuff)
                if (shift != 0 && key == '=')
                {
                    key = '+';
                }
                pc = asciiToPlato[key];
                if (pc != KEY1 (pc))
                {
                    // If this entry is for more than one keycode, 
                    // ignore the keystroke.
                    pc = -1;
                }
                shift = 040;
            }

            if (pc != None)
            {
                m_owner->ptermSendKey (pc | shift);
                return;
            }
        }
    }

    // If we get down to this point, then the following is true:
    // 1. It wasn't an ALT-key entry.
    // 2. If Control was active, it was with a non-printable key (where we
    //    ignore the Control flag).
    // At this point, we're going to look for function keys.  If it isn't
    // one of those, then it was a regular printable character code, possibly
    // shifted.  Those are handled in the EVT_CHAR handler because the
    // unshifted to shifted translation is keyboard specific (at least for
    // non-letters) and we want to let the system deal with that.

    pc = None;
    if (ptermApp->m_platoKb)
    {
#if defined (_WIN32)
        // This is a workaround for a Windows keyboard mapping bug
        if (key == '+')
        {
            pc = 0133; // =
        }
#endif
    }
    else if (ptermApp->m_numpadArrows)
    {
        // Check the numeric keypad keys separately and turn them
        // into the 8-way arrow keys of the PLATO main keyboard.
        switch (key)
        {
        case WXK_NUMPAD7:
        case WXK_NUMPAD_HOME:
            pc = 0121;      // up left (q)
            break;
        case WXK_NUMPAD8:
        case WXK_NUMPAD_UP:
            pc = 0127;      // up arrow (w)
            break;
        case WXK_NUMPAD9:
        case WXK_NUMPAD_PAGEUP:
            pc = 0105;      // up right (e)
            break;
        case WXK_NUMPAD4:
        case WXK_NUMPAD_LEFT:
            pc = 0101;      // left arrow (a)
            break;
        case WXK_NUMPAD6:
        case WXK_NUMPAD_RIGHT:
            pc = 0104;      // right arrow (d)
            break;
        case WXK_NUMPAD1:
        case WXK_NUMPAD_END:
            pc = 0132;      // down left (z)
            break;
        case WXK_NUMPAD2:
        case WXK_NUMPAD_DOWN:
            pc = 0130;      // down arrow (x)
            break;
        case WXK_NUMPAD3:
        case WXK_NUMPAD_PAGEDOWN:
            pc = 0103;      // down right (c)
            break;
        }
        pc = KEY1 (pc);
    }
    
    if (pc == None)
    {
        switch (key)
        {
        case WXK_SPACE:
        case WXK_NUMPAD_SPACE:
            pc = 0100;      // space
            break;
        case WXK_BACK:
            pc = 023;       // erase
            break;
        case WXK_RETURN:
        case WXK_NUMPAD_ENTER:
            pc = 026;       // next
            break;
        case WXK_HOME:
        case WXK_NUMPAD_HOME:
        case WXK_F8:
            pc = 030;       // back
            break;
        case WXK_PAUSE:
        case WXK_F10:
            pc = 032;       // stop
            break;
        case WXK_TAB:
            pc = 014;       // tab
            break;
        case WXK_ESCAPE:
            pc = 015;       // assign
            break;
        case WXK_ADD:
        case WXK_NUMPAD_ADD:
            if (ctrl)
            {
                pc = 056;   // Sigma
            }
            else
            {
                pc = 016;   // +
            }
            break;
        case WXK_SUBTRACT:
        case WXK_NUMPAD_SUBTRACT:
            if (ctrl)
            {
                pc = 057;   // Delta
            }
            else
            {
                pc = 017;   // -
            }
            break;
        case WXK_MULTIPLY:
        case WXK_NUMPAD_MULTIPLY:
        case WXK_NUMPAD_DELETE: 
        case WXK_DELETE:
            pc = 012;       // multiply sign
            break;
        case WXK_DIVIDE:
        case WXK_NUMPAD_DIVIDE:
        case WXK_NUMPAD_INSERT: 
        case WXK_INSERT:
            pc = 013;       // divide sign
            break;
        case WXK_LEFT:
        case WXK_NUMPAD_LEFT:
            pc = 0101;      // left arrow (a)
            break;
        case WXK_RIGHT:
        case WXK_NUMPAD_RIGHT:
            pc = 0104;      // right arrow (d)
            break;
        case WXK_UP:
        case WXK_NUMPAD_UP:
            pc = 0127;      // up arrow (w)
            break;
        case WXK_DOWN:
        case WXK_NUMPAD_DOWN:
            pc = 0130;      // down arrow (x)
            break;
        case WXK_PAGEUP:
        case WXK_NUMPAD_PAGEUP:
            pc = 020;       // super
            break;
        case WXK_PAGEDOWN:
        case WXK_NUMPAD_PAGEDOWN:
            pc = 021;       // sub
            break;
        case WXK_F3:
            pc = 034;       // square
            break;
        case WXK_F2:
            pc = 022;       // ans
            break;
        case WXK_F1:
        case WXK_F11:
            pc = 033;       // copy
            break;
        case WXK_F9:
            pc = 031;       // data
            break;
        case WXK_F5:
            pc = 027;       // edit
            break;
        case WXK_F4:
            pc = 024;       // micro/font
            break;
        case WXK_F6:
            pc = 025;       // help
            break;
        case WXK_F7:
            pc = 035;       // lab
            break;
        case WXK_NUMPAD_DECIMAL:
            pc = 0136;      // .
            break;
        case WXK_NUMPAD0:
        case WXK_NUMPAD1:
        case WXK_NUMPAD2:
        case WXK_NUMPAD3:
        case WXK_NUMPAD4:
        case WXK_NUMPAD5:
        case WXK_NUMPAD6:
        case WXK_NUMPAD7:
        case WXK_NUMPAD8:
        case WXK_NUMPAD9:
            pc = key - WXK_NUMPAD0;
            if (ctrl)
            {
                shift = 040;
            }
            break;
        default:
            // If it's not a function key, we'll let the OnChar handler
            // deal with it if we are using a regular keyboard.
            // But if we're using a PLATO keyboard, we'll take the
            // (unshifted) keycode, translate it to the PLATO code, apply
            // shift if any, and send that.  This is because the 
            // shifted non-letters on the PLATO keyboard are generally
            // different from those on the PC/Mac keyboard.
            if (key >= sizeof (asciiToPlato) / sizeof (asciiToPlato[0]))
            {
                return;
            }
            pc = asciiToPlato[key];
            if (!ptermApp->m_platoKb || pc == None)
            {
                event.Skip ();
                return;
            }
            // On Linux (wx/GTK), there's a keyboard handling error:
            // shifted comma key produces an OnCharHook event with 
            // key = '<', which is wrong, it should be the unshifted 
            // code.  All other codes appear to be correct, and it also
            // works right on other platforms.  Weird...  Work around
            // this by forcing the keycode.
            if (key == '<')
            {
                pc = asciiToPlato[(u8) ','];
            }
        }
        pc = KEY1 (pc);
    }

    m_owner->ptermSendKey (pc | shift);
}

void PtermCanvas::OnChar (wxKeyEvent& event)
{
    unsigned int key;
    u32 pc = None;

    debug ("onchar: ctrl %d shift %d alt %d keycode %d",
            event.RawControlDown (), event.ShiftDown (),
            event.AltDown (), event.GetKeyCode ());

    // Dumb TTY input is handled here, always
    if (m_owner->HasConnection () &&
        m_owner->m_conn->Ascii () && m_owner->m_dumbTty)
    {
        key = event.GetKeyCode ();
        if (key == 023 || key == WXK_PAUSE || key == WXK_F10)
        {
            /* Ctrl-S or F10.  Presumably shift-stop, so turn it into NEXT.  */
            key = 015;
        }
        tracex ("dumb tty key to plato %03o", key);
        debug ("dumb tty key to plato %03o", key);

        m_owner->m_conn->SendData (&key, 1);
        return;
    }
    
    // control and alt codes shouldn't come here, they are handled in KEY_DOWN
    if (!m_owner->HasConnection () ||
        event.RawControlDown () || event.AltDown ())
    {
        event.Skip ();
        return;
    }
    key = event.GetKeyCode ();

    //see if we can ignore the caplock
    if (ptermApp->m_ignoreCapLock && isalpha (key) &&
        wxGetKeyState (WXK_CAPITAL))
    {
        if (wxGetKeyState (WXK_SHIFT))
        {
            key = toupper (key);
        }
        else
        {
            key = tolower (key);
        }
    }

    if (key < sizeof (asciiToPlato) / sizeof (asciiToPlato[0]))
    {
        pc = asciiToPlato[key];
        if (pc != None)
        {
            m_owner->ptermSendKey (pc);
            return;
        }
    }
    event.Skip ();
}

void PtermCanvas::OnMouseWheel (wxMouseEvent &event)
{
    int key = event.m_wheelRotation;
    m_owner->ptermSendExt ((key>0 ? ~1 : 1) & 0xff);
    event.Skip ();
}

void PtermCanvas::OnMouseDown (wxMouseEvent &event)
{
    Unadjust (event.m_x, event.m_y, &m_mouseX, &m_mouseY);
    m_owner->ClearRegion ();
    event.Skip ();
}

void PtermCanvas::OnMouseMotion (wxMouseEvent &event)
{
    int xx, yy;
    
    if (m_mouseX >= 0 && event.m_leftDown)
    {
        Unadjust (event.m_x, event.m_y, &xx, &yy);
        m_owner->UpdateRegion (xx, yy, m_mouseX, m_mouseY);
    }
    
    event.Skip ();
}

void PtermCanvas::OnMouseContextMenu (wxMouseEvent &event)
{
    m_owner->PopupMenu (m_owner->menuPopup);
}

void PtermCanvas::OnMouseUp (wxMouseEvent &event)
{
    int x, y;
    
    if (m_mouseX < 0)
    {
        event.Skip ();
        return;
    }

    Unadjust (event.m_x, event.m_y, &x, &y);

    m_owner->UpdateRegion (x, y, m_mouseX, m_mouseY);

    m_mouseX = -1;

    if (!m_touchEnabled)
    {
        return;
    }
    if (x < 0 || x > 511 ||
        y < 0 || y > 511)
    {
        return;
    }
    m_owner->ptermSendTouch (x, y);

}

void PtermCanvas::ptermTouchPanel (bool enable)
{
    m_touchEnabled = enable;
    if (enable)
    {
        SetCursor (wxCursor (wxCURSOR_HAND));
    }
    else
    {
        SetCursor (wxNullCursor);
    }
}

#if defined (__WXMSW__)
// Override the window proc to avoid F10 being handled  as a hotkey
WXLRESULT PtermCanvas::MSWWindowProc (WXUINT message, WXWPARAM wParam,
                                      WXLPARAM lParam)
{
    if (message == WM_SYSKEYDOWN && wParam == VK_F10)
    { 
        m_lastKeydownProcessed = HandleKeyDown ((WORD) wParam, lParam);
        return 0;
    }
    return wxScrolledCanvas::MSWWindowProc (message, wParam, lParam);
}
#endif

// ----------------------------------------------------------------------------
// Pterm printing helper class
// ----------------------------------------------------------------------------

bool PtermPrintout::OnPrintPage (int page)
{
    wxDC *dc = GetDC ();

    if (dc)
    {
        if (page == 1)
        {
            DrawPage (dc);
        }

        return true;
    }

    return false;
}

void PtermPrintout::GetPageInfo (int *minPage, int *maxPage,
                                 int *selPageFrom, int *selPageTo)
{
    *minPage = 1;
    *maxPage = 1;
    *selPageFrom = 1;
    *selPageTo = 1;
}

bool PtermPrintout::HasPage (int pageNum)
{
    return (pageNum == 1);
}

void PtermPrintout::DrawPage (wxDC *dc)
{
    int obr, obg, obb;
    // Get the size of the DC in pixels
    int w, h;
    dc->GetSize (&w, &h);

    // Calculate a suitable scaling factor.  "Suitable" means chosen so
    // that the page size corresponds to 600 screen pixels, so we get
    // about 45 pixels worth of margin all around.
    double scaleX = w / 600.0;
    double scaleY = h / 600.0;

    // Use x or y scaling factor, whichever fits on the DC
    double actualScale = wxMin (scaleX, scaleY);

    // Calculate the position on the DC for centring the graphic
    double posX = (w - 512. * actualScale) / 2.0;
    double posY = (h - 512. * actualScale) / 2.0;

    // Set the scale and origin
    dc->SetUserScale (actualScale, actualScale);
    dc->SetDeviceOrigin ((long) posX, (long) posY);

    // Re-color the image
    wxImage screenImage = m_owner->m_bitmap->ConvertToImage ();

    unsigned char *data = screenImage.GetData ();
    
    w = screenImage.GetWidth ();
    h = screenImage.GetHeight ();
    obr = ptermApp->m_bgColor.Red ();
    obg = ptermApp->m_bgColor.Green ();
    obb = ptermApp->m_bgColor.Blue ();

    for (int j = 0; j < h; j++)
    {
        for (int i = 0; i < w; i++)
        {
            if (data[0] == obr &&
                data[1] == obg &&
                data[2] == obb)
            {
                // Background, make it white
                data[0] = data[1] = data[2] = 255;
            }
            else
            {
                // Foreground, make it black
                data[0] = data[1] = data[2] = 0;
            }
            data += 3;
        }
    }

    wxBitmap printmap (screenImage);

    dc->DrawBitmap (printmap, 0, 0);
}

/*---------------------------  End Of File  ------------------------------*/
